//=============================================================================
// ESRT_ButtonWindow.js
// ----------------------------------------------------------------------------
// Copyright (c) 2017 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : http://mn-s.net/
// [Twitter]: https://twitter.com/white_esorat
// ----------------------------------------------------------------------------
//=============================================================================

/*:
 * @plugindesc Button Window Plugin
 * @author white_esorat
 *
 * @help 
 *
 * This plugin does not provide plugin commands.
 * 
 * This plugin is released under the MIT License.
 *
 */
/*:ja
 * @plugindesc ボタンウィンドウ定義プラグイン
 * @author 白
 * 
 * @help 
 *
 * このプラグインには、プラグインコマンドはありません。
 * 
 * This plugin is released under the MIT License.
 *
 * 参考：
 * ・トリアコンタン 様
 * 　・サウンドテストプラグイン - http://triacontane.blogspot.jp/
 *
 */
 
var Esrt = Esrt || {};
//=============================================================================
// Window_Button
//    タップ用ウィンドウを定義します。
//=============================================================================
function Window_Button() {
    this.initialize.apply(this, arguments);
}

function Window_ButtonSprites() {
    this.initialize.apply(this, arguments);
}

(function(){
    'use strict';
    
    Window_Button.prototype = Object.create(Window_Selectable.prototype);
    Window_Button.prototype.constructor = Window_Button;
    
    Window_Button.prototype.initialize = function(x, y, width, height, spriteFileName) {
        Window_Selectable.prototype.initialize.apply(this, arguments);
        
        this._canProcessTouchNonActive = true;
        this.anchor= {};
        this.anchor.x = 0;
        this.anchor.y = 0;
        this._onMouse    = false;
        this._outMouse   = false;
        this._wasOnMouse = false;
        
        this._sprite = new Sprite();
        this.addChild(this._sprite);
        this._sprite.x = 0;
        this._sprite.y = 0;
        this._sprite.visible = true;
        this.setSprite(spriteFileName);
    };

    var _Window_Button_update              = Window_Button.prototype.update;
    Window_Button.prototype.update = function() {
        _Window_Button_update.apply(this, arguments);
        
        this.updateMouseMove();
        this.focusReflection();
    };

    Window_Button.prototype.isTouchPosInRect = function() {
        var dx  = TouchInput.x - this.x;
        var dy  = TouchInput.y - this.y;
        var sin = Math.sin(-this.rotation);
        var cos = Math.cos(-this.rotation);
        var rx  = this.x + Math.floor(dx * cos + dy * -sin);
        var ry  = this.y + Math.floor(dx * sin + dy * cos);
        return (rx >= this.minX() && rx <= this.maxX() &&
        ry >= this.minY() && ry <= this.maxY());
    };

    Window_Button.prototype.screenWidth = function() {
        return (this.width || 0) * this.scale.x;
    };

    Window_Button.prototype.screenHeight = function() {
        return (this.height || 0) * this.scale.y;
    };

    Window_Button.prototype.screenX = function() {
        return (this.x || 0) - this.anchor.x * this.screenWidth();
    };

    Window_Button.prototype.screenY = function() {
        return (this.y || 0) - this.anchor.y * this.screenHeight();
    };

    Window_Button.prototype.minX = function() {
        return Math.min(this.screenX(), this.screenX() + this.screenWidth());
    };

    Window_Button.prototype.minY = function() {
        return Math.min(this.screenY(), this.screenY() + this.screenHeight());
    };

    Window_Button.prototype.maxX = function() {
        return Math.max(this.screenX(), this.screenX() + this.screenWidth());
    };

    Window_Button.prototype.maxY = function() {
        return Math.max(this.screenY(), this.screenY() + this.screenHeight());
    };

    Window_Button.prototype.updateMouseMove = function() {
        if ( this.isTouchPosInRect()) {
            if (!this._wasOnMouse) {
                this._onMouse    = true;
                this._wasOnMouse = true;
            }
        } else {
            if (this._wasOnMouse) {
                this._outMouse   = true;
                this._wasOnMouse = false;
            }
        }
    };
    
    Window_Button.prototype.focusReflection = function() { //マウスオーバーの反映
        if(this.isOnFocus()){
            this._sprite.bitmap = ImageManager.loadPictureButton(this._spriteFileName + '_active');
            this._onMouse = false;
        }
        if(this.isOutFocus()){
            this._sprite.bitmap = ImageManager.loadPictureButton(this._spriteFileName);
            this._outMouse = false;
        }
    };

    Window_Button.prototype.isOnFocus = function() {
        return this._onMouse;
    };

    Window_Button.prototype.isOutFocus = function() {
        return this._outMouse;
    };
    
    Window_Button.prototype.setSprite = function(spriteFileName) { // 時の表示画像の設定
        this._spriteFileName = spriteFileName;
        if(this._spriteFileName){
            this._sprite.bitmap = ImageManager.loadPictureButton(this._spriteFileName);
            this.opacity = 0;
        }
    };
    
    Window_Button.prototype.updateArrows = function() { // 不要な矢印を表示しない
        this.downArrowVisible = false;
        this.upArrowVisible = false;
    };
    //=============================================================================
    // Window_ButtonSprites
    //  タッチ可能なスプライトを保持するウィンドウです(トリアコンタン様のMessageWindowSkipを参考)
    //=============================================================================
    Window_ButtonSprites.prototype = Object.create(Window_Base.prototype);
    Window_ButtonSprites.prototype.constructor = Window_ButtonSprites;
    
    Window_ButtonSprites.prototype.initialize = function() {
        Window_Base.prototype.initialize.apply(this, arguments);
        
        this._hoverCheckCount = 0;
        this._hoverCheckSpritelist = [];
        this._touchCheckSpritelist = [];
    };
    
    Window_ButtonSprites.prototype.update = function() {
        Window_Base.prototype.update.apply(this, arguments);
        
        this.updateHover();
    };
    
    Window_ButtonSprites.prototype.isTriggeredButton = function(button) {
        return button &&
            button.isTriggered(this.canvasToLocalX(TouchInput.x), this.canvasToLocalY(TouchInput.y));
    };
    
    Window_ButtonSprites.prototype.isPressedButton = function(button) {
        return button &&
            button.isPressed(this.canvasToLocalX(TouchInput.x), this.canvasToLocalY(TouchInput.y));
    };
    
    Window_ButtonSprites.prototype.isHoveredButton = function(button) {
        return button &&
            button.isHovered(this.canvasToLocalX(TouchInput.x), this.canvasToLocalY(TouchInput.y));
    };

    Window_ButtonSprites.prototype.createSpriteButton = function(button, fileName, x, y) {
        this[button] = new Sprite_Button(fileName);
        this.addChild(this[button]);
        this[button].x = x;
        this[button].y = y;
        this[button].visible = true;
    };

    // スプライトのタッチ状態を取得
    // タッチされていたら対応するスイッチをtrueにして、以降のゲーム内タッチ処理をキャンセルする
    Window_ButtonSprites.prototype.updateTouch = function() {
        if(!this.visible) return;
        
        for(var i=0, length = this._touchCheckSpritelist.length; i<length; i++){
            var data = this._touchCheckSpritelist[i];
            if(this[data.button] && this.isTriggeredButton(this[data.button])){
                $gameSwitches.setValue(data.switch, true);
                TouchInput.suppressEvents();
                return;
            }
        }
    };

    Window_ButtonSprites.prototype.updateHover = function() {
        this._hoverCheckCount += 1;
        if(this._hoverCheckCount % 5 != 0) return;
        for(var i=0, length = this._hoverCheckSpritelist.length; i<length; i++){
            if(this[this._hoverCheckSpritelist[i]]){
                this[this._hoverCheckSpritelist[i]].setFocus(this.isHoveredButton(this[this._hoverCheckSpritelist[i]]));
            }
        }
    };
    
    //=============================================================================
    // Sprite_Button
    //  メッセージボタン描画用スプライトです。(トリアコンタン様のPictureCallCommonを参考)
    //=============================================================================
    function Sprite_Button() {
        this.initialize.apply(this, arguments);
    }

    Sprite_Button.prototype             = Object.create(Sprite.prototype);
    Sprite_Button.prototype.constructor = Sprite_Button;

    Sprite_Button.prototype.initialize = function(fileName) {
        Sprite.prototype.initialize.call(this);
        this.fileName = fileName;
        this.bitmap   = ImageManager.loadPictureButton(fileName);
        this.anchor.x = 0;
        this.anchor.y = 0;
        this.visible = false;
        this.valid   = true
        this.opacity = 0;
    };

    Sprite_Button.prototype.update = function() {
        Sprite.prototype.update.call(this);
        this.updateOpacity();
        this.updateVisibility();
    };

    Sprite_Button.prototype.updateOpacity = function() {
        if(!this.valid) return;
        
        this.opacity = this.parent.openness;
    };

    Sprite_Button.prototype.updateVisibility = function() {
        this.visible = true;
    };

    Sprite_Button.prototype.isTriggered = function(targetX, targetY) {
        if(!this.valid) return false;
        
        var realX = targetX + this._frame.width * this.anchor.x;
        var realY = targetY + this._frame.height * this.anchor.y;
        if (TouchInput.isTriggered() && this.isInSprite(realX, realY)) {
            return true;
        }
        return false;
    };

    Sprite_Button.prototype.isPressed = function(targetX, targetY) {
        if(!this.valid) return false;
        
        var realX = targetX + this._frame.width * this.anchor.x;
        var realY = targetY + this._frame.height * this.anchor.y;
        if (TouchInput.isPressed() && this.isInSprite(realX, realY)) {
            return true;
        }
        return false;
    };

    Sprite_Button.prototype.isHovered = function(targetX, targetY) {
        if(!this.valid) return false;
        
        var realX = targetX + this._frame.width * this.anchor.x;
        var realY = targetY + this._frame.height * this.anchor.y;
        if (this.isInSprite(realX, realY)) {
            return true;
        }
        return false;
    };

    Sprite_Button.prototype.setFocus = function(flag) {
        if(!this.valid) return;
        
        var fileName = this.fileName;
        fileName += flag ? "_active" : "";
        
        this.bitmap   = ImageManager.loadPictureButton(fileName);
    };

    Sprite_Button.prototype.setValid = function(flag) {
        var fileName = this.fileName;
        fileName += flag ? "" : "_no";
        this.valid   = flag ? true : false;
        this.opacity = flag ? 255 : 127;
        
        this.bitmap   = ImageManager.loadPictureButton(fileName);
    };

    Sprite_Button.prototype.isInSprite = function(targetX, targetY) {
        return this.x <= targetX && this.x + this._frame.width >= targetX &&
            this.y <= targetY && this.y + this._frame.height >= targetY;
    };
    
    //=============================================================================
    // Window_Selectable
    //  ・タップ時にアクティブ化処理
    //  ・シフト押下時にシフトハンドラー追加処理
    //    ・トリアコンタン様のサウンドテストプラグインを参考にしました。
    //=============================================================================
    var _Window_Selectable_initialize      = Window_Selectable.prototype.initialize;
    Window_Selectable.prototype.initialize = function() {
        _Window_Selectable_initialize.apply(this, arguments);
    };
    
    var _Window_Selectable_processTouch      = Window_Selectable.prototype.processTouch;
    Window_Selectable.prototype.processTouch = function() {
        _Window_Selectable_processTouch.apply(this, arguments);
        
        if (this._canProcessTouchNonActive) {
            this.processTouchNonActive();
        }
    };

    Window_Selectable.prototype.processTouchNonActive = function() {
        if (this.isOpen() && !this.active && this.isTouchedInsideFrame() &&
            this.isHandled('touchInside') && TouchInput.isTriggered()) {
            this.callHandler('touchInside');
        }
    };

    Window_Selectable.prototype.processShift = function() {
        this.callHandler('shift');
    };

    var _Window_Selectable_processHandling      = Window_Selectable.prototype.processHandling;
    Window_Selectable.prototype.processHandling = function() {
        _Window_Selectable_processHandling.apply(this, arguments);
        if (this.isOpenAndActive()) {
            if (this.isShiftEnabled() && this.isShiftTriggered()) {
                this.processShift();
            }
        }
    };

    Window_Selectable.prototype.isShiftEnabled = function() {
        return this.isHandled('shift');
    };

    Window_Selectable.prototype.isShiftTriggered = function() {
        return Input.isRepeated('shift');
    };
}());
