//=============================================================================
// ESRT_EventHistory_override.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc 
 * @author white_esorat
 * 
 * @help 
 * プラグインコマンド詳細
 *  ESRT_EventHistoryにおいて汎用化できない『かなちゃんは魔法少女』独自の処理を記述する
 *
 * setEvent     [分類] [相手（イベント名）] [行為]
 *
 */

/*:ja
 * @plugindesc イベントの実行履歴を記録する
 * @author 白
 * 
 * @help 
 * ESRT_EventHistoryにおいて汎用化できない『かなちゃんは魔法少女』独自の処理を記述する
 *
 * setEvent     [分類] [相手（イベント名）] [行為]
 *
 */
 
var Esrt = Esrt || {};
Esrt.eventHistory   = Esrt.eventHistory || [];

Esrt.WindowMove.eventHistory = Esrt.WindowMove.eventHistory || {};
Esrt.WindowMove.eventHistory.isMoveIn = true;
Esrt.WindowMove.eventHistory.windowMoveDuration = 20;
Esrt.WindowMove.eventHistory.windowMoveCount = 0;
Esrt.WindowMove.eventHistory.windowMoveType = 1;
Esrt.WindowMove.eventHistory.windowMoveRate = 0;

(function() {
    'use strict';
    var pluginName    = 'ESRT_EventHistory_override';
    
    if (typeof Scene_EventHistory != "function") {
        console.error("ESRT_EventHistoryが先に読み込まれていません");
        return;
    }
    
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var menuShowSwitch     = 211;
    var menuAllowSwitch    = menuShowSwitch + 6;
    
    //=============================================================================
    // Scene_Map
    //    ウィンドウ移動情報を初期化します
    //=============================================================================
    var _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this, arguments);
        
        Esrt.WindowMove.eventHistory.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.eventHistory.isMoveIn;
        Esrt.WindowMove.eventHistory.windowMoveCount = (Esrt.WindowMove.eventHistory.isMoveIn) ? 0 : Esrt.WindowMove.eventHistory.windowMoveDuration;
    };

    //=============================================================================
    // Scene_EventHistory
    //  イベント履歴表示画面を扱うクラス
    //=============================================================================
    var _Scene_EventHistory_initialize = Scene_EventHistory.prototype.initialize;
    Scene_EventHistory.prototype.initialize = function() {
        // ウィンドウ移動演出情報の追加
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.eventHistory;
        this._windowMoveObject.windowMoveType = 1;
        this._windowMoveObject.windowMoveRate = 0;
        if (this._windowMoveObject.windowMoveCount > 0 && this._windowMoveObject.windowMoveCount < this._windowMoveObject.windowMoveDuration) {
            this._windowMoveObject.windowMoveCount = this._windowMoveObject.windowMoveDuration;
            this._windowMoveObject.windowMoveType = 0;
            this._windowMoveObject.isMoveIn = false;
        }
        
        _Scene_EventHistory_initialize.apply(this, arguments);
    };
    
    var _Scene_EventHistory_create = Scene_EventHistory.prototype.create;
    Scene_EventHistory.prototype.create = function() {
        _Scene_EventHistory_create.call(this);
        
        this.createButtonWindows();
    };
    
    Scene_EventHistory.prototype.createButtonWindows = function() {
        this._backButtonWindow = new Window_Button(0, 550, 190, 70, 'button_back_left');
        this._backButtonWindow.setHandler('touchInside', this.commandBack.bind(this));
        this.addWindow(this._backButtonWindow);
    }
    
    var _Scene_EventHistory_setData = Scene_EventHistory.prototype.setData;
    Scene_EventHistory.prototype.setData = function(item) {
        // 決定後ウィンドウが外に捌けていく処理
        this._windowMoveObject.windowMoveType = -1;
        
        // メッセージ操作ウィンドウの表示指定
        $gameSwitches.setValue(menuShowSwitch, true);
        $gameSwitches.setValue(menuAllowSwitch, true);
        
        // 変数指定
        var isStreet = (item.eventData.isStreet) ? true : false;
        var isNakedness = (item.eventData.isNakedness) ? true : false;
        var fertilizationId = (item.eventData.fertilizationId) ? item.eventData.fertilizationId : 0;
        var fertilizationFlag = (item.eventData.fertilizationId > 0) ? true : false;
        var endId = (item.eventData.endId) ? item.eventData.end : 0;
        var ifId = (item.ifId) ? item.ifId : 0;
        
        $gameSwitches.setValue(102,item.eventData.forceTranse);   // 強制変身フラグ
        $gameSwitches.setValue(105,isStreet);                     // 大通り変身フラグ
        $gameSwitches.setValue(111,item.eventData.bossBattle);    // ボス戦闘フラグ
        $gameSwitches.setValue(115,item.eventData.discharge);     // 変身解除フラグ
        $gameSwitches.setValue(116,item.eventData.trap);          // 束縛状態フラグ
        $gameSwitches.setValue(120,isNakedness);                  // 半裸フラグ
        $gameSwitches.setValue(132,true);                         // 敗北メッセージウィンドウフラグ
        $gameSwitches.setValue(136,fertilizationFlag);            // 受精回想フラグ
        $gameVariables.setValue(111,item.eventData.bossBattleId); // ボス戦闘フラグ
        $gameVariables.setValue(161,item.eventData.stageId);      // ステージ番号
        $gameVariables.setValue(206,fertilizationId);             // 受精イベント振り分け変数
        $gameVariables.setValue(207,endId);                       // エンディング振分フラグ
        $gameVariables.setValue(208,ifId);                          // if振分フラグ
        
        _Scene_EventHistory_setData.apply(this, arguments);
    }
        
    // オプション画面から戻ったときにはムーブイン演出を行わないようにフラグを立てる
    Scene_EventHistory.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.eventHistory.isMoveIn = false;
        Esrt.WindowMove.eventHistory.windowMoveCount = Esrt.WindowMove.eventHistory.windowMoveDuration;
    }
    
    //=============================================================================
    // Window_EventHistory
    //  イベント履歴一覧ウィンドウ
    //=============================================================================
    var _Window_EventHistory_initialize = Window_EventHistory.prototype.initialize;
    Window_EventHistory.prototype.initialize = function(x, y) {
        // ウィンドウ移動演出
        this._windowPositionDirection = 2; // ウィンドウ位置　2:下, 4:左, 6:右, 8:上
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.eventHistory;
        
        _Window_EventHistory_initialize.apply(this, arguments);
    };
    
    //=============================================================================
    // Window_EventHistoryStatus
    //  イベント履歴の実行時状態経験表示ウィンドウ
    //=============================================================================
    var _Window_EventHistoryStatus_initialize = Window_EventHistoryStatus.prototype.initialize;
    Window_EventHistoryStatus.prototype.initialize = function(x, y) {
        // ウィンドウ移動演出
        this._windowPositionDirection = 2; // ウィンドウ位置　2:下, 4:左, 6:右, 8:上
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.eventHistory;
        
        _Window_EventHistoryStatus_initialize.apply(this, arguments);
    };
    
    var _Window_EventHistoryStatus_drawWindow = Window_EventHistoryStatus.prototype.drawWindow;
    Window_EventHistoryStatus.prototype.drawWindow = function() {
        _Window_EventHistoryStatus_drawWindow.apply(this, arguments);
        
        var x = 0;
        var y = 0;
        
        if (this._statusVariables && this._statusVariables[201]) {
            x = 300;
            var count = this._statusVariables[201];
            var max   = this._statusVariables[202] > 5 ? "∞" : this._statusVariables[202];
            this.changeTextColor(this.systemColor());
            this.drawIcon(Esrt.GetFertilizationIconNumber(Number(this._statusVariables[201]) + 1, Number(this._statusVariables[202])), x + 200, y);
            this.drawText("受精カウント", x + 50, y, 134);
            this.resetTextColor();
            this.drawText(count + "／" + max, x + 250, y);
        }
    }
    //=============================================================================
    // Window_EventHistoryTitle
    //  敗北イベント履歴タイトルウィンドウです。
    //=============================================================================
    var _Window_EventHistoryTitle_initialize = Window_EventHistoryTitle.prototype.initialize;
    Window_EventHistoryTitle.prototype.initialize = function(x, y) {
        // ウィンドウ移動演出
        this._windowPositionDirection = 8; // ウィンドウ位置　2:下, 4:左, 6:右, 8:上
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.eventHistory;
        
        _Window_EventHistoryTitle_initialize.apply(this, arguments);
    };
})();
