//=============================================================================
// ESRT_EventHpGauge.js
// ----------------------------------------------------------------------------
// Copyright (c) 2018 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc イベントにHPゲージを表示するプラグイン
 * @author 白
 * 
 * @help 
 * 参考：
 * ・tomoaky(ひきも記は閉鎖しました。) 様
 * 　・ボスＨＰゲージ - http://hikimoki.sakura.ne.jp/plugin/plugin_map.html
 * ・Yanfly 様
 * 　・YEP.40 – Event Mini Label - http://yanfly.moe/2015/12/08/yep-40-event-mini-label/
 *
 *
 */
 
var Esrt = Esrt || {};

(function() {
    'use strict';
	var pluginName    = 'ESRT_EventHpGauge';
    
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var EHpWBufferX = 0;
    var EHpWBufferY = 40;
    var EHpWWidth   = 136;
    
    //=============================================================================
    // Window_EventHpGauge
    //=============================================================================

    function Window_EventHpGauge() {
        this.initialize.apply(this, arguments);
    }

    Window_EventHpGauge.prototype = Object.create(Window_Base.prototype);
    Window_EventHpGauge.prototype.constructor = Window_EventHpGauge;
    
    Window_EventHpGauge.prototype.initialize = function() {
        this._bufferX = EHpWBufferX;
        this._bufferY = EHpWBufferY;
        var width = EHpWWidth;
        var height = this.windowHeight() * 1.5;
        this._range = 500;
        Window_Base.prototype.initialize.call(this, 0, 0, width, height);
        this.opacity = 0;
        this.contentsOpacity = 0;
        this._isHide = true;
        this.visible = false;
        this._updateCount = 0;
        this._isBoss = false;
    };
    
    Window_EventHpGauge.prototype.windowHeight = function() {
        var height = this.fittingHeight(1)
        height = Math.max(height, 36 + this.standardPadding() * 2);
        return height;
    };

    Window_EventHpGauge.prototype.bufferX = function() {
        if (this._bufferX !== undefined) return this._bufferX;
        return EHpWBufferX;
    };
    
    Window_EventHpGauge.prototype.bufferY = function() {
        if (this._bufferY !== undefined) return this._bufferY;
        return EHpWBufferY;
    };
    
    Window_EventHpGauge.prototype.setCharacter = function(character) {
        this._character = character;
        if (!character) return;
        if (character._eventId) this.gatherDisplayData();
        
    };

    Window_EventHpGauge.prototype.gatherDisplayData = function() {
        if(!this._character || !this._character.event()) return;
        this._page = this._character.page();
        this._pageIndex = this._character._pageIndex;
        if (!this._character.page()) {
          return this._isHide = true;
        }
        // 注釈をリストアップ
        var list = this._character.list();
        var max = list.length;
        var comment = '';
        for (var i = 0; i < max; ++i) {
          var ev = list[i];
          if ([108, 408].contains(ev.code)) comment += ev.parameters[0] + '\n';
        }
        this.drawHp();
        this.extractNotedata(comment);
    };

    Window_EventHpGauge.prototype.extractNotedata = function(comment) {
      if (comment === '') return;
      var setBufferY  = /<HP GAUGE Y BUFFER:[ ]([\+\-]\d+)>/i;
      var setRange    = /<HP GAUGE RANGE:[ ](\d+)>/i;
      var setBufferX  = /<HP GAUGE X BUFFER:[ ]([\+\-]\d+)>/i;
      var showHpGauge = /<SHOW HP GAUGE>/i;
      var hideHpGauge = /<HIDE HP GAUGE>/i;
      var bossHpGauge = /<BOSS HP GAUGE>/i;
      var notedata = comment.split(/[\r\n]+/);
      
      for (var i = 0; i < notedata.length; ++i) {
        var line = notedata[i];
        if (line.match(setBufferY)) {
          this._bufferY = parseInt(RegExp.$1);
          
        } else if (line.match(setRange)) {
          this._range = parseInt(RegExp.$1);
          
        } else if (line.match(setBufferX)) {
          this._bufferX = parseInt(RegExp.$1);
          
        } else if (line.match(showHpGauge)) {
          this._isHide = false;
          
        } else if (line.match(hideHpGauge)) {
          this._isHide = true;
          
        } else if (line.match(bossHpGauge)) {
          this._isBoss = true;
          this.width = EHpWWidth * 2;
          this.createContents();
          this.drawHp();
        }
      }

    };
    
    Window_EventHpGauge.prototype.drawHp = function() {
        var event = $gameMap.event(this._character._eventId);
        var battler = event && event.battler ? event.battler() : null;
        if (!battler) return;
        
        var color1 = this.textColor(2);
        var color2 = this.textColor(18);
        this.drawGauge(0, 0, this.width, battler.hpRate(), color1, color2);
    }
    
    Window_EventHpGauge.prototype.drawGauge = function(x, y, width, rate, color1, color2) {
        var fillW = Math.floor((width-40) * rate);
        var gaugeY = y + this.lineHeight() - 8;
        var frameHeight = (this._isBoss) ? 20 : 10;
        var gaugeHeight = (this._isBoss) ? 16 : 6;
        this.contents.fillRect(x, gaugeY-2, width, frameHeight, this.normalColor()); // 枠を追加
        this.contents.fillRect(x+2, gaugeY, width-40, gaugeHeight, this.gaugeBackColor());
        this.contents.gradientFillRect(x+2, gaugeY, fillW, gaugeHeight, color1, color2);
    };
    
    Window_EventHpGauge.prototype.getHp = function() {
        var event = $gameMap.event(this._character._eventId);
        var battler = event && event.battler ? event.battler() : null;
        if (!battler) return 0;
        return battler.hp;
    }
    
    Window_EventHpGauge.prototype.refresh = function() {
        this.contents.clear();
        this.drawHp();
    };
    
    Window_EventHpGauge.prototype.update = function() {
        Window_Base.prototype.update.call(this);
        this.updatePage();
        this.updateOpacity();
        this.updateGauge();
    };
    
    Window_EventHpGauge.prototype.updatePage = function() {
        if (this._pageIndex === this._character._pageIndex) return;
        this._pageIndex = this._character._pageIndex;
        this.contents.clear();
        this.gatherDisplayData();
    };
    
    Window_EventHpGauge.prototype.updateOpacity = function() {
        if (this.showHpGauge() && this._character && !this._character._transparent) {
          this.show();
        } else {
          this.hide();
        }
    };
    
    Window_EventHpGauge.prototype.updateGauge = function() {
        this._updateCount += 1;
        if (this._updateCount % 6 != 0) return;
        
        var event = $gameMap.event(this._character._eventId);
        if (!event || !event._needUpdateHpGauge) return;
        
        event._needUpdateHpGauge = false;
        this.refresh();
    }

    Window_EventHpGauge.prototype.show = function() {
        if (this.contentsOpacity >= 255) return;
        this.contentsOpacity += 16;
        this.visible = true;
    };

    Window_EventHpGauge.prototype.hide = function() {
        if (this._character._transparent) { this.contentsOpacity = 0;}
        if (this.contentsOpacity <= 0) {
          if (this.visible) this.visible = false;
          return;
        }
        this.contentsOpacity -= 8;
    };

    Window_EventHpGauge.prototype.showHpGauge = function() {
        if (!this.getHp()) return false;
        if (this._isHide) return false;
        return true;
    };
    
    //=============================================================================
    // Sprite_Character
    //=============================================================================

    var _Sprite_Character_update = Sprite_Character.prototype.update;
    Sprite_Character.prototype.update = function() {
        _Sprite_Character_update.call(this);
        this.updateHpGauge();
    };

    Sprite_Character.prototype.updateHpGauge = function() {
        this.setupHpGauge();
        if (!this._hpGauge) return;
        this.positionHpGauge();
    };

    Sprite_Character.prototype.positionHpGauge = function() {
        var win = this._hpGauge;
        var width = win.width * win.scale.x;
        win.x = this.x + width / -2 + win.bufferX();
        var height = win.height * win.scale.y;
        var buffer = win.bufferY() * win.scale.y;
        win.y = this.y + (this.height * -1) - height + buffer;
    };

    Sprite_Character.prototype.setupHpGauge = function() {
        if (this._hpGauge) return;
        if (!SceneManager._scene._spriteset) return;
        this._hpGauge = new Window_EventHpGauge();
        this._hpGauge.setCharacter(this._character);
        SceneManager._scene._spriteset.addChild(this._hpGauge);
    };
    
    
    
}());
    
