//=============================================================================
// ESRT_EventPictureTimeline.js
// ----------------------------------------------------------------------------
// Copyright (c) 2017 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:ja
 * @plugindesc イベント表示コマンド一括プラグイン
 * @author 白
 *
 * @help
 * イベントスチルの表示・重ね合わせを管理します。
 *
 *
 */
var Esrt = Esrt || {};
Esrt.event   = Esrt.event || {};
Esrt.event.picture   = Esrt.event.picture || {};
Esrt.event.timeline  = Esrt.event.timeline || [];
Esrt.event.index     = -1;

(function() {
    'use strict';
	var pluginName    = 'ESRT_EventPictureTimeline';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var cgModeSwitch = 81;


    //=============================================================================
    // Scene_Map
    //  表示オプションの切り替えを行った直後に反映します
    //=============================================================================
    var _Scene_Map_createDisplayObjects = Scene_Map.prototype.createDisplayObjects
    Scene_Map.prototype.createDisplayObjects = function() {
        _Scene_Map_createDisplayObjects.apply(this, arguments);

        if(Esrt.event.index >= 0){
            showPresentPictures()
        }
    };
    
    //=============================================================================
    // Game_Interpreter
    //  ・デバッグ時に各種オプション差分を最初から表示するようにします
    //=============================================================================
    var _Game_Interpreter_initialize = Game_Interpreter.prototype.initialize;
    Game_Interpreter.prototype.initialize = function() {
        _Game_Interpreter_initialize.apply(this, arguments);
        if(DataManager.isEventTest()){
            $gameSwitches._data[161] = true; // 断面図を表示
            $gameSwitches._data[162] = true; // 拡大カットを表示
            $gameVariables._data[202] = 5;   // 受精カウント最大を基本の5に初期化
            $gameVariables._data[211] = 1;   // 手前の男性を表示
        }
    };

    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'Timeline'){
            switch(args[0]){
                case 'setPicture':
                    SetPicture(args);
                    break;
                case 'push':
                    PushTimeline(this, args);
                    break;
                case 'next':
                    NextTimeline();
                    break;
                case 'allClear':
                    EventStillAllClear(args);
                    break;
                default:
                    break;
            }
        }
    };


    // 立ち絵ファイル名のセット
    function SetPicture(args) {
        var regVar = /v\[(\d+)\]/;
        var name  = args[1];
        var pictureId = Number(args[2]);
        var filename  = args[3];
        var requirement  = args[4] ? args[4] : 0;
        var blendMode  = args[5] ? args[5] : 0;

        if(!(name in Esrt.event.picture)){ Esrt.event.picture[name] = [] }
        Esrt.event.picture[name].push({
                            "pictureId" : pictureId,
                            "filename": filename,
                            "opacity" : 0,
                            "blendMode" : blendMode,
                            "requirement" : requirement
                        });

    }

    // タイムラインのセット
    function PushTimeline(interpriter, args) {
        var datas = [];
        var names = {};
        for(var i=1; i<args.length; i+=2){
            var name = args[i];
            var index = args[i+1];
            datas.push({"name":name, "index":index,"x":0,"y":0});
            names[name] = true;

            if(Esrt.event.timeline.length == 0){ // タイムラインの一番最初ならプリロードする
                if(!parseInt(index)){ continue;}
                if(!isSetStill(name)){ continue;}
                Esrt.event.picture[name].forEach(function(still) {
                    ImageManager.requestPicture(still.filename + '_' + ('0000' + index).slice( -4 ));
                });
            }
        }

        // 一つ前のデータを引き継ぐ
        var lastData = Esrt.event.timeline[Esrt.event.timeline.length - 1];
        if(typeof lastData == 'object'){
            for(var i=0; i<lastData.length; i++){
                if (!(lastData[i].name in names)) {
                    if(lastData[i].index > 0){
                        datas.push(lastData[i]);
                    }
                }
            }
        }

        Esrt.event.timeline.push(datas);

        //if(DataManager.isEventTest() || $gameSwitches.value(cgModeSwitch)) {
        if($gameVariables.value(cgModeSwitch)) {
            Esrt.event.index += 1;
            showPresentPictures();
            $gameMessage.setBackground(2);
            $gameMessage.add("");
            interpriter.setWaitMode('message');
        }
    }

    // タイムラインの次の画像群を表示
    function NextTimeline() {
        Esrt.event.index += 1;
        showPresentPictures();
        preloadNextPictures();
    }

    // 画像の表示
    function showPresentPictures() {
        if(Esrt.event.index < 0 || Esrt.event.index > Esrt.event.timeline.length) return;
        if(!Array.isArray(Esrt.event.timeline[Esrt.event.index])) return;
        
        if(DataManager.isEventTest()){
            console.info(Esrt.event.index);
        }
        
        Esrt.event.timeline[Esrt.event.index].forEach(function(timelineData) {
            if(!isSetStill(timelineData.name)){ return;}
            Esrt.event.picture[timelineData.name].forEach(function(still) {

                if(!parseInt(timelineData.index)){ // indexが0の時は画像の透明度を0にする
                     $gameScreen.movePicture(still.pictureId, 0,
                                            timelineData.x, timelineData.y, 100, 100,
                                            0, still.blendMode, 60);
                     still.opacity = 0;
                    return;
                }

                if(!checkShowPicture(still.requirement)){ // オプション非表示設定の反映
                     $gameScreen.movePicture(still.pictureId, 0,
                                            timelineData.x, timelineData.y, 100, 100,
                                            0, still.blendMode, 60);
                     still.opacity = 0;
                    return;
                }
                $gameScreen.showPicture(still.pictureId, still.filename + '_' + ('0000' + timelineData.index).slice( -4 ), 0,
                                       timelineData.x, timelineData.y, 100, 100,
                                       still.opacity, still.blendMode);

               if(still.opacity == 0){ // 元の透過度0の時はスゥッと表示
                    $gameScreen.movePicture(still.pictureId, 0,
                                           timelineData.x, timelineData.y, 100, 100,
                                           255, still.blendMode, 60);
                    still.opacity = 255;
               }
           });
           
           if(DataManager.isEventTest()){
               console.info(timelineData.name + " " + timelineData.index);
           }
        });
    }

    // 画像のロード
    function preloadNextPictures() {
        if(Esrt.event.index < 0 || Esrt.event.index + 1 > Esrt.event.timeline.length) return;
        if(!Array.isArray(Esrt.event.timeline[Esrt.event.index + 1])) return;

        Esrt.event.timeline[Esrt.event.index + 1].forEach(function(timelineData) {
            if(!parseInt(timelineData.index)){ return;}
            if(!isSetStill(timelineData.name)){ return;}
            Esrt.event.picture[timelineData.name].forEach(function(still) {
                $gameScreen.showPicture(still.pictureId + 1, still.filename + '_' + ('0000' + timelineData.index).slice( -4 ), 0,
                                       timelineData.x, timelineData.y, 100, 100,
                                       0, still.blendMode);
            });
        });
    }


    // オプションによる画像非表示設定の判定
    function checkShowPicture(requirement) {
        if(requirement == 0) return true;

        // スイッチ指定
        if (requirement.match(/s\[(\d+)\](.*)/i)) {
            var switchId = Number(RegExp.$1);
            if(RegExp.$2 != "f"){
                return $gameSwitches.value(switchId);
            }else{
                return !$gameSwitches.value(switchId);
            }
        }

        // 変数指定
        if (requirement.match(/v\[(\d+)\](\d*)/i)) {
          var variableId = Number(RegExp.$1);
          return $gameVariables.value(variableId) === Number(RegExp.$2);
        }

        // その他、引数が数字のみの場合はスイッチONのとき表示として扱う
        return $gameSwitches.value(Number(requirement));
    }

    // 該当イベントスチルがセットされてるかチェック
    function isSetStill(name) {
        if(!(name in Esrt.event.picture)){
            if (Utils.isNwjs() && Utils.isOptionValid('test')) {
                console.info("error : " + name);
            }
            return false;
        }
        return true;
    }

    // イベントファイル名のセット
    function EventStillAllClear() {
        Object.keys(Esrt.event.picture).forEach(function(key) {
            var still = Esrt.event.picture[key];

            Esrt.event.picture[key].forEach(function(still) {
                $gameScreen.erasePicture(Number(still.pictureId));
                $gameScreen.erasePicture(Number(still.pictureId + 1));
            });
            delete Esrt.event.picture[key];
        });
        Esrt.event.picture = {};
        Esrt.event.timeline  = [];
        Esrt.event.index     = -1;
    }
})();
