//=============================================================================
// ESRT_EventReSpawnList.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc リストデータに従ってテンプレートイベントからイベントを生成するプラグインコマンドを追加するプラグイン
 * @author 白
 *
 *
 * @help 必要プラグイン
 * ・イベント動的生成プラグイン（トリアコンタン様）
 * ・テンプレートイベントプラグイン（トリアコンタン様）
 *
 */
 
 var Esrt = Esrt || {};
 Esrt.EventRespawnList = Esrt.EventRespawnList || {};
 Esrt.EventRespawnList.wait = 30;
 Esrt.EventRespawnList.moveSpeed = 4;

//=============================================================================
//    汎用関数の定義
//=============================================================================
(function() {
    'use strict';
	var pluginName    = 'ESRT_EventReSpawnList';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var gameModeName = ["normal", "extra"];
    
    //=============================================================================
    // DataManager
    //  イベント生成リスト設定ファイル読み込み処理を定義します
    //=============================================================================
    DataManager.loadEventRespawnList = function() {
        this.loadDataFile('$dataEventRespawn', 'EventRespawnList.json');
        return;
    };

    //=============================================================================
    // Scene_Boot
    //  イベント生成リスト設定ファイル読み込み処理を追加定義します
    //=============================================================================
    var _Scene_Boot_create      = Scene_Boot.prototype.create;
    Scene_Boot.prototype.create = function() {
        _Scene_Boot_create.call(this);
        DataManager.loadEventRespawnList();
    };
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'EventRespawnList'){
            switch(args[0]){
                case 'setID':
                    this.setStageID(args);
                    break;
                case 'setLoop':
                    this.setRespawnListLoop(args);
                    break;
                case 'next':
                    this.nextRespawnList(args);
                    break;
                case 'setSpeed':
                    this.setSpeed(args);
                    break;
                case 'eventWait':
                    this.eventWait();
                    break;
                default:
                    break;
            }
        }
    };
    
    // 生成リストIDの指定
    Game_Interpreter.prototype.setStageID = function(args) {
        var regVar = /v\[(\d+)\]/;
        var id  = args[1];
        var gameMode = gameModeName[$gameVariables.value(171)];
        
        if (regVar.test(id)) {
            id = Number(id.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        if (!$dataEventRespawn[id] || !$dataEventRespawn[id][gameMode] ) {
            id = -1;
        }
        var event = this.character(0);
        event._stageId = id;
        event._listNum = 0;
    }
    
    // 生成リストのループ有無を設定
    Game_Interpreter.prototype.setRespawnListLoop = function(args) {
        var isLoop  = (args[1] || '').toUpperCase() === 'ON';
        
        var event = this.character(0);
        event._respawnListRoop = isLoop;
    }
    
    // 生成リストを一業読み込み・実行
    Game_Interpreter.prototype.nextRespawnList = function(args) {
        var gameMode = gameModeName[$gameVariables.value(171)];
        var event = this.character(0);
        
        if (event._stageId === -1) {return;}
        if (typeof event._stageId === "undefined") {return;}
        
        this.EventsRespawn($dataEventRespawn[event._stageId][gameMode][event._listNum]);
        
        event._listNum += 1;
        
        this.EventsRespawnListEnd();
    }
    
    // 生成リストの実行間隔・レース時オブジェクト移動速度の設定
    Game_Interpreter.prototype.setSpeed = function(args) {
        
        if (args[1] == "default") {
            Esrt.EventRespawnList.wait = 30;
            Esrt.EventRespawnList.moveSpeed = 4;
            
        } else if (args[1] == "up") {
            $gameSwitches.setValue(128, false);
            
            Esrt.EventRespawnList.wait -= 3;
            
            if (Esrt.EventRespawnList.wait < 9) {
                Esrt.EventRespawnList.wait = 9;
            }
            
            this.setMoveSpeed();
            
            $gameMap._parallaxSy -= 2;
            if (Esrt.EventRespawnList.wait == 9) {
                $gameSwitches.setValue(127, true);
            }
        } else if (args[1] == "down") {
            $gameSwitches.setValue(127, false);
            
            Esrt.EventRespawnList.wait += 3;
            
            if (Esrt.EventRespawnList.wait > 30) {
                Esrt.EventRespawnList.wait = 30;
            }
            
            this.setMoveSpeed();
            
            $gameMap._parallaxSy += 2;
            if (Esrt.EventRespawnList.wait == 30) {
                $gameSwitches.setValue(128, true);
            }
        }
    }
    
    
    // 生成リストの実行間隔・レース時オブジェクト移動速度の設定
    Game_Interpreter.prototype.setMoveSpeed = function(args) {
        if (Esrt.EventRespawnList.wait <= 20) {
            Esrt.EventRespawnList.moveSpeed = 5;
        } else if (Esrt.EventRespawnList.wait <= 30) {
            Esrt.EventRespawnList.moveSpeed = 4;
        }
    }
    
    // 生成リストの実行間隔でウェイトを実行する
    Game_Interpreter.prototype.eventWait = function() {
        this.wait(Esrt.EventRespawnList.wait);
    }
    
    // イベントを生成する
    Game_Interpreter.prototype.EventsRespawn = function(eventList) {
        eventList.forEach(function(parameter) {
            $gameMap.spawnEvent(this.getEventIdForEventReSpawn(parameter.id, true), parameter.x, parameter.y, true);
            
            this.EventsRespawnSetSelfSwitch(parameter);
            this.EventsRespawnSetSelfVariable(parameter);
        }.bind(this));
    }
    
    // 生成したイベントに対してセルフスイッチを設定
    Game_Interpreter.prototype.EventsRespawnSetSelfSwitch = function(parameter) {
        if (typeof parameter.sw === "object") { // 生成したイベントに対してセルフスイッチを設定
            var eventId = $gameMap.getLastSpawnEventId();
            
            Object.keys(parameter.sw).forEach(function(id) {
                $gameTemp.setSelfSwVarEvent($gameMap.mapId(), eventId);
                $gameSwitches.setValue(id, parameter.sw[id]);
            }.bind(this));
        }
    }
    
    // 生成したイベントに対してセルフ変数を設定（要YEP_SelfSwVar.js）
    Game_Interpreter.prototype.EventsRespawnSetSelfVariable = function(parameter) {
        if (typeof parameter.var === "object") {
            var eventId = $gameMap.getLastSpawnEventId();
            
            Object.keys(parameter.var).forEach(function(id) {
                $gameTemp.setSelfSwVarEvent($gameMap.mapId(), eventId);
                $gameVariables.setValue(id, parameter.var[id]);
            }.bind(this));
        }
    }
    
    // リスト終端に辿り着いたとき、フラグがあればループする
    Game_Interpreter.prototype.EventsRespawnListEnd = function ListEnd() {
        var event = this.character(0);
        var gameMode = gameModeName[$gameVariables.value(171)];
        
        if (event._listNum === $dataEventRespawn[event._stageId][gameMode].length) {
            if (event._respawnListRoop) {
                event._listNum = 0;
                
            } else {
                event._stageId = -1;
            }
        }
    }

}());
