//=============================================================================
// ESRT_EventRectangle.js
//=============================================================================
// Copyright (c) 2018 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 *
 * @plugindesc イベント周囲矩形描画プラグイン
 * @author 白
 *
 * @help Lyson様製プレイヤー探索プラグインの探索範囲を描画します。
 *　※マンカインド様製プレイヤー探索プラグインの視界描画部分を移植・改造し、
 *　Lyson様製プレイヤー探索プラグインに視界を描画しています。
 *  マンカインド様
 *    プレイヤー探索プラグイン http://mankind-games.blogspot.com/2016/09/blog-post.html
 *
 *  Lyson様 
 *    Sensor SelfSwitch https://forums.rpgmakerweb.com/index.php?threads/sensor-selfswitch.48715/
 *
*/
(function () {
    'use strict';
	const pluginName = 'ESRT_EventRectangle';

    const parameters = PluginManager.parameters(pluginName);

    let DefRangeVisible = parameters['Range_Visible'];
    let DefRangeColor = parameters['Range_Color'];
    let DefRangeOpacity = parseInt(parameters['Range_Opacity']);

    let DIR_UP = 8;
    let DIR_DOWN = 2;
    let DIR_RIGHT = 6;
    let DIR_LEFT = 4;


    //=========================================================================
    // Game_Interpreter
    //  ・プレイヤー探索制御プラグインコマンド
    //  ・イベントをプレイヤー近くまで移動させるコマンド
    //  を定義
    //=========================================================================
    const _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === "EventRectangle") {
            switch (args[0]) {
                case "SetSquare": // 矩形表示
      				var event = this.character(0);
                    event.setRangeVisible(true);
                    event.setRectColor(args[1]);
                    event.setRectOpacity(args[2]);
                    event.setRectSize(args[3],args[4]);
                    SceneManager._scene._spriteset.addEventRectangle(event);
                    break;
                case "HideSquare": // 矩形非表示
      				var event = this.character(0);
                    event.setRangeVisible(false);
                    break;
                case "SetSquareSize": // 矩形サイズ再設定
      				var event = this.character(0);
                    event.setRectSize(args[1],args[2]);
                break;
                case "ReDrawSquare": //  矩形再描画
      				var event = this.character(0);
                    SceneManager._scene._spriteset.reDrawEventRectangle(event);
                    break;
            }
        }
    };

    //=========================================================================
    // Game_CharacterBase
    //
    //  視界描画状態：
    //    0 = 描画停止
    //    1 = 描画更新
    //    2 = 描画新規
    //=========================================================================
    const _Game_CharacterBaseInitMembers = Game_CharacterBase.prototype.initMembers;
    Game_CharacterBase.prototype.initMembers = function() {
        _Game_CharacterBaseInitMembers.call(this);

        this._viewRangeStatus = 0;
        this._rangeVisible = false;
        this._rectWidth  = -1;
        this._rectHeight = -1;
    };

    Game_CharacterBase.prototype.setRectColor = function(color) {
        this._rectColor  = color;
    };

    Game_CharacterBase.prototype.getRectColor = function() {
        return this._rectColor;
    };

    Game_CharacterBase.prototype.setRectOpacity = function(opacity) {
        this._rectOpacity  = parseInt(opacity);
    };

    Game_CharacterBase.prototype.getRectOpacity = function() {
        return this._rectOpacity;
    };

    Game_CharacterBase.prototype.setRectSize = function(width, height) {
        var regVar = /v\[(\d+)\]/;
        $gameTemp.setSelfSwVarEvent($gameMap.mapId(), this.eventId());
        this._rectWidth = (regVar.test(width)) ? $gameVariables.value(RegExp.$1) : width;
        this._rectHeight = (regVar.test(height)) ? $gameVariables.value(RegExp.$1) : height;
        $gameTemp.clearSelfSwVarEvent();
    };

    Game_CharacterBase.prototype.getRectWidth = function() {
        return this._rectWidth;
    };

    Game_CharacterBase.prototype.getRectHeight = function() {
        return this._rectHeight;
    };

    Game_CharacterBase.prototype.setRangeVisible = function(rangeVisible) {
        this._rangeVisible = rangeVisible;
    };

    Game_CharacterBase.prototype.getRangeVisible = function() {
        return this._rangeVisible;
    };

    Game_CharacterBase.prototype.isCreateRange = function() {
        return this._createRange;
    };

    Game_CharacterBase.prototype.enableCreateRange = function() {
        this._createRange = true;
    };
    
    //=========================================================================
    // Spriteset_Map
    //  探索者の周囲に図形を描画させる処理を追加定義します。
    //=========================================================================
    const _Spriteset_Map_createLowerLayer = Spriteset_Map.prototype.createLowerLayer;
    Spriteset_Map.prototype.createLowerLayer = function() {
        _Spriteset_Map_createLowerLayer.call(this);
        this.createEventRectangle();
    }

    Spriteset_Map.prototype.createEventRectangle = function() {
        this._viewRangeSpriteParent = new Sprite();
        this._eventRectangleSprites = []
        this._viewRangeSpriteParent.z = 3;
        this._tilemap.addChild(this._viewRangeSpriteParent);
        
        
        $gameMap.events().forEach(function(event) {
            if(event.getRectWidth() >= 0 && event.getRectHeight() >= 0) {
                this._eventRectangleSprites.push(new Sprite_EventRectangle(event));
                event.enableCreateRange();
            }
        }, this);
        for (let i = 0; i < this._eventRectangleSprites.length; i++) {
            this._viewRangeSpriteParent.addChild(this._eventRectangleSprites[i]);
        }
    };

    Spriteset_Map.prototype.addEventRectangle = function(event) {
        let cnt = this._eventRectangleSprites.length;
        
        this._eventRectangleSprites.push(new Sprite_EventRectangle(event));
        event.enableCreateRange();
        
        this._viewRangeSpriteParent.addChild(this._eventRectangleSprites[cnt]);
    };
    
    Spriteset_Map.prototype.reDrawEventRectangle = function(event) {
        let cnt = this._eventRectangleSprites.length;
        
        this._eventRectangleSprites[cnt-1].bitmap = null;
    };

    //=========================================================================
    // Sprite_EventRectangle
    //  探索者の周囲に図形を描画させる処理を定義します。
    //=========================================================================
    function Sprite_EventRectangle() {
        this.initialize.apply(this, arguments);
    }

    Sprite_EventRectangle.prototype = Object.create(Sprite.prototype);
    Sprite_EventRectangle.prototype.constructor = Sprite_EventRectangle;

    Sprite_EventRectangle.prototype.initialize = function(character) {
        Sprite.prototype.initialize.call(this);
        this.initMembers();
        this.setCharacter(character);
        this._frameCount = 0;
        this.z = 6;
    };

    Sprite_EventRectangle.prototype.initMembers = function() {
        this._character = null;
        this._coordinates = null;
    };

    Sprite_EventRectangle.prototype.setCharacter = function(character) {
        this._character = character;
    };

    Sprite_EventRectangle.prototype.update = function() {
        Sprite.prototype.update.call(this);
        
        this._frameCount += 1;
        if(this._frameCount % 3 != 0) return;

        let rangeVisible = this._character.getRangeVisible();
        let defVisible = DefRangeVisible;

        if(rangeVisible) {
            this.updatePosition();
            if(!this.bitmap) {
                // 描画新規
                this.createBitmap();
            }
            this.visible = true;
        } else {
            this.visible = false;
        }
    };

    Sprite_EventRectangle.prototype.createBitmap = function() {
        let direction = this._character.direction();
        let tileWidth = $gameMap.tileWidth();
        let tileHeight = $gameMap.tileHeight();
        let color = this._character.getRectColor();
        let opacity = this._character.getRectOpacity();
        let rectWidth = this._character.getRectWidth();
        let rectHeight = this._character.getRectHeight();

        let width = tileWidth * rectWidth * 2 + tileWidth;
        let height = tileHeight * rectHeight * 2 + tileHeight;
        this.anchor.x = 0.5;
        this.anchor.y = 0.55;
        this.bitmap = new Bitmap(width, height);
        this.bitmap.fillAll(color);

        this.opacity = opacity;
        this.baseOpacity = opacity;
        this.blendMode = Graphics.BLEND_ADD;
        this.visible = true;
    };

    Sprite_EventRectangle.prototype.updatePosition = function() {
        let tileWidth = $gameMap.tileWidth();
        let tileHeight = $gameMap.tileHeight();
        let cx = this._character.screenX();
        let cy = this._character.screenY();

        this.x = cx;
        this.y = cy - 15;
        this.opacity = this.baseOpacity * 0.75 + Math.sin(this._frameCount / 20) * this.baseOpacity / 4;
    };

})();
