//=============================================================================
// ESRT_EventSettingHold.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc イベントの画像・移動速度・移動頻度をページ変更時に変更せず固定するプラグイン
 * @author 白
 * 
 * @help 
 *
 * This plugin is released under the MIT License.
 *
 */
 
var Esrt = Esrt || {};

(function() {
    'use strict';
	var pluginName    = 'ESRT_EventSettingHold';
    
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);

    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'SettingHold'){
            switch(args[0]){
                case 'setting':
                    this.SetSettingHold(args[1]);
                    break;
                default:
                    break;
            }
        }
    };
    
    Game_Interpreter.prototype.SetSettingHold = function(param) {
        var flag = (param == "on") ? true : false;
        var event = $gameMap.event(this._eventId);
        event.SetSettingHold(flag);
    }
    
    //=============================================================================
    // Game_Event
    //  画像固定フラグがオンの時、画像の変更処理をスキップします
    //=============================================================================
    Game_CharacterBase.prototype.SetSettingHold = function(flag) {
        this._isImageHold = flag;
        this._isMoveSpeedHold = flag;
        this._isMoveFrequencyHold = flag;
    };
    
    Game_CharacterBase.prototype.ImageHold = function() {
        return this._isImageHold;
    };
    
    //=============================================================================
    // Game_Event
    //  画像固定フラグがオンの時、画像の変更処理をスキップします
    //=============================================================================
    var _Game_Event_setImage      = Game_Event.prototype.setImage;
    Game_Event.prototype.setImage = function(characterName, characterIndex) {
        if (this._isImageHold) {return;}
        
        _Game_Event_setImage.apply(this, arguments);
    };
    
    var _Game_Event_setTileImage      = Game_Event.prototype.setTileImage;
    Game_Event.prototype.setTileImage = function(tileId) {
        if (this._isImageHold) {return;}
        
        _Game_Event_setTileImage.apply(this, arguments);
    };
    
    var _Game_Event_setMoveSpeed      = Game_Event.prototype.setMoveSpeed;
    Game_Event.prototype.setMoveSpeed = function(moveSpeed) {
        if (this._isMoveSpeedHold) {return;}
        
        _Game_Event_setMoveSpeed.apply(this, arguments);
    };
    
    var _Game_Event_setMoveFrequency      = Game_Event.prototype.setMoveFrequency;
    Game_Event.prototype.setMoveFrequency = function() {
        if (this._isMoveFrequencyHold) {return;}
        
        _Game_Event_setMoveFrequency.apply(this, arguments);
    };
}());
    
