//=============================================================================
// ESRT_MapCoinWindow.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc Allows you to display the  PC1,s status on your map.
 * @author white_esorat
 *
 * @param ミニステータス表示変数
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 108
 *
 * @help 
 *
 * This plugin does not provide plugin commands.
 * 
 * This plugin is released under the MIT License.
 */
 
/*:ja
 * @plugindesc マップ上にステータスウィンドウ・変身立ち絵を表示する
 * @author 白
 *
 * @param ミニステータス表示変数
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 108
 * 
 * @help 
 *
 * このプラグインには、プラグインコマンドはありません。
 * 
 * This plugin is released under the MIT License.
 *
 */
 
 var Esrt = Esrt || {};
 Esrt.WindowMove.Coin = Esrt.WindowMove.Coin || {};
 Esrt.WindowMove.Coin.isMoveIn = true;
 Esrt.WindowMove.Coin.windowMoveDuration = 60;
 Esrt.WindowMove.Coin.windowMoveCount = 0;
 Esrt.WindowMove.Coin.windowMoveType = 1;
 Esrt.WindowMove.Coin.windowMoveRate = 0;
 
(function() {
    'use strict';
    var pluginName    = 'ESRT_MapCoinWindow';
    var TRANS_COUNT_CYCLE = 1296;

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var windowWidth  = 170;
    var windowHeight = 72;
    var windowX      = 500;
    var windowY      = -10;
    var windowShowVariable = parseInt(parameters['ミニステータス表示変数']);
    var stillTypeVariable = 91;
    var forceTransSwitch = 102;
    
    //=============================================================================
    // Scene_Map
    //    ステータスウィンドウをマップ上に表示します。
    //=============================================================================
    var _Scene_Menu_addSceneMapUpdateObject = Scene_Map.prototype.addSceneMapUpdateObject;
    Scene_Map.prototype.addSceneMapUpdateObject = function() {
        _Scene_Menu_addSceneMapUpdateObject.apply(this, arguments);
        
        Esrt.WindowMove.SceneMapUpdateObjects.push(Esrt.WindowMove.Coin);
        Esrt.WindowMove.Coin.windowMoveType = ($gameSwitches.value(109)) ? -1 : 1;
    };
    
    var _Scene_Map_createAllWindows = Scene_Map.prototype.createAllWindows;
    Scene_Map.prototype.createAllWindows = function() {
        this._time = 0;
        
        this.createCoinWindow();
        
        _Scene_Map_createAllWindows.call(this);
    };
    
    Scene_Map.prototype.createCoinWindow = function() {
        this._miniStatusWindow = new Window_Coin(100, 0);
        this.addWindow(this._miniStatusWindow);
    };
    
    //=============================================================================
    // Window_Coin
    //    ミニステータスウィンドウを定義します。
    //=============================================================================
    function Window_Coin() {
        this.initialize.apply(this, arguments);
    }
    Window_Coin.prototype = Object.create(Window_Base.prototype);
    Window_Coin.prototype.constructor = Window_Coin;
    Window_Coin.prototype.initialize = function() {
        var width = windowWidth;
        var height = windowHeight;
        var x = windowX;
        var y = windowY;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        this.hide();
        
        this._count = 0;
        this._maxCoin = 0;
        
        this._windowPositionDirection = 8; // ウィンドウ位置　8:上
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Coin;
    };
    
    Window_Coin.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_Name');
    };
    
    var _Window_Coin_update = Window_Coin.prototype.update;
    Window_Coin.prototype.update = function() {
        _Window_Coin_update.call(this);
        
        this._count++;
        if (!this.drawCheck()) {return;}
        
        this.refresh();
        
        this.drawCoin();
    }
    
    Window_Coin.prototype.drawCheck = function() {
        if (!$dataSystem) { return false;}
        if (this._count % 12 != 0) { return false;}
        
        var stageNum = $gameVariables.value(161);
        var gameMode = $gameVariables.value(171);
        this._maxCoin = ($dataStage.maxCoin && $dataStage.maxCoin[stageNum]) ? $dataStage.maxCoin[stageNum][Esrt.gameModeName[gameMode]] : 0;
        
        if (this._maxCoin >= 1 && $gameVariables.value(windowShowVariable) >= 1) {
            this.show();
            return true;
        } else {
            this.hide();
            return false;
        }
        
    };
    
	Window_Coin.prototype.drawCoin = function() {
        var coin    = ( '  ' + $gameVariables.value(128) ).slice( -2 );
        
        this.drawIcon(87, 0, 0)
        this.drawText(coin + "／" + this._maxCoin, 40, 0);
        
	};

    Window_Coin.prototype.refresh = function() {
        if (this.contents) {
            this.contents.clear();
        }
    };
})();
