//=============================================================================
// ESRT_MapStatusWindow.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc Allows you to display the  PC1,s status on your map.
 * @author white_esorat
 *
 * @param 変身ボタン表示スイッチ
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 241
 *
 * @param 変身タッチ状態スイッチ
 * @desc 状態を記録するスイッチを指定します。
 * @default 242
 *
 * @param リトライボタン表示スイッチ
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 251
 *
 * @param ミニステータス表示変数
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 108
 *
 * @help 
 *
 * This plugin does not provide plugin commands.
 * 
 * This plugin is released under the MIT License.
 */
 
/*:ja
 * @plugindesc マップ上にステータスウィンドウ・変身立ち絵を表示する
 * @author 白
 *
 * @param 変身ボタン表示スイッチ
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 241
 *
 * @param 変身タッチ状態スイッチ
 * @desc 状態を記録するスイッチを指定します。
 * @default 242
 *
 * @param リトライボタン表示スイッチ
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 251
 *
 * @param ミニステータス表示変数
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 108
 * 
 * @help 
 *
 * このプラグインには、プラグインコマンドはありません。
 * 
 * This plugin is released under the MIT License.
 *
 */
 
 var Esrt = Esrt || {};
 Esrt.transButton = Esrt.transButton || {};
 Esrt.transButton.fadeCount = 0;
 Esrt.WindowMove.MiniStatus = Esrt.WindowMove.MiniStatus || {};
 Esrt.WindowMove.MiniStatus.hidden = false;
 Esrt.WindowMove.MiniStatus.isMoveIn = true;
 Esrt.WindowMove.MiniStatus.windowMoveDuration = 60;
 Esrt.WindowMove.MiniStatus.windowMoveCount = 0;
 Esrt.WindowMove.MiniStatus.windowMoveType = 1;
 Esrt.WindowMove.MiniStatus.windowMoveRate = 0;
 Esrt.WindowMove.MiniStatusGauge = Esrt.WindowMove.MiniStatusGauge || {};
 Esrt.WindowMove.MiniStatusGauge.isMoveIn = true;
 Esrt.WindowMove.MiniStatusGauge.windowMoveDuration = 60;
 Esrt.WindowMove.MiniStatusGauge.windowMoveCount = 0;
 Esrt.WindowMove.MiniStatusGauge.windowMoveType = 1;
 Esrt.WindowMove.MiniStatusGauge.windowMoveRate = 0;
 
(function() {
    'use strict';
    var pluginName    = 'ESRT_MapStatusWindow';
    var TRANS_COUNT_CYCLE = 1296;

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var windowWidth  = 220;
    var windowHeight = 500;
    var windowY      = 50;
    var transButtonShowSwitch = parseInt(parameters['変身ボタン表示スイッチ']);
    var transTouchSwitch = parseInt(parameters['変身タッチ状態スイッチ']);
    var retryButtonShowSwitch = parseInt(parameters['リトライボタン表示スイッチ']);
    var windowShowVariable = parseInt(parameters['ミニステータス表示変数']);
    var stillTypeVariable = 91;
    var forceTransSwitch = 102;
    
    //=============================================================================
    // ImageManager
	//  ボタンフォルダからロードする関数を追加
	//  メニューコマンドにリトライ・ステージ選択へ戻るを追加する
    //=============================================================================
    ImageManager.loadPictureMapStill = function(filename, hue) {
        return this.loadBitmap('img/pictures/map_still/', filename, hue, true);
    };
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'MapStatus'){
            switch(args[0]){
                case 'fadeOut':
                    this.MapStatusFadeOut();
                    break;
                case 'fadeOutExclusiveOfGauge':
                    this.MapStatusFadeOutExclusiveOfGauge();
                    break;
                case 'fadeOutOff':
                    this.MapStatusFadeOutOff();
                    break;
                case 'transButtonFadeIn':
                    this.TransButtonFadeIn();
                    break;
                default:
                    break;
            }
        }
    };
    
    Game_Interpreter.prototype.MapStatusFadeOut = function() {
        if (Esrt.WindowMove.MiniStatus) {
            Esrt.WindowMove.MiniStatus.hidden = true;
            Esrt.WindowMove.MiniStatus.windowMoveCount = 0;
            Esrt.WindowMove.MiniStatus.windowMoveType = -1;
        }
        if (Esrt.WindowMove.Coin) {
            Esrt.WindowMove.Coin.windowMoveCount = 0;
            Esrt.WindowMove.Coin.windowMoveType = -1;
        }
        if (Esrt.WindowMove.MiniStatusGauge) {
            Esrt.WindowMove.MiniStatusGauge.windowMoveCount = 0;
            Esrt.WindowMove.MiniStatusGauge.windowMoveType = -1;
        }
    }
    
    Game_Interpreter.prototype.MapStatusFadeOutExclusiveOfGauge = function() {
        if (Esrt.WindowMove.MiniStatus) {
            Esrt.WindowMove.MiniStatus.hidden = true;
            Esrt.WindowMove.MiniStatus.windowMoveCount = 0;
            Esrt.WindowMove.MiniStatus.windowMoveType = -1;
        }
        if (Esrt.WindowMove.Coin) {
            Esrt.WindowMove.Coin.windowMoveCount = 0;
            Esrt.WindowMove.Coin.windowMoveType = -1;
        }
    }
    
    Game_Interpreter.prototype.MapStatusFadeOutOff = function() {
        Esrt.WindowMove.MiniStatus.hidden = false;
    }
    
    Game_Interpreter.prototype.TransButtonFadeIn = function() {
        Esrt.transButton.fadeCount = 30;
    }
    //=============================================================================
    // Window_Base
    //    ウィンドウ枠データを読み込みます。
    //=============================================================================
    
    var _Window_Base_loadWindowskin = Window_Base.prototype.loadWindowskin;
    Window_Base.prototype.loadWindowskin = function() {
        _Window_Base_loadWindowskin.apply(this, arguments);
        
        ImageManager.loadSystem('Window_FourceTrance');
    };
    
    //=============================================================================
    // Scene_Map
    //    ステータスウィンドウをマップ上に表示します。
    //=============================================================================
    var _Scene_Map_addSceneMapUpdateObject = Scene_Map.prototype.addSceneMapUpdateObject;
    Scene_Map.prototype.addSceneMapUpdateObject = function() {
        _Scene_Map_addSceneMapUpdateObject.apply(this, arguments);
        
        Esrt.WindowMove.SceneMapUpdateObjects.push(Esrt.WindowMove.MiniStatus);
        Esrt.WindowMove.SceneMapUpdateObjects.push(Esrt.WindowMove.MiniStatusGauge);
        Esrt.WindowMove.MiniStatus.windowMoveType = ($gameSwitches.value(109)) ? -1 : 1;
        Esrt.WindowMove.MiniStatusGauge.windowMoveType = ($gameSwitches.value(109)) ? -1 : 1;
    };
    
    var _Scene_Map_createAllWindows = Scene_Map.prototype.createAllWindows;
    Scene_Map.prototype.createAllWindows = function() {
        this._time = 0;
        
        this.createMiniStatusWindow();
        this.createLeftMiniStatusWindow();
        this.createTransButtonWindow();
        this.createRetryButtonWindow();
        this.createTransButtonEmphasisWindow();
        
        _Scene_Map_createAllWindows.call(this);
    };
    
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update.call(this);
        
        this._time += 1;
        if (this._time % 12 != 0) {return;}
        
        this.CheckandChangeTransButtonVisible();
        this.CheckandChangeRetryButtonVisible();
        this.CheckandChangeTransButtonFileName();
        this.TransButtonFade();
    };
    
    Scene_Map.prototype.CheckandChangeTransButtonFileName = function() {
        var stilltype = $gameVariables.value(91);
        
        if (stilltype == 1 || stilltype == 4 || stilltype == 8) {
            this._transButtonWindow.setSprite('button_attack');
            
        } else if (stilltype == 3) {
            this._transButtonWindow.setSprite('button_trap');
            
        } else if (stilltype == 6) {
            this._transButtonWindow.setSprite('button_charm');
            
        }  else if (stilltype == 7) {
            this._transButtonWindow.setSprite('button_accel');
            
        } else if (stilltype == 9) {
            this._transButtonWindow.setSprite('button_goodbye');
        }
    }
    
    Scene_Map.prototype.CheckandChangeTransButtonVisible = function() {
        if ($gameSwitches.value(transButtonShowSwitch)) {
            this._transButtonWindow.show();
            this._transButtonWindow.open();
        } else {
            this._transButtonWindow.hide();
            this._transButtonWindow.close();
        }
    };
    
    Scene_Map.prototype.TransButtonFade = function() {
        if (Esrt.transButton.fadeCount > 0) {
            this._transButtonWindow.alpha = 1 / Esrt.transButton.fadeCount;
            Esrt.transButton.fadeCount -= 1;
        } else {
            this._transButtonWindow.alpha = 1;
        }
    };
    
    Scene_Map.prototype.CheckandChangeRetryButtonVisible = function() {
        if ($gameSwitches.value(retryButtonShowSwitch)) {
            this._retryButtonWindow.show();
            this._retryButtonWindow.open();
        } else {
            this._retryButtonWindow.hide();
            this._retryButtonWindow.close();
        }
    };
    
    Scene_Map.prototype.createMiniStatusWindow = function() {
        this._miniStatusWindow = new Window_MiniStatus(100, 0);
        this.addWindow(this._miniStatusWindow);
        
        this._miniStatusGaugeWindow = new Window_MiniStatusGauge(100, 0);
        this.addWindow(this._miniStatusGaugeWindow);
    };
    
    Scene_Map.prototype.createLeftMiniStatusWindow = function() {
        this._leftMiniStatusWindow = new Window_LeftMiniStatus(100, 0);
        this.addWindow(this._leftMiniStatusWindow);
    };
    
    Scene_Map.prototype.createTransButtonWindow = function() {
        this._transButtonWindow = new Window_Button(630, 410, 190, 210, 'button_trans');
        this._transButtonWindow.setHandler('touchInside', this.commandTrans.bind(this));
        this._transButtonWindow.hide();
        this._transButtonWindow.close();
        this.addWindow(this._transButtonWindow);
        
    };
    
    Scene_Map.prototype.createRetryButtonWindow = function() {
        this._retryButtonWindow = new Window_Button(630, 410, 190, 210, 'button_retry');
        this._retryButtonWindow.setHandler('touchInside', this.commandRetray.bind(this));
        this._retryButtonWindow.hide();
        this._retryButtonWindow.close();
        this.addWindow(this._retryButtonWindow);
    };
    
    Scene_Map.prototype.createTransButtonEmphasisWindow = function() {
        this._transButtonEmphasisWindow = new Window_ButtonEmphasis(780, 580, 10, 10);
        this.addWindow(this._transButtonEmphasisWindow);
    };
    
    Scene_Map.prototype.commandTrans = function() {
        $gameSwitches.setValue(transTouchSwitch, true);
    };
    
    Scene_Map.prototype.commandRetray = function() {
        $gameSelfSwitches.clear();
        $gamePlayer.setTransparent(true);
        $gamePlayer.reserveTransfer(36, 2, 2, 0, 0);
    };
    
    //=============================================================================
    // Scene_Map
	//  ステージ失敗時にメニューボタンをリトライに割当
    //=============================================================================
    var _Scene_Map_callMenu = Scene_Map.prototype.callMenu;
    Scene_Map.prototype.callMenu = function() {
        if ($gameSwitches.value(retryButtonShowSwitch)) {
            this.commandRetray();
        } else {
            _Scene_Map_callMenu.call(this);
        }
    };
    //=============================================================================
    // Sprite_Still
    //    立ち絵スプライトを定義します。
    //=============================================================================
    function Sprite_Still()
    {
        this.initialize.apply(this, arguments);
    }

    Sprite_Still.prototype = Object.create(Sprite_Base.prototype);
    Sprite_Still.prototype.constructor = Sprite_Still;

    Sprite_Still.prototype.initialize = function()
    {
        Sprite_Base.prototype.initialize.call(this);
        
        this.sprite1 = new Sprite();
        this.addChild(this.sprite1);
        this.effectSprite1 = new Sprite();
        this.addChild(this.effectSprite1);
        this.effectSprite2 = new Sprite();
        this.addChild(this.effectSprite2);
        this.eyeSprite = new Sprite();
        this.addChild(this.eyeSprite);
        
        this.mask = new Sprite();
        this.addChild(this.mask);
        
        this._baseOpacity = 255;
    };
    
    //=============================================================================
    // Window_MiniStatus
    //    ミニステータスウィンドウを定義します。
    //=============================================================================
    function Window_MiniStatus() {
        this.initialize.apply(this, arguments);
    }
    Window_MiniStatus.prototype = Object.create(Window_Base.prototype);
    Window_MiniStatus.prototype.constructor = Window_MiniStatus;
    Window_MiniStatus.prototype.initialize = function() {
        var width = windowWidth;
        var height = windowHeight;
        var x = Graphics.boxWidth - width;
        var y = windowY;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        this.hide();
        this._still = new Sprite_Still();
        this.addChild(this._still);
        
        this._maskGraphic = new PIXI.Graphics();
        this._maskGraphic.beginFill(0x000000);
        this._maskGraphic.drawRect(-100, 0, width+105, height+3);
        this._maskGraphic.endFill();

        this.addChild(this._maskGraphic);
        this.mask = this._maskGraphic;
        this._count           = 0;
        this._isTrans         = 0;
        this._isForceTrans    = 0;
        this._isBare          = 0;
        this._searchNum       = 0;
        this._backNum         = 0;
        this._transsSucsess   = 0;
        this._transDeps       = 0;
        this._isTrans         = 0;
        this._lastTransDeps   = 0;
        this._lastIsTrans     = 0;
        this._playSeFlag      = 0;
        this._hpRate          = 0;
        this._mpRate          = 0;
        this._tpRate          = 0;
        this._isPlayedPinchSe = false;
        this._setForceWindow  = false;
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.MiniStatus;
        
        if (Esrt.WindowMove.MiniStatus.hidden) {
            this.hidden = true;
        }
    };
    
    Window_MiniStatus.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_Other');
    };
    
    var _Window_MiniStatus_update = Window_MiniStatus.prototype.update;
    Window_MiniStatus.prototype.update = function() {
        _Window_MiniStatus_update.call(this);
        
        this._count++;
        if (!this.drawCheck()) {return;}
        this.setParameter();
        this.drawFace(0,0,144,144);
        this.drawEffect(0,0,144,144);
        if (this._isForceTrans && !this._setForceWindow) {
            this.setFourceWindowSkin();
        }
        if (this.isNeedPlayTransSe()) {
            this.playTransSe();
        }
        if (this.isNeedPlayPinchSe()) {
            this.playPinchSe();
        }
    }
    
    Window_MiniStatus.prototype.isNeedPlayTransSe = function() {
        var result = false;
        if (!(this._isTrans == this._lastIsTrans && this._transDeps == this._lastTransDeps)) {
            result = true;
        }
        
        this._lastIsTrans   = this._isTrans;
        this._lastTransDeps = this._transDeps;
        return result;
    };
    
    Window_MiniStatus.prototype.playTransSe = function() {
        if (!this._isTrans || this._transsSucsess)return;
        
        if (this._transDeps == 1) {
            AudioManager.playSe({"name":"Heal4","volume":100,"pitch":100,"pan":0});
        } else if (this._transDeps == 2) {
            AudioManager.playSe({"name":"Saint8","volume":100,"pitch":100,"pan":0});
        } else if (this._transDeps == 3) {
            AudioManager.playSe({"name":"Saint7","volume":100,"pitch":100,"pan":0});
        } else if (this._transDeps == 4) {
            AudioManager.playSe({"name":"Saint7","volume":100,"pitch":100,"pan":0});
        }
    }
    
    Window_MiniStatus.prototype.isNeedPlayPinchSe = function() {
        var result = false;
        if (this._isPlayedPinchSe) return result;
        if (this._hpRate == 0 && this._mpRate == 0 && this._tpRate == 0) return result;
        if (this._stillType == 6) return result;
        
        if (this._hpRate < 0.2) {
            result = true;
            this._isPlayedPinchSe = true;
        }
        if ($gameVariables.value(111) == 0) { // 変身ステージのとき、街の被害が大きいときにピンチ音
            if (this._tpRate > 0.8) {
                result = true;
                this._isPlayedPinchSe = true;
            }
        } else if ($gameVariables.value(91) != 4) { //ボスステージでレース以外の時、変身パワーが少ないときにピンチ音
            if (this._mpRate < 0.2) {
                result = true;
                this._isPlayedPinchSe = true;
            }
        }
        return result;
    };
    
    Window_MiniStatus.prototype.playPinchSe = function() {
        AudioManager.playSe({"name":"Pinch1","volume":100,"pitch":100,"pan":0});
    }
    
    Window_MiniStatus.prototype.drawCheck = function() {
        
        if (!$dataSystem) {
            return false;
        }
        if (this._count % 3 != 0) {
            return false;
        }
        
        if ($gameVariables.value(windowShowVariable) >= 1 && !this.hidden) {
            this.show();
            return true;
        } else {
            this.hide();
            return false;
        }
        
    };
    
    Window_MiniStatus.prototype.drawFace = function(x, y, width, height) {
        var width = 200;
        var y_offset = 130;
        var faceData = this.getFaceTypeAndNum();
        
        if (!faceData.faceNum) return;
        
        var fileIndex = faceData.faceType + "_" + ( '0000' + faceData.faceNum ).slice( -4 );
        
        var maskFileName = (faceData.isNeedMask) ? "kana_map_mask_" + fileIndex : "kana_map_mask_all";
        this._still.mask.bitmap = ImageManager.loadPictureMapStill(maskFileName);
        this._still.mask.x = -60;
        //this._still.mask.x = (maskNum == 4 ) ? 48 : this._still.mask.x;
        this._still.mask.y = y_offset + 10;
        this._still.mask.visible = true;
        
    	var fileName = "kana_map_still_";
        fileName += ( this.isNeedSafeStill(faceData) ) ? "safe_" : "";
        fileName += fileIndex;
        if ($gameVariables.value(112) > 0) { // 時間停止演出
            fileName += ($gameVariables.value(112) > 2) ? "_stop" : "_negaposi";
        }        
        this._still.sprite1.bitmap = ImageManager.loadPictureMapStill(fileName);
        this._still.sprite1.x = -60;
        //this._still.sprite1.x = (maskNum == 4 ) ? 48 : this._still.sprite1.x;
        this._still.sprite1.y = y_offset + 10;
        this._still.sprite1.visible = true;
        
        if (this._transsSucsess || this._isBare) {
            this._still.eyeSprite.visible = false;
            return;
        }
        
        var eyeFileName = (faceData.isVisibleEye) ? 'kana_map_still_eye_' + ( '0000' + faceData.faceNum ).slice( -4 ) : "kana_map_still_eye_0001";
        this._still.eyeSprite.bitmap = ImageManager.loadPictureMapStill(eyeFileName);
        this._still.eyeSprite.x = -60;
        this._still.eyeSprite.y = y_offset + 10;
    };
    
    Window_MiniStatus.prototype.isNeedSafeStill = function(faceData) {
        if ($gameVariables.value(82) == 0) return false;
        
        if (faceData.faceType == 1 && 
            (faceData.faceNum >= 3 && faceData.faceNum <= 5)) return true;
        if ((faceData.faceType == 41 || faceData.faceType == 42 || faceData.faceType == 45 ) && 
            faceData.faceNum == 31) return true;
        
        return false;
    }
    
    Window_MiniStatus.prototype.getFaceTypeAndNum = function() {
        var faceType = 1;
        var faceNum  = 1;
        var isNeedMask = false;
        var isVisibleEye = false;
        
        faceType = (this._stillType == 1) ? 11 : faceType; // 変身戦闘モード
        faceNum  = (this._stillType == 1) ? 2  : faceNum;  // 変身戦闘モード
        faceType = (this._stillType == 2) ? 21 : faceType; // 水着モード
        faceType = (this._stillType == 3) ? 31 : faceType; // 害虫駆除戦闘モード
        faceType = (this._stillType == 4) ? 11 : faceType; // 逃走戦闘モード
        faceType = (this._stillType == 5) ? 11 : faceType; // 対時間停止戦闘モード
        faceType = (this._stillType == 6) ? 51 : faceType; // 誘惑モード
        faceType = (this._stillType == 7) ? 11 : faceType; // 時間破壊モード
        faceType = (this._stillType == 8) ? 71 : faceType; // 最終戦闘モード
        faceType = (this._stillType == 9) ? 71 : faceType; // 最終戦闘モード
        
        if (this._isSafeMode) {
            faceType = (this._nakednessNum > 0) ? 41 : faceType; // 半裸モード
            faceType = (this._nakednessNum > 2) ? 42 : faceType; // 全裸モード
            faceType = (this._nakednessNum == 10) ? 45 : faceType; // 全裸モード
            
        } else {
            faceType = (this._nakednessNum > 0) ? 41 : faceType; // 半裸モード
            faceType = (this._nakednessNum > 2) ? 42 : faceType; // 全裸モード
            faceType = (this._nakednessNum == 10) ? 45 : faceType; // 全裸モード
        }
                
        if (this._isTrans == 0 && this._isForceTrans == 0) { // 変身マップ・変身していない時
            
            faceNum = (this._searchNum > 0) ? 11 : faceNum; // バレかけ
            faceNum = (this._searchNum == 0 && this._backNum > 0) ? 21 : faceNum; // バレかけ後人が遠ざかる
            
        } else { // 変身中
            isNeedMask = true;
            faceNum = this._transDeps + 1;
            isVisibleEye = (faceType == 1 && this._transDeps > 2) ? true : false;
        }
        
        isVisibleEye = (this._stillType == 2) ? false : isVisibleEye; // 水着モード
        isVisibleEye = (this._stillType == 6) ? false : isVisibleEye; // 誘惑モード
        isNeedMask   = (this._stillType == 6) ? false : isNeedMask; // 誘惑モード
        
        if (this._stillType == 9) { // 7-3の表情変化
            if (this._mpRate < 0.2) {
                faceNum = 31;
                
            } else if (this._mpRate < 0.6) {
                faceNum = 3;
                
            } else if (this._mpRate < 0.8) {
                faceNum = 2;
            }
        }
        
        if (this._isKnockBack) { // ダメージ中のスチル
            faceNum = (this._hpRate > 0.3) ? 51 : 61;
            isNeedMask = false;
            isVisibleEye = false;
        }
        
        if (this._isBare == 1) {
            faceNum    = 31; 
            isNeedMask = false;
            isVisibleEye = false;
        }
        
        if (this._isHP0 == 1) {
            faceNum    = 71;
            isNeedMask = false;
            isVisibleEye = false;
        }
        
        if (this._transsSucsess) { // 変身成功
            faceType = 11;
            faceNum = 1;
            isNeedMask = false;
            isVisibleEye = false;
            
            if (this._stillType == 9) { // ラストバトル
                faceType = 71;
                faceNum = 31;
            }
        }
        
        return {"faceType":faceType, "faceNum":faceNum, "isNeedMask":isNeedMask, "isVisibleEye":isVisibleEye};
    }
    
    Window_MiniStatus.prototype.drawEffect = function(x, y, width, height) {
        
        if (this._isTrans == 0 || this._isBare) {
            this._still.effectSprite1.visible = false;
            this._still.effectSprite2.visible = false;
            this._still.effectSprite1.blendMode = 0;
            this._still.effectSprite2.blendMode = 0;
            return;
        }
        
        var transName = (this._stillType != 6) ? 'trans_effect_1' : 'trans_effect_2';
        transName += '_000';
        var effectNum = parseInt((this._count / (TRANS_COUNT_CYCLE / 6)) % 3);
        effectNum = parseInt(effectNum * 2 + 1);
        this._still.effectSprite1.bitmap = ImageManager.loadPictureMapStill(transName + effectNum);
        this._still.effectSprite1.x = -100;
        this._still.effectSprite1.y = (this._count * -1) % TRANS_COUNT_CYCLE;
        this._still.effectSprite1.visible = true;
        this._still.effectSprite1.blendMode = 1;
        this._still.effectSprite1.opacity = CalcEffectOpacity(this._count);
        
        var effectNum2 = parseInt(((this._count + (TRANS_COUNT_CYCLE / 6 / 2)) / (TRANS_COUNT_CYCLE / 6)) % 2);
        effectNum2 = (effectNum2 + 1) * 2;
        this._still.effectSprite2.bitmap = ImageManager.loadPictureMapStill(transName + effectNum2);
        this._still.effectSprite2.x = -100;
        this._still.effectSprite2.y = (this._count * -2) % TRANS_COUNT_CYCLE;
        this._still.effectSprite2.visible = true;
        this._still.effectSprite2.blendMode = 1;
        this._still.effectSprite2.opacity = CalcEffectOpacity(this._count + (TRANS_COUNT_CYCLE / 6 / 2));
    };
    
	Window_MiniStatus.prototype.setParameter = function() {
        if (this._count % 12 != 0) {
            return false;
        }
        this._isTrans       = ($gameSwitches.value(101)) ? 1 : 0;
        this._isForceTrans  = ($gameSwitches.value(102)) ? 1 : 0;
        this._isBare        = ($gameSwitches.value(103)) ? 1 : 0;
        this._transsSucsess = ($gameSwitches.value(109)) ? 1 : 0;
        this._isKnockBack   = ($gameSwitches.value(118)) ? 1 : 0;
        this._isSafeMode    = $gameVariables.value(82);
        this._stillType     = $gameVariables.value(91);
        this._transDeps     = $gameVariables.value(101);
        this._searchNum     = $gameVariables.value(102);
        this._backNum       = $gameVariables.value(106);
        this._nakednessNum  = $gameVariables.value(119);
        this._isHP0         = $gameParty.members()[0].hp < 1 ? 1 : 0;
        this._hpRate        = $gameParty.members()[0].hpRate();
        this._mpRate        = $gameParty.members()[0].mpRate();
        this._tpRate        = $gameParty.members()[0].tpRate();
	};
    
    Window_MiniStatus.prototype.setFourceWindowSkin = function() {
        this.windowskin = ImageManager.loadSystem('Window_FourceTrance');
        this._setForceWindow = true;
        
        this._refreshAllParts();
    };
    
    // 変身時発光エフェクト切替時に透過度を低下
    function CalcEffectOpacity(value) {
        var opacity = 255;
        var opacitySep = 255 / 20;
        var effectCycle = TRANS_COUNT_CYCLE / 6;
        
        if (parseInt(value % effectCycle) < 20) {
            opacity = opacitySep * (value % effectCycle);
        } else if (parseInt(value % effectCycle) > effectCycle - 20) {
            opacity = 255 - opacitySep * ((value % effectCycle) - (effectCycle - 21));
        }
        return opacity;
    }
    
    //=============================================================================
    // Window_MiniStatusGauge
    //    ミニステータス用ゲージウィンドウを定義します。
    //=============================================================================
    function Window_MiniStatusGauge() {
        this.initialize.apply(this, arguments);
    }
    Window_MiniStatusGauge.prototype = Object.create(Window_Base.prototype);
    Window_MiniStatusGauge.prototype.constructor = Window_MiniStatusGauge;
    Window_MiniStatusGauge.prototype.initialize = function() {
        var width = windowWidth;
        var height = windowHeight;
        var x = Graphics.boxWidth - width;
        var y = windowY;
        this._count = 0;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this.hide();
        this.opacity = 0;
        this.gaugeHeight = 30;
        
        this._warningBitmap1 = ImageManager.loadPicture('warningMpGauge');
        this._warningBitmap2 = ImageManager.loadPicture('warningMpGaugeRed');
        this._warningHpGauge = new Sprite();
        this._warningHpGauge.x = 18;
        this._warningHpGauge.y = 0;
        this._warningHpGauge.bitmap = null;
        this._warningHpGauge.blendMode = 1;
        this._warningHpGauge.opacity = 0;
        this.addChild(this._warningHpGauge);
        this._warningMpGauge = new Sprite();
        this._warningMpGauge.x = 18;
        this._warningMpGauge.y = 0;
        this._warningMpGauge.bitmap = null;
        this._warningMpGauge.blendMode = 1;
        this._warningMpGauge.opacity = 0;
        this.addChild(this._warningMpGauge);
        this._warningTpGauge = new Sprite();
        this._warningTpGauge.x = 18;
        this._warningTpGauge.y = 0;
        this._warningTpGauge.bitmap = null;
        this._warningTpGauge.blendMode = 1;
        this._warningTpGauge.opacity = 0;
        this.addChild(this._warningTpGauge);
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.MiniStatusGauge;

    };
    
    var Window_MiniStatusGauge_update = Window_MiniStatusGauge.prototype.update;
    Window_MiniStatusGauge.prototype.update = function() {
        Window_MiniStatusGauge_update.call(this);
        
        this._count++;
        if (!this.drawCheck()) {return;}
        this.updateAlpha();
        this.drawData();
    }
    
    Window_MiniStatusGauge.prototype.drawCheck = function() {
        
        if (!$dataSystem) {
            return false;
        }
        if (this._count % 6 != 0) {
            return false;
        }
        
        if ($gameVariables.value(windowShowVariable) >= 1) {
            this.show();
            return true;
        } else {
            this.hide();
            return false;
        }
        
    };
    
    Window_MiniStatusGauge.prototype.updateAlpha = function() {
        this.alpha = (Esrt.WindowMove.MiniStatus.hidden) ? 0.5 : 1;
        
        return;
    };
    
    Window_MiniStatusGauge.prototype.drawData = function() {
        var actor = $gameParty.members()[0];
        var x = 0, y = 0, width = windowWidth　- 40;
        
        y = 0;
        if ($gameSwitches.value(261)) {
            this.drawHp(actor, x, y, width);
            this.drawWarningHp(actor, x, y, width);
            y += 40;
        }        
        if ($gameSwitches.value(262)) {
            this.drawMp(actor, x, y, width);
            this.drawWarningMp(actor, x, y, width);
            y += 40;
        }
        
        if ($gameSwitches.value(263)) {
            this.drawTp(actor, x, y, width);
            this.drawWarningTp(actor, x, y, width);
        }
    }
    
    Window_MiniStatusGauge.prototype.gaugeBackColor = function() {
        return this.textColor(7);
    };
    
    Window_MiniStatusGauge.prototype.drawHp = function(actor, x, y, width) {
        var color1 = this.hpGaugeColor2();
        var color2 = this.hpGaugeColor1();
        this.drawGauge(x, y, width, actor.hpRate(), color1, color2);
        this.drawText(TextManager.hpA, 0, y, 100);
    };
    
    Window_MiniStatusGauge.prototype.drawMp = function(actor, x, y, width) {
        var color1 = this.textColor(0);
        var color2 = this.textColor(2);
        this.drawGauge(x, y, width, actor.mpRate(), color1, color2);
        this.drawText(TextManager.mpA, x, y, 100);
    };
    
    Window_Base.prototype.drawTp = function(actor, x, y, width) {
        var color1 = this.textColor(2);
        var color2 = this.textColor(18);
        this.drawGauge(x, y, width, actor.tpRate(), color1, color2);
        this.drawText(TextManager.tpA, x, y, 100);
    };
    
    Window_MiniStatusGauge.prototype.drawWarningHp = function(actor, x, y, width) {
        this._warningHpGauge.y = y+35;
        this._warningHpGauge.bitmap = null;
        
        if (actor.hpRate() < 0.2) {
            this._warningHpGauge.bitmap = this._warningBitmap1;
            this._warningHpGauge.opacity = (this._count * 3) % 64 + 28;
        } else {
            this._warningHpGauge.opacity = 0;
        }
    };
    Window_MiniStatusGauge.prototype.drawWarningMp = function(actor, x, y, width) {
        this._warningMpGauge.y = y+35;
        this._warningMpGauge.bitmap = null;
        
        if ($gameVariables.value(111) > 0 && actor.mpRate() < 0.2) {
            this._warningMpGauge.bitmap = this._warningBitmap1;
            this._warningMpGauge.opacity = (this._count * 3) % 64 + 28
        } else {;
            this._warningMpGauge.opacity = 0;
        }
    };
    
    Window_MiniStatusGauge.prototype.drawWarningTp = function(actor, x, y, width) {
        this._warningTpGauge.y = y+35;
        this._warningTpGauge.bitmap = null;
        
        if (actor.tpRate() > 0.8) {
            this._warningTpGauge.bitmap = this._warningBitmap1;
            this._warningTpGauge.opacity = (this._count * 3) % 64 + 28;
        } else {
            this._warningTpGauge.opacity = 0;
        }
        if ($gameSwitches.value(forceTransSwitch)) {
            this._warningTpGauge.bitmap = this._warningBitmap2;
            this._warningTpGauge.opacity = (this._count * 3) % 64 + 28;
        }
    };
    
    Window_MiniStatusGauge.prototype.drawGauge = function(x, y, width, rate, color1, color2) {
        var fillW = Math.floor(width * rate);
        var gaugeY = y + this.lineHeight() - 20;
        this.contents.fillRect(x, gaugeY, width, this.gaugeHeight, this.gaugeBackColor());
        this.contents.gradientFillRect(x, gaugeY, fillW, this.gaugeHeight, color1, color2);
    };
    
    //=============================================================================
    // Window_ButtonEmphasis
    //    変身ボタンの強調を行います
    //=============================================================================
    function Window_ButtonEmphasis() {
        this.initialize.apply(this, arguments);
    }
    Window_ButtonEmphasis.prototype = Object.create(Window_Base.prototype);
    Window_ButtonEmphasis.prototype.constructor = Window_MiniStatus;
    Window_ButtonEmphasis.prototype.initialize = function(x, y, width, height) {
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this.opacity = 0;
        this._count = 0;
        
        this._still = new Sprite(ImageManager.loadPictureButton("button_emphasis"));
        this._still.anchor.x = 0.5;
        this._still.anchor.y = 0.5;
        this._still.scale.x = 1;
        this._still.scale.y = 1;
        this._still.opacity = 0;
        this.addChild(this._still);
    };
    
    Window_ButtonEmphasis.prototype.update = function() {
        Window_Base.prototype.update.call(this);
        
        if (!$gameSwitches.value(260)) { // 指定したステージのみ強調
            return;
        }
        if (!$gameVariables.value(windowShowVariable)) {
            this._still.opacity = 0;
            return;
        }
        
        this._count++;
        if (this._count % 2 != 0) {
            return false;
        }
        
        this._still.scale.x += 0.1;
        this._still.scale.y += 0.1;
        this._still.opacity -= 10;
        
        
        if (this._count % 100 == 0) {
            this._still.scale.x = 1;
            this._still.scale.y = 1;
            this._still.opacity = 255;
            
        };
    }
    //=============================================================================
    // Window_LeftMiniStatus
    //    ミニステータスウィンドウを定義します。
    //=============================================================================
    function Window_LeftMiniStatus() {
        this.initialize.apply(this, arguments);
    }
    Window_LeftMiniStatus.prototype = Object.create(Window_Base.prototype);
    Window_LeftMiniStatus.prototype.constructor = Window_LeftMiniStatus;
    Window_LeftMiniStatus.prototype.initialize = function() {
        var width = windowWidth;
        var height = windowHeight;
        var x = 0;
        var y = windowY;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        this.hide();
        this._still = new Sprite_Still();
        this.addChild(this._still);
        
        this._maskGraphic = new PIXI.Graphics();
        this._maskGraphic.beginFill(0x000000);
        this._maskGraphic.drawRect(-100, 0, width+105, height+3);
        this._maskGraphic.endFill();

        this.addChild(this._maskGraphic);
        this.mask = this._maskGraphic;
        this._count           = 0;
        this._isTrans         = 0;
        this._transsSucsess   = 0;
        this._hpRate          = 0;
        this._mpRate          = 0;
        this._tpRate          = 0;
        
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.MiniStatus;
    };
    
    Window_LeftMiniStatus.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_Other');
    };
    
    var Window_LeftMiniStatus_update = Window_LeftMiniStatus.prototype.update;
    Window_LeftMiniStatus.prototype.update = function() {
        Window_LeftMiniStatus_update.call(this);
        
        this._count++;
        if (!this.drawCheck()) {return;}
        this.setParameter();
        this.drawFace(0,0,144,144);
    }
    
    Window_LeftMiniStatus.prototype.drawCheck = function() {
        
        if (!$dataSystem) {
            return false;
        }
        if (this._count % 3 != 0) {
            return false;
        }
        
        if ($gameVariables.value(windowShowVariable) >= 3) {
            this.show();
            return true;
        } else {
            this.hide();
            return false;
        }
        
    };
    
    Window_LeftMiniStatus.prototype.drawFace = function(x, y, width, height) {
        var width = 200;
        var y_offset = 130;
        var faceData = this.getFaceTypeAndNum();
        
        if (!faceData.faceNum) return;
        
        var fileIndex = faceData.faceType + "_" + ( '0000' + faceData.faceNum ).slice( -4 );
        
        var maskFileName = "kana_map_mask_all";
        this._still.mask.bitmap = ImageManager.loadPictureMapStill(maskFileName);
        this._still.mask.x = -60;
        this._still.mask.y = y_offset + 10;
        this._still.mask.visible = true;
        
    	var fileName = "porin_map_still_" + fileIndex;
        this._still.sprite1.bitmap = ImageManager.loadPictureMapStill(fileName);
        this._still.sprite1.x = -60;
        this._still.sprite1.y = y_offset + 10;
        this._still.sprite1.visible = true;
    };
    
    Window_LeftMiniStatus.prototype.getFaceTypeAndNum = function() {
        var faceType = 1;
        var faceNum  = 1;
        
        if (this._mpRate < 0.2) {
            faceNum = 31;
            
        } else if (this._mpRate < 0.6) {
            faceNum = 3;
            
        } else if (this._mpRate < 0.8) {
            faceNum = 2;
        }
        
        if (this._transsSucsess) { // 変身成功
            faceNum = 1;
        }
        
        return {"faceType":faceType, "faceNum":faceNum};
    }
    
	Window_LeftMiniStatus.prototype.setParameter = function() {
        if (this._count % 12 != 0) {
            return false;
        }
        this._isTrans       = ($gameSwitches.value(101)) ? 1 : 0;
        this._transsSucsess = ($gameSwitches.value(109)) ? 1 : 0;
        this._stillType     = $gameVariables.value(91);
        this._isHP0         = $gameParty.members()[0].hp < 1 ? 1 : 0;
        this._hpRate        = $gameParty.members()[0].hpRate();
        this._mpRate        = $gameParty.members()[0].mpRate();
        this._tpRate        = $gameParty.members()[0].tpRate();
	};
})();
