//=============================================================================
// ESRT_MessagePlugins_override.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

//=============================================================================
// Copyright (c) 2015 Triacontane
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ※ MessageSkip.js及びMessageWindowHidden.jsの独自修正を行うにあたって、
// 　トリアコンタン様のパラメータ取得用関数を利用させていただいています。
// ----------------------------------------------------------------------------
// [Blog]   : http://triacontane.blogspot.jp/
// [Twitter]: https://twitter.com/triacontane/
// [GitHub] : https://github.com/triacontane/
//=============================================================================

/*:
 * @plugindesc Other Pluguin Override
 * @author white_esorat
 *
 * @param MenuStatusSwitch
 * @desc 
 * @default 216
 *
 * @param MenuShowSwitch
 * @desc 
 * @default 211
 *
 * @param SkipTouchSwitch
 * @desc 
 * @default 221
 *
 * @param AutoTouchSwitch
 * @desc 
 * @default 222
 *
 * @param HiddenTouchSwitch
 * @desc 
 * @default 223
 *
 * @param BackLogTouchSwitch
 * @desc 
 * @default 224
 *
 * @param BackLogUpTouchSwitch
 * @desc 
 * @default 225
 *
 * @param BackLogDownTouchSwitch
 * @desc 
 * @default 226
 *
 * @param HiddenTriggerButton
 * @desc Hidden Trigger button
 * (light_click or shift or control)
 * @default light_click
 *
 * @help 
 *
 */
/*:ja
 * @plugindesc メッセージに関する他プラグイン書き換え用独自プラグイン
 * @author 白
 *
 * @param メニュー開閉状態保存用スイッチ
 * @desc  メニュー開閉状態を保存するスイッチ番号
 * @default 216
 *
 * @param テキストメニュー表示スイッチ
 * @desc  メニュー開閉に該当するスイッチ番号
 * @default 211
 *
 * @param スキップ押下スイッチ
 * @desc メッセージスキップに該当するスイッチ番号
 * @default 221
 *
 * @param オート押下スイッチ
 * @desc メッセージオートに該当するスイッチ番号
 * @default 222
 *
 * @param 一時消去押下スイッチ
 * @desc メッセージ一時消去に該当するスイッチ番号
 * @default 223
 *
 * @param バックログ押下スイッチ
 * @desc バックログ表示に該当するスイッチ番号
 * @default 224
 *
 * @param バックログアップ押下スイッチ
 * @desc バックログ表示に該当するスイッチ番号
 * @default 225
 *
 * @param バックログダウン押下スイッチ
 * @desc バックログ表示に該当するスイッチ番号
 * @default 226
 *
 * @param 瞬間スキップ押下スイッチ
 * @desc メッセージ瞬間スキップに該当するスイッチ番号
 * @default 227
 *
 * @param 一時消去ボタン名称
 * @desc ウィンドウを消去するボタンです。
 * (右クリック or shift or control)
 * ※このプラグインがオンの間、MessageWindowHidden.jsの「ボタン名称」の値は無効になります。
 * @default 右クリック
 * 
 * @help 
 *
 */

var Esrt = Esrt || {};
Esrt.Message = Esrt.Message || {};
Esrt.Message._controllVisible = false;
Esrt.Message._choiceWindowBusy = false;
Esrt.Message._isClickLeftKeyForOption =  Esrt.Message._isClickLeftKeyForOption || false;

(function() {
    'use strict';
	var pluginName    = 'ESRT_MessagePlugins_override';
    
    Input.keyMapper[18] = '';
    Input.gamepadMapper[8] = 'messageSkip';
    Input.gamepadMapper[9] = 'messageAuto';
    Input.gamepadMapper[10] = 'messageAllSkip';

    //=============================================================================
    // トリアコンタン様製プラグイン上書き用　
    //   トリアコンタン様のプラグインで宣言されているパラメータ取得用関数
    //=============================================================================
    var getParamNumber = function(paramNames, min, max) {
        var value = getParamOther(paramNames);
        if (arguments.length < 2) min = -Infinity;
        if (arguments.length < 3) max = Infinity;
        return (parseInt(value, 10) || 0).clamp(min, max);
    };

    var getParamString = function(paramNames) {
        var value = getParamOther(paramNames);
        return value === null ? '' : value;
    };
    
    var getParamBoolean = function(paramNames) {
        var value = getParamOther(paramNames);
        return (value || '').toUpperCase() === 'ON';
    };

    var getParamOther = function(paramNames) {
        if (!Array.isArray(paramNames)) paramNames = [paramNames];
        for (var i = 0; i < paramNames.length; i++) {
            var name = PluginManager.parameters(pluginName)[paramNames[i]];
            if (name) return name;
        }
        return null;
    };
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var menuShowSwitch     = getParamNumber(['MenuSwitch', 'テキストメニュー表示スイッチ'], 0);
    var menuTouchSwitch    = menuShowSwitch + 1;
    var menuAllowSwitch    = menuShowSwitch + 6;
    var menuStatusSwitch   = getParamNumber(['MenuStatusSwitch', 'メニュー開閉状態保存用スイッチ'], 0);
    var skipTouchSwitch     = getParamNumber(['SkipTouchSwitch', 'スキップ押下スイッチ'], 0);
    var autoTouchSwitch     = getParamNumber(['AutoTouchSwitch', 'オート押下スイッチ'], 0);
    var hiddenTouchSwitch   = getParamNumber(['HiddenTouchSwitch', '一時消去押下スイッチ'], 0);
    var backLogTouchSwitch  = getParamNumber(['backLogTouchSwitch', 'バックログ押下スイッチ'], 0);
    var backLogUpTouchSwitch  = getParamNumber(['backLogUpTouchSwitch', 'バックログアップ押下スイッチ'], 0);
    var backLogDownTouchSwitch  = getParamNumber(['backLogDownTouchSwitch', 'バックログダウン押下スイッチ'], 0);
    var allSkipTouchSwitch     = getParamNumber(['SkipTouchSwitch', '瞬間スキップ押下スイッチ'], 0);
    var systemMenuButtonShowSwitch   = 201;
    
    //=============================================================================
    // Window_Message
    //  メッセージ操作用変数初期化
    //=============================================================================
	
    var _Window_Message_initialize = Window_Message.prototype.initialize;
    Window_Message.prototype.initialize = function() {
        _Window_Message_initialize.call(this);
        Esrt.Message._menuButtonVisible       = false;
        Esrt.Message._menuShowSwitch          = false;
        Esrt.Message._backLogSubButtonVisible = false;
    };
    
    //=============================================================================
    // PLUGIN OVERRIDE
    //  適用プラグインの処理を上書き
    //=============================================================================
    //=============================================================================
    // MessageSkip.js
    //  ・スキップ中にオートにした場合、スキップを解除してオートにします。
    //  ・指定されたスイッチがオンのとき、スキップ・オートキーが押されたときと同じ動作をします。
    //  　その際、スイッチはオフにします。(コモンピクチャとの連携)
    //　・MessageWindowHidden.jsと組み合わせた際、
    //    スキップ中に一時消去するとスキップ状態を解除するようにします。
    //=============================================================================
    
    Game_Message.prototype.toggleAuto = function() {
        this.setAutoFlg(!this._autoFlg)
        if (this._autoFlg) this._skipFlg = false;
    };
    
    var _Window_Message_update = Window_Message.prototype.update;
    Window_Message.prototype.update = function() {
        if (!this.visible) {
            $gameMessage.setSkipFlg(false);
            $gameMessage.setAutoFlg(false);
        }
        return _Window_Message_update.apply(this, arguments);
    };
    
    var Window_Message_isTriggeredMessageSkip = Window_Message.prototype.isTriggeredMessageSkip;
    Window_Message.prototype.isTriggeredMessageSkip = function() {
        var result = Window_Message_isTriggeredMessageSkip.call(this);
        var switch_value = $gameSwitches.value(skipTouchSwitch);
        
        $gameSwitches.setValue(skipTouchSwitch, false);
        
        return result || switch_value;
    };
    
    var Window_Message_isPressedMessageSkip = Window_Message.prototype.isPressedMessageSkip;
    Window_Message.prototype.isPressedMessageSkip = function() {
        var result = Window_Message_isPressedMessageSkip.call(this);
        var switch_value = $gameSwitches.value(skipTouchSwitch);
                
        return result || switch_value;
    };
    
    var Window_Message_isTriggeredMessageAuto = Window_Message.prototype.isTriggeredMessageAuto;
    Window_Message.prototype.isTriggeredMessageAuto = function() {
        var result = Window_Message_isTriggeredMessageAuto.call(this);
        var switch_value = $gameSwitches.value(autoTouchSwitch);
        $gameSwitches.setValue(autoTouchSwitch, false);
        return result || switch_value;
    };
    
    var Window_Message_isTriggeredAllMessageSkip = Window_Message.prototype.isTriggeredAllMessageSkip;
    Window_Message.prototype.isTriggeredAllMessageSkip = function() {
        if(!$gameSwitches.value(131)) {return false};
        
        var result = Window_Message_isTriggeredAllMessageSkip.call(this);
        var switch_value = $gameSwitches.value(allSkipTouchSwitch);
        
        $gameSwitches.setValue(allSkipTouchSwitch, false);
        
        return result || switch_value;
    };
    
    //=============================================================================
    // MessageWindowHidden.js
    //  ・ウィンドウ再表示は決定キーでも行えるようにします。
    //  ・指定されたスイッチがオンのとき、一時消去キーが押されたときと同じ動作をします。
    //=============================================================================
    Window_Message.prototype.isTriggeredHidden = function() {
        var switch_value = $gameSwitches.value(hiddenTouchSwitch);
        $gameSwitches.setValue(hiddenTouchSwitch, false);
        if(this.visible){
            var buttonName = getParamString(['一時消去ボタン名称','HiddenTriggerButton']).toLowerCase();
            switch (buttonName) {
                case '':
                case '右クリック':
                case 'light_click':
                    return TouchInput.isCancelled() || switch_value;
                case 'ok':
                    return false;
                default:
                    return Input.isTriggered(buttonName) || switch_value;
            }
        }else{
            var buttonName = getParamString(['一時消去ボタン名称','HiddenTriggerButton']).toLowerCase();
            switch (buttonName) {
                case '':
                case '右クリック':
                case 'light_click':
                    return TouchInput.isCancelled() || Input.isTriggered('ok') || switch_value;
                case 'ok':
                    return false;
                default:
                    return Input.isTriggered(buttonName) || Input.isTriggered('ok') || switch_value;
            }
        }
    };
    // ウィンドウ一時消去時に、メッセージ操作用ボタンも一時消去する。
    // ピクチャのタッチ状態からのコモンイベント呼び出し予約を追加定義します。
    var _Window_Message_updateWait = Window_Message.prototype.updateWait;
    Window_Message.prototype.updateWait = function() {
                
        if(TouchInput.isTriggered()){
            Esrt.setMessageControllVisible(true);
        }
        
        return _Window_Message_updateWait.call(this);
    };
    //=============================================================================
    //  ・ウィンドウ表示時にメッセージ操作ボタンを表示します。
    //=============================================================================
    var Game_Interpreter_initialize_message_button = Game_Interpreter.prototype.initialize;
    Game_Interpreter.prototype.initialize = function(depth) {
        Game_Interpreter_initialize_message_button.apply(this, arguments);
        
        this._messageMenuButtonShow = false;
    };
    
    var Game_Interpreter_command101_message_button = Game_Interpreter.prototype.command101;
    Game_Interpreter.prototype.command101 = function() {
        if (!$gameMessage.isBusy()) {
            this.setMessageMenu(true);
        }
        
        Game_Interpreter_command101_message_button.call(this);
        return false;
    };
    
    var Game_Interpreter_setupChoices_message_button = Game_Interpreter.prototype.setupChoices;
    Game_Interpreter.prototype.setupChoices = function(params) {
        Game_Interpreter_setupChoices_message_button.apply(this, arguments);
        
        this.setMessageMenu(false);
    };
    
    var Game_Interpreter_setupNumInput_message_button = Game_Interpreter.prototype.setupNumInput;
    Game_Interpreter.prototype.setupNumInput = function(params) {
        Game_Interpreter_setupNumInput_message_button.apply(this, arguments);
        
        this.setMessageMenu(false);
    };
    
    var Game_Interpreter_setupItemChoice_message_button = Game_Interpreter.prototype.setupItemChoice;
    Game_Interpreter.prototype.setupItemChoice = function(params) {
        Game_Interpreter_setupItemChoice_message_button.apply(this, arguments);
        
        this.setMessageMenu(false);
    };
    
    // Show Scrolling Text
    var Game_Interpreter_command105_message_button = Game_Interpreter.prototype.command105;
    Game_Interpreter.prototype.command105 = function() {
        if (!$gameMessage.isBusy()) {
            this.setMessageMenu(false);
        }
        
        Game_Interpreter_command105_message_button.call(this);
        return false;
    };
    
    // イベント終了時にメッセージ操作ボタンを消去し、システムメニューを表示する
    var Game_Interpreter_terminate_message_button = Game_Interpreter.prototype.terminate;
    Game_Interpreter.prototype.terminate = function() {
        Game_Interpreter_terminate_message_button.call(this);
        this._list = null;
        this._comments = '';
        this.setMessageMenu(false);
    };
    
    Game_Interpreter.prototype.setMessageMenu = function(flag) {
        if(flag){
            if($gameSwitches.value(menuAllowSwitch) && !this._messageMenuButtonShow){
                $gameSwitches.setValue(menuShowSwitch, true);
                $gameSwitches.setValue(systemMenuButtonShowSwitch, false);
                Esrt.Message._menuButtonVisible = true;
                this._messageMenuButtonShow = true;
                Esrt.Message._controllVisible = true;
            }
            
        }else{
            if(this._messageMenuButtonShow){
                $gameSwitches.setValue(menuShowSwitch, false);
                $gameSwitches.setValue(systemMenuButtonShowSwitch, true);
                this._messageMenuButtonShow = false;
                Esrt.Message._controllVisible = false;
            }
        }
    };
    
    //=============================================================================
    // ponidog_BackLog_utf8.js
    //  ・メッセージ操作ボタンとの連携を追記
    //=============================================================================
    var _Window_Message_updateInput_button = Window_Message.prototype.updateInput;
    Window_Message.prototype.updateInput = function() {
        if(typeof oBackLog != "undefined" && oBackLog._BackLogBusy){
            this.setBackLogSubButtonVisible(true);
            
            // スプライトボタンで遡る
            if ($gameSwitches.value(backLogUpTouchSwitch) ){
                    Input.update();
                    oBackLog._nBackLogPage++;
                    oBackLog.BackLog_Call();
                    $gameSwitches.setValue(backLogUpTouchSwitch, false);
                    return true;
            }
            
            // スプライトボタンで進める
            if ($gameSwitches.value(backLogDownTouchSwitch) ){
                    Input.update();   
                    oBackLog._nBackLogPage--;
                    oBackLog.BackLog_Call();
                    $gameSwitches.setValue(backLogDownTouchSwitch, false);
                    
                    //ページマイナスで閉じる。
                    if(oBackLog._nBackLogPage <= 0){
                         oBackLog._BackLogBusy=false;
                         oBackLog._nBackLogPage=0;
                         Input.clear();
                         this.pause = false;
                         //メッセージウィンドウを表示させる。
                         this.show();
                         this.subWindows().forEach(function(subWindow) { subWindow.show();});
                    }//
                    return true;
            }
        }else{
            this.setBackLogSubButtonVisible(false);
        }
        if (this.pause) {
            if ($gameSwitches.value(backLogTouchSwitch) && !oBackLog._BackLogBusy || TouchInput._wheelY <0 && !oBackLog._BackLogBusy) {
                Input.update();         
                oBackLog._nBackLogPage = 1;
                oBackLog.BackLog_Call();
                $gameSwitches.setValue(backLogTouchSwitch, false);
                return true;
            }
        }
        
        return _Window_Message_updateInput_button.call(this);
    }
    
    Window_Message.prototype.setBackLogSubButtonVisible = function(flag){
        Esrt.Message._backLogSubButtonVisible = flag;
        return;
    }
    
    //=============================================================================
    // メッセージ操作ウィンドウを表示します
    //=============================================================================
    var _Scene_Map_createAllWindows = Scene_Map.prototype.createAllWindows;
    Scene_Map.prototype.createAllWindows = function() {
        _Scene_Map_createAllWindows.call(this);
        
        this.createMessageControllWindow();
    };
    
    Scene_Map.prototype.createMessageControllWindow = function() {        
        this._messageControllWindow = new Window_MessageControll(828, 50, 200, 360);
        this.addWindow(this._messageControllWindow);
        this._messageControllWindow.hide();
        this._backLogControllWindow = new Window_BackLogControll(730, 130, 190, 50);
        this.addWindow(this._backLogControllWindow);
        this._backLogControllWindow.opacity = 0;
        this._backLogControllWindow.hide();
        this._optionControllWindow = new Window_OptionControll(0, 565, 190, 210);
        this.addWindow(this._optionControllWindow);
        this._optionControllWindow.opacity = 0;
        this._optionControllWindow.hide();
    };
    
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        this._backLogControllWindow.updateTouch(); // suppressEventsを効かせるために先に判定する
        this._messageControllWindow.updateTouch();
        this._optionControllWindow.updateTouch();
        
        _Scene_Map_update.apply(this, arguments);
        
        this.CheckandChangeMessageControllVisible();
        this.CheckandChangeBackLogControllVisible();
        
        if(!$dataSystem) return;
    };
    
    Scene_Map.prototype.CheckandChangeMessageControllVisible = function() {
        if(this.isShowMessageControll()){
            this._messageControllWindow.show();
            this._messageControllWindow.open();
            this._optionControllWindow.show();
            this._optionControllWindow.open();
        }else{
            this._messageControllWindow.hide();
            this._messageControllWindow.close();
            this._optionControllWindow.hide();
            this._optionControllWindow.close();
        }
    };
    
    Scene_Map.prototype.isShowMessageControll = function() {
        return $gameSwitches.value(menuAllowSwitch) && (typeof oBackLog != "undefined" && !oBackLog._BackLogBusy) && Esrt.Message._controllVisible && !Esrt.Message._choiceWindowBusy;
    }
    
    Scene_Map.prototype.CheckandChangeBackLogControllVisible = function() {
        if(typeof oBackLog != "undefined" && oBackLog._BackLogBusy){
            this._backLogControllWindow.show();
            this._backLogControllWindow.open();
        }else{
            this._backLogControllWindow.hide();
            this._backLogControllWindow.close();
        }
    };
    
    //=============================================================================
    // Window_ChoiceList
    //  ・選択肢表示中はオプションボタン及びメッセージ操作ボタンを表示しないようにする
    //  ・インスタンス作成時に非表示フラグをtrueにする
    //  ・決定時に非表示フラグをfalseにする
    //=============================================================================
    var _Window_ChoiceList_start = Window_ChoiceList.prototype.start;
    Window_ChoiceList.prototype.start = function() {
    	_Window_ChoiceList_start.call(this);
        Esrt.Message._choiceWindowBusy = true;
    };
    
    var _Window_ChoiceList_processOk = Window_ChoiceList.prototype.processOk;
    Window_ChoiceList.prototype.processOk = function() {
    	_Window_ChoiceList_processOk.call(this);
        Esrt.Message._choiceWindowBusy = false;
        
    }
    //=============================================================================
    // Window_MessageControll
    //  メッセージボタン操作用ウィンドウです。
    //=============================================================================
    
    function Window_MessageControll() {
        this.initialize.apply(this, arguments);
    }
    var _Window_Base_loadWindowskin = Window_Base.prototype.loadWindowskin;
    Window_Base.prototype.loadWindowskin = function() {
        // Window_Baseで呼び出しておかないと、初回起動時にロード遅延で表示がおかしくなる対策
        this.windowskin = ImageManager.loadSystem('Window_MessageControl');
        _Window_Base_loadWindowskin.call(this);        
    };
    
    Window_MessageControll.prototype = Object.create(Window_ButtonSprites.prototype);
    Window_MessageControll.prototype.constructor = Window_MessageControll;
    
    Window_MessageControll.prototype.initialize = function(x, y, width, height) {
        Window_ButtonSprites.prototype.initialize.call(this, x, y, width, height);
        
        this.windowskin = ImageManager.loadSystem('Window_MessageControl');   
        this._refreshAllParts();
        
        this.baseX = x;
        this._baseControllButtonX = -190;
        this._leftKeyInterval = 0;
        this._rightKeyInterval = 0;
        
        var spriteX = 0;
        var spriteY = 0;
        this.createSpriteButton("_skipButton", "button_rapid", spriteX, spriteY);
        spriteY += 80;
        this.createSpriteButton("_autoButton", "button_auto", spriteX, spriteY);
        spriteY += 80;
        var skip_button_name = ($gameSwitches.value(131)) ? "button_skip": "button_skip_no"
        this.createSpriteButton("_allSkipButton", skip_button_name, spriteX, spriteY);
        spriteY += 80;
        this.createSpriteButton("_hiddenButton", "button_hidden", spriteX, spriteY);
        spriteY += 80;
        this.createSpriteButton("_backLogButton", "button_backlog", spriteX, spriteY);
        spriteX = this._baseControllButtonX;
        spriteY += 195;
        this.createSpriteButton("_messageControllButton", "button_message", spriteX, spriteY);
        
        this._hoverCheckSpritelist.push("_skipButton");
        this._hoverCheckSpritelist.push("_autoButton");
        if($gameSwitches.value(131)) { this._hoverCheckSpritelist.push("_allSkipButton"); }
        this._hoverCheckSpritelist.push("_hiddenButton");
        this._hoverCheckSpritelist.push("_backLogButton");
        this._hoverCheckSpritelist.push("_messageControllButton");
        this._touchCheckSpritelist.push({button:"_skipButton",            switch:skipTouchSwitch});
        this._touchCheckSpritelist.push({button:"_autoButton",            switch:autoTouchSwitch});
        if($gameSwitches.value(131)) { this._touchCheckSpritelist.push({button:"_allSkipButton",         switch:allSkipTouchSwitch}); }
        this._touchCheckSpritelist.push({button:"_hiddenButton",          switch:hiddenTouchSwitch});
        this._touchCheckSpritelist.push({button:"_backLogButton",         switch:backLogTouchSwitch});
        this._touchCheckSpritelist.push({button:"_messageControllButton", switch:menuTouchSwitch});
        
    };
    
    Window_MessageControll.prototype.update = function() {
        Window_ButtonSprites.prototype.update.apply(this, arguments);
        
        this.updateRightKeyTriggerd();
        this.updatePullState();
    };

    // スプライトのタッチ状態を取得
    Window_MessageControll.prototype.updateTouch = function() {
        if(!this.visible) return;
        
        Window_ButtonSprites.prototype.updateTouch.apply(this, arguments);
        
        // スキップのみタッチ解除でスイッチも解除する
        if(this._skipButton && !this.isTriggeredButton(this._skipButton) && !this.isPressedButton(this._skipButton)){
            $gameSwitches.setValue(skipTouchSwitch, false);
            return;
        }
    };
    
    // メッセージ操作ウィンドウの位置移動
    Window_MessageControll.prototype.updatePullState = function() {
        if(!this.visible) return;
        var pullSpeed = 10;
        
        if ($gameSwitches.value(menuStatusSwitch)) {
            this._messageControllButton.x += (this.baseX - this.width) < this.x ? pullSpeed : 0;
            this.x -= (this.baseX - this.width) < this.x ? pullSpeed : 0;
            this._messageControllButton.x =  (this.baseX - this.width) > this.x ? this._baseControllButtonX + this.width : this._messageControllButton.x;
            this.x =  (this.baseX - this.width) > this.x ? this.baseX - this.width : this.x;
            
        }else{
            this._messageControllButton.x -= this.baseX > this.x ? pullSpeed : 0;
            this.x += this.baseX > this.x ? pullSpeed : 0;
            this._messageControllButton.x =  this.baseX < this.x ? this._baseControllButtonX : this._messageControllButton.x;
            this.x =  this.baseX < this.x ? this.baseX : this.x;
        }
        
        return;
    };
    
    // メッセージ操作ウィンドウの全表示・一部表示切り替え
    Window_MessageControll.prototype.updateRightKeyTriggerd = function() {
        if(this._rightKeyInterval > 0){ // 1クリックで複数呼び出されることに対処
            this._rightKeyInterval -= 1;
            return;
        }
        
        if (this.visible && (Input.isTriggered('right') || $gameSwitches.value(menuTouchSwitch))) {
            if($gameSwitches.value(menuStatusSwitch)){
                $gameSwitches.setValue(menuStatusSwitch, false);
            }else{
                $gameSwitches.setValue(menuStatusSwitch, true);
            }
            $gameSwitches.setValue(menuTouchSwitch, false);
            this._rightKeyInterval = 15;
        }
        
        return;
    };
    
    //=============================================================================
    // Window_OptionControll
    //  オプション表示ボタン用ウィンドウです。
    //=============================================================================
    
    function Window_OptionControll() {
        this.initialize.apply(this, arguments);
    }
    
    Window_OptionControll.prototype = Object.create(Window_ButtonSprites.prototype);
    Window_OptionControll.prototype.constructor = Window_OptionControll;
    
    Window_OptionControll.prototype.initialize = function(x, y, width, height) {
        Window_ButtonSprites.prototype.initialize.call(this, x, y, width, height);
        
        this._refreshAllParts();
        
        this.baseX = x;
        this._leftKeyInterval = 0;
        this._rightKeyInterval = 0;
        
        var spriteX = 0;
        var spriteY = 0;
        this.createSpriteButton("_optionButton", "button_option", spriteX, spriteY);
        this._hoverCheckSpritelist.push("_optionButton");
    };
    
    Window_OptionControll.prototype.update = function() {
        Window_ButtonSprites.prototype.update.apply(this, arguments);
        
        this.updateLeftKeyTriggerd();
    };

    // スプライトのタッチ状態を取得
    Window_OptionControll.prototype.updateTouch = function() {
        if(!this.visible) return;
        
        if(this._optionButton && this.isTriggeredButton(this._optionButton)){
            this.commandOptionButton();
            TouchInput.suppressEvents();
            return;
        }
    };
    
    Window_OptionControll.prototype.commandOptionButton = function() {
        SceneManager.push(Scene_Options);
        this._leftKeyInterval = 15;
        Esrt.Message._isClickLeftKeyForOption = true;
    };
    
    // キー入力によるオプションの表示
    Window_OptionControll.prototype.updateLeftKeyTriggerd = function() {
        if(this._leftKeyInterval > 0){ // 1クリックで複数呼び出されることに対処
            this._leftKeyInterval -= 1;
            return;
        }
        
        if (this.visible && Input.isTriggered('left')) {
            SceneManager.push(Scene_Options);
            this._leftKeyInterval = 15;
            Esrt.Message._isClickLeftKeyForOption = true;
        }
        
        return;
    };
    //=============================================================================
    // Window_BackLogControll
    //  メッセージボタン操作用ウィンドウです。
    //=============================================================================
    
    function Window_BackLogControll() {
        this.initialize.apply(this, arguments);
    }
    
    Window_BackLogControll.prototype = Object.create(Window_ButtonSprites.prototype);
    Window_BackLogControll.prototype.constructor = Window_MessageControll;
    
    Window_BackLogControll.prototype.initialize = function(x, y, width, height) {
        Window_ButtonSprites.prototype.initialize.call(this, x, y, width, height);
        
        this.baseX = x;
        this._rightKeyInterval = 0;
        this._hoverCheckCount = 0;
        
        var spriteX = 0;
        var spriteY = 0;
        this.createSpriteButton("_pageupButton", "button_pageup", spriteX, spriteY);
        spriteY += 160;
        this.createSpriteButton("_pagedownButton", "button_pagedown", spriteX, spriteY);
        
        this._hoverCheckSpritelist.push("_pageupButton");
        this._hoverCheckSpritelist.push("_pagedownButton");
        this._touchCheckSpritelist.push({button:"_pageupButton",   switch:backLogUpTouchSwitch});
        this._touchCheckSpritelist.push({button:"_pagedownButton", switch:backLogDownTouchSwitch});
        
    };
    
    Window_BackLogControll.prototype.update = function() {
        Window_ButtonSprites.prototype.update.apply(this, arguments);
        
    };
    
    //=============================================================================
    //  メッセージ操作ウィンドウの表示のオンオフを切り替える
    //=============================================================================
    Esrt.setMessageControllVisible = function(flag) {
        Esrt.Message._controllVisible = flag;
    };
})();
