//=============================================================================
// ESRT_MessageRecollectionAllSkip.js
// ----------------------------------------------------------------------------
// Copyright (c) 2020 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:ja
 * @plugindesc 回想時完全スキッププラグイン
 * @author 白
 *
 * @help 
 *
 * This plugin is released under the MIT License.
 *
 * 参考：
 * ・トリアコンタン 様
 * 　・メッセージスキッププラグイン - http://triacontane.blogspot.jp/
 *
 */
 
(function() {
    'use strict';
    var pluginName = 'ESRT_MessageRecollectionAllSkip';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var allSkipKeyName = 's';
    var allSkipKeyCode = 83;
    if (allSkipKeyCode) {
        if (!Input.keyMapper[allSkipKeyCode]) {
            Input.keyMapper[allSkipKeyCode] = 'messageAllSkip';
        } else {
            allSkipKeyName = Input.keyMapper[allSkipKeyCode];
        }
    }

    //=============================================================================
    // Game_Message
    //  メッセージスキップ情報を保持します。
    //=============================================================================
    var _Game_Message_initialize      = Game_Message.prototype.initialize;
    Game_Message.prototype.initialize = function() {
        _Game_Message_initialize.apply(this, arguments);
        this.clearAllSkipInfo();
    };

    Game_Message.prototype.onAllSkip = function() {
        this.setAllSkipFlg(true);
    };

    Game_Message.prototype.offAllSkip = function() {
        this.setAllSkipFlg(false);
    };
    
    Game_Message.prototype.allSkipFlg = function() {
        return this._allSkipFlg;
    };

    Game_Message.prototype.setAllSkipFlg = function(value) {
        this._allSkipFlg = value;
    };

    Game_Message.prototype.clearAllSkipInfo = function() {
        this._allSkipFlg = false;
    };

    //=============================================================================
    // Game_Interpreter
    //  マップイベント終了時にメッセージ完全スキップフラグを初期化します。
    //=============================================================================
    var _Game_Interpreter_terminate      = Game_Interpreter.prototype.terminate;
    Game_Interpreter.prototype.terminate = function() {
        _Game_Interpreter_terminate.apply(this, arguments);
        $gameMessage.clearAllSkipInfo();
    };
    
    //=============================================================================
    // Window_Message
    //  メッセージスキップ状態を反映します。
    //=============================================================================
    var _Window_Message_updateWait      = Window_Message.prototype.updateWait;
    Window_Message.prototype.updateWait = function() {
        this.updateAllSkip();
        
        return _Window_Message_updateWait.apply(this, arguments);
    };
    
    var _Window_Message_isTriggered      = Window_Message.prototype.isTriggered;
    Window_Message.prototype.isTriggered = function() {
        return _Window_Message_isTriggered.apply(this, arguments) ||
            this.messageAllSkip();
    };
    Window_Message.prototype.messageAllSkip = function() {
        return $gameMessage.allSkipFlg();
    };
    
    Window_Message.prototype.updateAllSkip = function() {
        if (this.isClosed()) return;
        if (this.isTriggeredAllMessageSkip()) {
            $gameMessage.onAllSkip();
            $gameSwitches.setValue(135, true);
        }
    };

    Window_Message.prototype.isTriggeredAllMessageSkip = function() {
        if(!$gameSwitches.value(131)) {return false};
        return Input.isTriggered('messageAllSkip') || Input.isTriggered(allSkipKeyName) ;
    };
    
    //=============================================================================
    // Game_Interpreter
    //  メッセージ完全スキップを反映します。
    //=============================================================================
    
    var _Game_Interpreter_updateWait      = Game_Interpreter.prototype.updateWait;
    Game_Interpreter.prototype.updateWait = function() {
        if (this.updateAllSkip()) {
            return true;
        }
        return _Game_Interpreter_updateWait.apply(this, arguments);
    };

    Game_Interpreter.prototype.updateAllSkip = function() {
        if (this.messageAllSkip()) {
            $gameMessage.offAllSkip();
            this.jumpTo(this._list.length - 2);
            
            return true;
        }
    };
    
    Game_Interpreter.prototype.messageAllSkip = function() {
        return $gameMessage.allSkipFlg();
    };
})();
