//=============================================================================
// ESRT_PictureAddMove.js
// ----------------------------------------------------------------------------
// Copyright (c) 2018 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================
/*:
 * @plugindesc ピクチャに移動と併用できる動きを追加する
 * @author 白
 *
 * @help  
 *
 */

 var Esrt = Esrt || {};

(function() {
    'use strict';
    var pluginName    = 'ESRT_PictureAddMove';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);


    //=============================================================================
    // Game_Picture
    //  
    //=============================================================================
    var _Game_Picture_update =  Game_Picture.prototype.update;
    Game_Picture.prototype.update = function() {
        _Game_Picture_update.call(this);
        this.updateAddMove();
    };
     
    Game_Picture.prototype.updateAddMove = function() {
        if (this._duration2 > 0) {
            var d = this._duration2;
            this._time2++;
            if(this._time2 > this._offsetTime) {
                if (typeof this._baseX === "undefined") {
        			this._baseX = this._x;
        			this._baseY = this._y;
        			this._baseScaleX = this._scaleX;
        			this._baseScaleY = this._scaleX;
        			this._baseAngle  = this._angle;
                }
                var angle = (this._time2 - this._offsetTime) * (Math.PI / this._duration1);
                var move  = Math.sin(angle);
                
                if (this._moveType.x) {
                    this._x = (this._moveType.x == "abs_sin") ? this._baseX + Math.abs(move) * this._distance : this._baseX + move * this._distance;
                }
                if (this._moveType.y) {
                    this._y = (this._moveType.y == "abs_sin") ? this._baseY + Math.abs(move) * this._distance : this._baseY + move * this._distance;
                }
                if(0){
                    this._scaleX  = (this._scaleX  * (d - 1) + this._targetScaleX)  / d;
                    this._scaleY  = (this._scaleY  * (d - 1) + this._targetScaleY)  / d;
                }
            }
            this._duration2--;
        }
    };
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        
        if(command === 'PictureAddMove'){
            this.moveAdd(args);
        }
    };

    // Move Picture
    Game_Interpreter.prototype.moveAdd = function(args) {
        var pictureId  = args[0];
        var moveType   = args[1];
        var distance   = args[2].match(/:/) ? args[2].split(":")[1] : args[2];
        var duration   = args[3].match(/:/) ? args[3].split(":")[1] : args[3];
        var count      = args[4].match(/:/) ? args[4].split(":")[1] : args[4];
        var offsetTime = args[5].match(/:/) ? args[5].split(":")[1] : args[5];
        
        $gameScreen.moveAddPicture(pictureId, moveType, distance, duration, count, offsetTime);
        return true;
    };

    Game_Screen.prototype.moveAddPicture = function(pictureId, moveType, distance, duration, count, offsetTime) {
        var picture = this.picture(pictureId);
        var allDuration = Number(duration) * Number(count) + Number(offsetTime);
        
        if (picture) {
            picture.moveAdd(moveType, distance, duration, count, offsetTime, allDuration);
        }
    };

    Game_Picture.prototype.moveAdd = function(moveType, distance, duration, count, offsetTime, allDuration) {
        this._time2 = 0;
        
        this._moveType = {};
        var moveTipes = moveType.split("_").forEach(function(moveType) {
			switch (moveType) {
				case 'x':
					this._moveType.x = "sin";
					break;
				case 'y':
					this._moveType.y = "sin";
					break;
				case 'absX':
					this._moveType.x = "abs_sin";
					break;
				case 'absY':
					this._moveType.y = "abs_sin";
					break;
				default:
					break;
			}
        }.bind(this));
        
        this._distance   = distance;
        this._duration1  = duration;
        this._count      = count;
        this._offsetTime = offsetTime;
        this._duration2  = allDuration;
    }
}());
