//=============================================================================
// ESRT_Plugins.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc 独自プラグイン
 * @author 白
 *
 * @param 敗北シーンフラグスイッチ
 * @desc 敗北シーンでウィンドウ枠の変更に使用するスイッチを指定します。
 * @default 132
 *
 * @param システムメニューボタン表示スイッチ
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 201
 *
 * @param システムメニューボタンタッチ状態スイッチ
 * @desc 表示のオンオフに使用するスイッチを指定します。
 * @default 202
 *
 * @param システムメニューボタンホバー状態スイッチ
 * @desc ホバー状態を記録するスイッチを指定します。
 * @default 203
 *
 * @param システムメニューボタンホバー状態オフスイッチ
 * @desc ホバー状態をオフにするスイッチを指定します。
 * @default 204
 *
 * @param システムメニューボタンタッチ状態オフスイッチ
 * @desc タッチ状態をオフにするスイッチを指定します。
 * @default 205
 *
 * @param システムメニュータッチコモンイベント
 * @desc タッチによって起動するコモンイベント
 * @default 101
 *
 * @param システムメニューボタンのピクチャID
 * @desc 利用するピクチャIDを指定します。
 * @default 99
 *
 * @help
 *
 *
 */

var Esrt = Esrt || {};
Esrt.imageReservationIds = Esrt.imageReservationIds || {};
Esrt.SeInterval = {};

Esrt.WindowMove.Option = Esrt.WindowMove.Option || {};
Esrt.WindowMove.Option.isMoveIn = true;
Esrt.WindowMove.Option.windowMoveDuration = 10;
Esrt.WindowMove.Option.windowMoveCount = 0;
Esrt.WindowMove.Option.windowMoveType = 1;
Esrt.WindowMove.Option.windowMoveRate = 0;

(function() {
    'use strict';
    var pluginName    = 'ESRT_Plugins';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var windowWidth  = 220;
    var windowHeight = 500;
    var windowY      = 0;
    var loseSceneSwitch = parseInt(parameters['敗北シーンフラグスイッチ']);
    var systemMenuShowSwitch = parseInt(parameters['システムメニューボタン表示スイッチ']);
    var systemMenuTouchSwitch = parseInt(parameters['システムメニューボタンタッチ状態スイッチ']);
    var systemMenuHoverSwitch = parseInt(parameters['システムメニューボタンホバー状態スイッチ']);
    var systemMenuHoverOffSwitch = parseInt(parameters['システムメニューボタンホバー状態オフスイッチ']);
    var systemMenuTouchOffSwitch = parseInt(parameters['システムメニューボタンタッチ状態オフスイッチ']);
    var systemMenuCommonID = parseInt(parameters['システムメニュータッチコモンイベント']);
    var systemMenuButtonPictureId = parseInt(parameters['システムメニューボタンのピクチャID']);


    //=============================================================================
    //  受精カウント用アイコン番号の取得
    //=============================================================================
    Esrt.GetFertilizationIconNumber = function(count, max) {
        count = (typeof count === "undefined") ? Math.min($gameVariables.value(201), 6) : count;
        max = (typeof max === "undefined") ? $gameVariables.value(202) : max;

        if (count > max) {
            max = 5;
            count = max;
        }
        
        return 320 + ((max-1) * 16) + count;
    };

    //=============================================================================
    // ImageManager
    //  pictureサブフォルダからロードする関数を追加
    //=============================================================================
    ImageManager.loadPictureButton = function(filename, hue) {
        return this.loadBitmap('img/pictures/button/', filename, hue, true);
    };

    ImageManager.loadPictureTumbnail = function(filename, hue) {
        return this.loadBitmap('img/pictures/thumbnail/', filename, hue, true);
    };
    
    //=============================================================================
    // ImageManager
    //  サムネイルフォルダからロードする関数を追加
    //=============================================================================
    DataManager.maxSavefiles = function() {
        return 100;
    };
    //=============================================================================
    // Scene_Map
    //  立ち絵のある画面右側のタッチに反応しない
    //=============================================================================
    var _Scene_Map_processMapTouch = Scene_Map.prototype.processMapTouch;
    Scene_Map.prototype.processMapTouch = function() {
        if (TouchInput.isTriggered() || this._touchCount > 0) {
            if (TouchInput.isPressed()) {
                var x = $gameMap.canvasToMapX(TouchInput.x);
                var y = $gameMap.canvasToMapX(TouchInput.y);
                if ($gameVariables.value(108) < 2 || x <= 12){
                    _Scene_Map_processMapTouch.call(this);
                } else if ($gameSwitches.value(125) && !(x > 12 && y > 9)){
                    _Scene_Map_processMapTouch.call(this);
                }
            }
        }
    };
    //=============================================================================
    // Game_Event
    //=============================================================================
    var _Game_Event_initMembers = Game_Event.prototype.initMembers;
    Game_Event.prototype.initMembers = function() {
        _Game_Event_initMembers.apply(this, arguments);
        this._moveStopTime = 0;
        this._moveStopCount = 0;
    };
    // OVERWRITE
    // 画面外でもイベントが移動するよう変更
    Game_Event.prototype.updateSelfMovement = function() {
        if (!this._locked &&
            this.checkStop(this.stopCountThreshold())) {
            switch (this._moveType) {
            case 1:
                this.moveTypeRandom();
                break;
            case 2:
                this.moveTypeTowardPlayer();
                break;
            case 3:
                this.moveTypeCustom();
                break;
            }
        };
    };

    // 特定の位置までイベントを移動させる動作
    Game_Character.prototype.esratSearchMovement = function(x, y) {
        if(this._moveStopTime>0){
            this._moveStopTime -= 1;
            return;
        }
            var direction = this.findDirectionTo(x, y);
            if (direction > 0) this.moveStraight(direction);

        if(!this.isMovementSucceeded()){
            // 移動できないときに数フレームの間処理を飛ばす（findDirectionToが大量に飛ぶと重くなるため）
            // 同時に、移動を諦める判断用のカウントを増やす
            this._moveStopTime = this._moveSpeed * 15;
            this._moveStopCount += 1;
            return;
        }
        this._moveStopCount = 0;
    };

    // イベントの初期位置に歩いて戻る動作
    Game_Event.prototype.esratReturnMovement = function(x, y) {
        var direction = this.findDirectionTo(x, y);
        if (direction > 0) this.moveStraight(direction);
    };

    // 通常時は下移動で、y軸が同じになった時だけ横に近づいてくる動作
    Game_Event.prototype.esratBWXApproachMovement = function() {
        //var direction = this.findDirectionTo(x, y);
        var direction = 2;
        if (this._y === $gamePlayer._y) {
            direction = (this._x > $gamePlayer._x) ? 4 : 6;
        }

        if (direction > 0) this.moveStraight(direction);
    };
    //=============================================================================
    // Game_Map
    //=============================================================================
    // OVERWRITE
    var _Game_Event_update = Game_Event.prototype.update;
    Game_Event.prototype.update = function() {
        _Game_Event_update.call(this);
        //this.calcOpacity();
    };

    // イベントの透過度を定義
    Game_Event.prototype.calcOpacity = function(){

        var visibleDistance = 4;
        visibleDistance -= ($gameSwitches.value(101)) ? $gameVariables.value(101) : 0;
        if($gameSwitches.value(103) || $gameSwitches.value(106)){
            visibleDistance = 100;
        }

        var inRange = Math.abs(this.deltaXFrom($gamePlayer.x));
        inRange += Math.abs(this.deltaYFrom($gamePlayer.y));

        var calcOpacity = 255;
        if (inRange <= visibleDistance) {
            calcOpacity = 255;
        }else if (inRange <= visibleDistance + 2) {
            calcOpacity = 255 - (inRange - visibleDistance) * 128 - 1;
        } else {
            calcOpacity = 0;
        };

        calcOpacity = calcOpacity < 0 ? 0 : calcOpacity;
        this._opacity += this._opacity < calcOpacity ? 10 : -10;

        if(Math.abs(this._opacity - calcOpacity) <= 10){
            this._opacity = calcOpacity;
        }
    };

    //=============================================================================
    // Scene_Map
    //    ウィンドウ移動情報を初期化します
    //=============================================================================
    var _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this, arguments);

        Esrt.WindowMove.Option.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.Option.isMoveIn;
        Esrt.WindowMove.Option.windowMoveCount = (Esrt.WindowMove.Option.isMoveIn) ? 0 : Esrt.WindowMove.Option.moveDuration;
    };
    //=============================================================================
    // Scene_Option
    //  戻るボタン・オプションウィンドウ2,3を追加
    //=============================================================================
    var Scene_Options_initialize = Scene_Options.prototype.initialize;

    Scene_Options.prototype.initialize = function() {
        Scene_Options_initialize.call(this);
        this.CheckOption4Available();

        this._backFlag = false;

        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Option;
        this._windowMoveObject.windowMoveRate = 0;
    };

    Scene_Options.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.Option.isMoveIn = false;
        Esrt.WindowMove.Option.windowMoveCount = (Esrt.WindowMove.Option.isMoveIn) ? 0 : Esrt.WindowMove.Option.windowMoveDuration;
        Esrt.WindowMove.Option.windowMoveType = 1;
    }

    Scene_Options.prototype.CheckOption4Available = function() {
        this._isAvailableOption4 = false;
        if(Esrt.gallery._isClickLeftKeyForOption || $gameSwitches.value(301)){
            this._isAvailableOption4 = true;
        }
    };

    Scene_Options.prototype.createBackground = function() {
        var width = Graphics.boxWidth * 2;
        var height = Graphics.boxHeight * 2;

        this._backgroundSprite = new Sprite();
        this.addChild(this._backgroundSprite);

        this._backgroundTile = new TilingSprite(ImageManager.loadTitle1("Background_3"));
        this._backgroundTile.move(0,0, width, height);
        this._backgroundTile.anchor.x = 0.5;
        this._backgroundTile.anchor.y = 0.5;
        this._backgroundSprite.addChild(this._backgroundTile);
    };

    var Scene_Options_create = Scene_Options.prototype.create;
    Scene_Options.prototype.create = function() {
        Scene_Options_create.call(this);
        this.createOptionsWindow2();
        this.createOptionsWindow3();
        this.createOptionsWindow4();
        this.createBackButtonWindow();
        this.createOptionsWindow4Over();

        if(Esrt.Message._isClickLeftKeyForOption){
            this.activateOption3();
            Esrt.Message._isClickLeftKeyForOption = false;
        }
        if(Esrt.gallery._isClickLeftKeyForOption){
            this.activateOption4();
            Esrt.gallery._isClickLeftKeyForOption = false;
        }
    };

    var Scene_Options_createOptionsWindow = Scene_Options.prototype.createOptionsWindow;
    Scene_Options.prototype.createOptionsWindow = function() {
        Scene_Options_createOptionsWindow.call(this);
        this._optionsWindow.setHandler('touchInside', this.activateOption.bind(this));
        this._optionsWindow._scene = this;
    };

    Scene_Options.prototype.createOptionsWindow2 = function() {
        this._optionsWindow2 = new Window_Options2();
        this._optionsWindow2.setHandler('cancel', this.popScene.bind(this));
        this._optionsWindow2.setHandler('touchInside', this.activateOption2.bind(this));
        this._optionsWindow2.deselect();
        this._optionsWindow2.deactivate();
        this.addWindow(this._optionsWindow2);
        this._optionsWindow2._scene = this;
    };

    Scene_Options.prototype.createOptionsWindow3 = function() {
        this._optionsWindow3 = new Window_Options3();
        this._optionsWindow3.setHandler('cancel', this.popScene.bind(this));
        this._optionsWindow3.setHandler('touchInside', this.activateOption3.bind(this));
        this._optionsWindow3.deselect();
        this._optionsWindow3.deactivate();
        this.addWindow(this._optionsWindow3);
        this._optionsWindow3._scene = this;
    };

    Scene_Options.prototype.createOptionsWindow4 = function() {
        this._optionsWindow4 = new Window_Options4();
        this._optionsWindow4.setHandler('cancel', this.popScene.bind(this));
        if (this._isAvailableOption4) {
            this._optionsWindow4.setHandler('touchInside', this.activateOption4.bind(this));
        }
        this._optionsWindow4.deselect();
        this._optionsWindow4.deactivate();
        this.addWindow(this._optionsWindow4);
        this._optionsWindow4._scene = this;
    };

    Scene_Options.prototype.createOptionsWindow4Over = function() {
        if (this._isAvailableOption4) {return;}

        this._optionsWindow4Over = new Window_Option4Over();
        this.addWindow(this._optionsWindow4Over);
    };

    Scene_Options.prototype.createBackButtonWindow = function() {
        this._backButtonWindow = new Window_Button(0, 550, 190, 70, 'button_back_left');
        this._backButtonWindow.setHandler('touchInside', this.commandBack.bind(this));
        this.addWindow(this._backButtonWindow);
    }

    Scene_Options.prototype.commandBack = function() {
        if(this._backFlag){ return; }// 1クリックで複数呼び出されることに対する対処

        this._backFlag = true;
        SoundManager.playCancel();
        this.popScene();
    };

    var _Scene_Options_update = Scene_Options.prototype.update;
    Scene_Options.prototype.update = function() {
        _Scene_Options_update.call(this);
        this.updateBackgroundEffects();
    };

    Scene_Options.prototype.updateBackgroundEffects = function() {
        this._backgroundTile.origin.x += 0.5;
        this._backgroundTile.origin.y += 0.5;
    };

    Scene_Options.prototype.activateOption = function() {
        this._optionsWindow2.deselect();
        this._optionsWindow2.deactivate();
        this._optionsWindow3.deselect();
        this._optionsWindow3.deactivate();
        this._optionsWindow4.deselect();
        this._optionsWindow4.deactivate();
        this._optionsWindow.activate();
        this._optionsWindow.select(0);
        this._optionsWindow.onTouchSelectOnly();
        SoundManager.playCursor();
    };

    Scene_Options.prototype.activateOption2 = function() {
        this._optionsWindow.deselect();
        this._optionsWindow.deactivate();
        this._optionsWindow3.deselect();
        this._optionsWindow3.deactivate();
        this._optionsWindow4.deselect();
        this._optionsWindow4.deactivate();
        this._optionsWindow2.activate();
        this._optionsWindow2.select(0);
        this._optionsWindow2.onTouchSelectOnly();
        SoundManager.playCursor();
    };

    Scene_Options.prototype.activateOption3 = function() {
        this._optionsWindow.deselect();
        this._optionsWindow.deactivate();
        this._optionsWindow2.deselect();
        this._optionsWindow2.deactivate();
        this._optionsWindow4.deselect();
        this._optionsWindow4.deactivate();
        this._optionsWindow3.activate();
        this._optionsWindow3.select(0);
        this._optionsWindow3.onTouchSelectOnly();
        SoundManager.playCursor();
    };

    Scene_Options.prototype.activateOption4 = function() {
        this._optionsWindow.deselect();
        this._optionsWindow.deactivate();
        this._optionsWindow2.deselect();
        this._optionsWindow2.deactivate();
        this._optionsWindow3.deselect();
        this._optionsWindow3.deactivate();
        this._optionsWindow4.activate();
        this._optionsWindow4.select(0);
        this._optionsWindow4.onTouchSelectOnly();
        SoundManager.playCursor();
    };

    //=============================================================================
    // Window_Options
    //  表示位置を調整します
    //=============================================================================
    Window_Options.prototype.updatePlacement = function() {
        this.x = 0;
        this.y = 0;
    };
    Window_Options2.prototype.updatePlacement = function() {
        this.x = 0;
        this.y = 280;
    };
    Window_Options3.prototype.updatePlacement = function() {
        this.x = 400;
        this._baseX = 400;
        this.y = 0;
    };
    Window_Options4.prototype.updatePlacement = function() {
        this.x = 400;
        this._baseX = 400;
        this.y = 230;
    };

    //=============================================================================
    // Window_Option4Over
    //    性経験オプション未開放時のウィンドウを描画します
    //=============================================================================
    function Window_Option4Over() {
        this.initialize.apply(this, arguments);
    }

    Window_Option4Over.prototype = Object.create(Window_Base.prototype);

    Window_Option4Over.prototype.constructor = Window_Option4Over;
    Window_Option4Over.prototype.initialize = function() {
        var x = 400;
        var y = 230;
        var width = 400;
        var height = this.fittingHeight(7);

        // ウィンドウ移動演出
        this._windowPositionDirection = 6; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Option;

        Window_Base.prototype.initialize.call(this, x, y, width, height);

        this.drawData();
    };

    Window_Option4Over.prototype.drawData = function(type) {
        this.contents.clear();

        var y = 80;

        this.drawText("この項目は", 40, y);
        y += 40;
        this.drawText("エリア7クリア後に", 40, y);
        y += 40;
        this.drawText("開放されます", 40, y);
        this.resetTextColor();

    }

    Window_Option4Over.prototype.lineHeight = function() {
        return 48;
    };

    Window_Options2.prototype.drawItem = function(index) {
        var rect = this.itemRectForText(index);
        var statusWidth = this.statusWidth();
        var titleWidth = rect.width - statusWidth;
        this.resetTextColor();
        this.changePaintOpacity(this.isCommandEnabled(index));
        this.drawText(this.commandName(index), rect.x, rect.y, titleWidth - 50, 'left');
        this.drawText(this.statusText(index), titleWidth, rect.y, statusWidth, 'right');

        this.initialize_BarSprite();
    };
    //=============================================================================
    // Window_Help
    //  ヘルプウィンドウにx,y座標の位置入力を指定します
    //=============================================================================
    Window_Help.prototype.initialize = function(numLines, x, y) {
        var width = Graphics.boxWidth;
        var height = this.fittingHeight(numLines || 2);
        x = (typeof x != undefined) ? x : 0;
        y = (typeof y != undefined) ? y : 0;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this._text = '';
    };
    //=============================================================================
    // ウィンドウが重なり合ったとき、下のレイヤーを表示します。
    //=============================================================================
    if (!WindowLayer.throughWindow) {
        WindowLayer.throughWindow = true;
        WindowLayer.prototype._maskWindow = function(window) {};

        WindowLayer.prototype._canvasClearWindowRect = function(renderSession, window) {};
    }

    //=============================================================================
    // スマホ用に、画面右上に常にシステムメニュー表示ボタンを表示します
    //=============================================================================
    var _Scene_Map_createAllWindows_sysMenu = Scene_Map.prototype.createAllWindows;
    Scene_Map.prototype.createAllWindows = function() {
        _Scene_Map_createAllWindows_sysMenu.call(this);

        this.createSystemMenuWindow();
    };

    Scene_Map.prototype.createSystemMenuWindow = function() {
        this._systemMenuButtonWindow = new Window_Button(660, 0, 190, 210, 'button_sysmenu');
        this._systemMenuButtonWindow.setHandler('touchInside', this.commandSystemMenu.bind(this));
        this.addWindow(this._systemMenuButtonWindow);
    };

    Scene_Map.prototype.commandSystemMenu = function() {
        $gameTemp.reserveCommonEvent(systemMenuCommonID);
    };

    var _Scene_Map_update_sysMenu = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update_sysMenu.call(this);

        this.CheckandChangeSystemMenuButtonVisible();

        if(!$dataSystem) return;
    };

    Scene_Map.prototype.CheckandChangeSystemMenuButtonVisible = function() {
        if($gameMap.isEventRunning() || !$gameSwitches.value(systemMenuShowSwitch)){//
            this._systemMenuButtonWindow.hide();
            this._systemMenuButtonWindow.close();
        }else{
            this._systemMenuButtonWindow.show();
            this._systemMenuButtonWindow.open();
        }
    };

    //=============================================================================
    // ・メニュー時に背景をぼかす際、黒い線が表示されるのを防止
    //      var w = this.width - 1;
    //      var h = this.height - 1;
    //=============================================================================
    var _SceneManager_snapForBackground = SceneManager.snapForBackground;
    SceneManager.snapForBackground = function() {
        if (1) {
            this._backgroundBitmap = this.snap();
            this._backgroundBitmap.blurRefine();
        } else {
            _SceneManager_snapForBackground.call(this);
        }
    };

    Bitmap.prototype.blurRefine = function() {
        for (var i = 0; i < 2; i++) {
            var w = this.width - 1;
            var h = this.height - 1;
            var canvas = this._canvas;
            var context = this._context;
            var tempCanvas = document.createElement('canvas');
            var tempContext = tempCanvas.getContext('2d');
            tempCanvas.width = w + 2;
            tempCanvas.height = h + 2;
            tempContext.drawImage(canvas, 0, 0, w, h, 1, 1, w, h);
            tempContext.drawImage(canvas, 0, 0, w, 1, 1, 0, w, 1);
            tempContext.drawImage(canvas, 0, 0, 1, h, 0, 1, 1, h);
            tempContext.drawImage(canvas, 0, h - 1, w, 1, 1, h + 1, w, 1);
            tempContext.drawImage(canvas, w - 1, 0, 1, h, w + 1, 1, 1, h);
            context.save();
            context.fillStyle = 'black';
            context.fillRect(0, 0, w, h);
            context.globalCompositeOperation = 'lighter';
            context.globalAlpha = 1 / 9;
            for (var y = 0; y < 3; y++) {
                for (var x = 0; x < 3; x++) {
                    context.drawImage(tempCanvas, x, y, w, h, 0, 0, w, h);
                }
            }
            context.restore();
        }
        this._setDirty();
    };

    Game_BattlerBase.prototype.maxTp = function() {
        return 605;
    };

    //=============================================================================
    // 変身失敗シーンでメッセージウィンドウの枠画像を変更
    //=============================================================================
    // 画像の読み込み
    var _Scene_Boot_prototype_loadSystemWindowImage = Scene_Boot.prototype.loadSystemWindowImage;
    Scene_Boot.prototype.loadSystemWindowImage = function() {
        _Scene_Boot_prototype_loadSystemWindowImage.apply(this, arguments);
        ImageManager.loadSystem('Window_Lose');
        ImageManager.loadSystem('Window_Status');
        ImageManager.loadPicture('stageBack');
        if(typeof ImageManager.loadPictureMapStill == "function"){
            ImageManager.loadPictureMapStill('kana_map_mask_1_0001');
            ImageManager.loadPictureMapStill('kana_map_still_1_0001');
        }

        Esrt.imageReservationIds.VNButtons = Utils.generateRuntimeId();
        ImageManager.reserveSystem('VNButtons', 0, Esrt.imageReservationIds.VNButtons);
    };

    var _Window_Message_Initialize = Window_Message.prototype.initialize;
    Window_Message.prototype.initialize = function() {
        _Window_Message_Initialize.call(this);
        this.windowskin = ($gameSwitches.value(loseSceneSwitch) || DataManager.isEventTest()) ? ImageManager.loadSystem('Window_Lose') : ImageManager.loadSystem('Window_Talk');
        this._refreshAllParts();
    };
    if(typeof Window_NameBox == "function"){
        var _Window_NameBox_Initialize = Window_NameBox.prototype.initialize;
        Window_NameBox.prototype.initialize = function(parentWindow) {
            _Window_NameBox_Initialize.apply(this,arguments);
            this.windowskin = ($gameSwitches.value(loseSceneSwitch) || DataManager.isEventTest()) ? ImageManager.loadSystem('Window_Lose') : ImageManager.loadSystem('Window_Name');
        };
    }

    Window_Message.prototype.setSubWindowSkin = function(subWindow, systemImage) {
        subWindow.windowskin = ImageManager.loadSystem(systemImage);
    };

    var _Window_Message_standardFontFace      = Window_Message.prototype.standardFontFace;
    Window_Message.prototype.standardFontFace = function() {
        return ($gameSwitches.value(loseSceneSwitch) || DataManager.isEventTest()) ? "LoseFont" : _Window_Message_standardFontFace.apply(this, arguments);
    };
    if(typeof Window_NameBox == "function"){
        Window_NameBox.prototype.loadWindowskin = Window_Message.prototype.loadWindowskin;
        Window_NameBox.prototype.standardFontFace = Window_Message.prototype.standardFontFace;
    }

    //=============================================================================
    // ・未ログイン時のスイッチ削除対策
    //=============================================================================
    var _DataManager_createGameObjects = DataManager.createGameObjects;
    DataManager.createGameObjects = function() {
        var execFlag = false;
        if($gameSwitches){
            var tmpGameSwitches      = $gameSwitches;
            var tmpGameVariables     = $gameVariables;
            execFlag = true;
        }

        _DataManager_createGameObjects.apply(this, arguments);

        if(execFlag == true){
            $gameSwitches      = tmpGameSwitches;
            $gameVariables     = tmpGameVariables;
        }
    };

    //=============================================================================
    // ・効果音別に多重再生を禁止
    //=============================================================================

    var _SceneManager_update = SceneManager.update;
    SceneManager.update = function() {
        _SceneManager_update.call(this);

        Object.keys(Esrt.SeInterval).forEach(function(key) {
            if(Esrt.SeInterval[key] > 0){
                Esrt.SeInterval[key] -= 1
            }else{
                delete Esrt.SeInterval[key];
            }
        });
    }
    var _AudioManager_playSe = AudioManager.playSe;
    AudioManager.playSe = function(se) {
        if(se.name in Esrt.SeInterval) return;
        _AudioManager_playSe.apply(this, arguments);
        Esrt.SeInterval[se.name] = 2;
    };
    var _AudioManager_playStaticSe = AudioManager.playStaticSe;
    AudioManager.playStaticSe = function(se) {
        if(se.name in Esrt.SeInterval) return;
        _AudioManager_playStaticSe.apply(this, arguments);
        Esrt.SeInterval[se.name] = 2;
    };

    //=============================================================================
    // ・フェードアウトするが消さない
    //=============================================================================
    AudioManager.fadeOutMuteBgm = function(duration) {
        if (this._bgmBuffer && this._currentBgm) {
            this._bgmBuffer.fadeOut(duration);
        }
    };

    AudioManager.fadeInBgm = function(duration) {
        if (this._bgmBuffer && this._currentBgm) {
            this._bgmBuffer.fadeIn(duration);
        }
    };
    //=============================================================================
    // ・ルビを振る関係で文字の高さをちょっと上げる
    //=============================================================================
    Window_Help.prototype.lineHeight = function() {
        return 40;
    };

    //=============================================================================
    // ・歩行時の足踏み禁止
    //=============================================================================
    var _Game_Event_updatePattern      = Game_Event.prototype.updatePattern;
    Game_Event.prototype.updatePattern = function() {
        var noStep = this.event().meta.no_step;
        if (noStep) {return;}
        _Game_Event_updatePattern.apply(this, arguments);
    };

    //=============================================================================
    // セーブファイル一覧への記載事項追加
    //=============================================================================
    var _DataManager_makeSavefileInfo = DataManager.makeSavefileInfo;
    DataManager.makeSavefileInfo = function() {
        var info = _DataManager_makeSavefileInfo.apply(this, arguments);
        info.lastStage  = (Esrt.stageSelect.areaNum) ? $dataStage["stageList"][Esrt.stageSelect.areaNum]["stages"][Esrt.stageSelect.stageIndex].stageText : null;
        info.ferCnt     = $gameVariables.value(201);
        info.ferCntMax  = $gameVariables.value(202);
        info.virgin     = $gameVariables.value(241);
        info.gameMode   = $gameVariables.value(171);
        info.coinNormal = Esrt.stageScoreFanc.GetAllStageHighCoin(Esrt.gameModes.normal)[0];
        info.coinExtra  = Esrt.stageScoreFanc.GetAllStageHighCoin(Esrt.gameModes.extra)[0];
        return info;
    };

    var _Window_SavefileList_initialize = Window_SavefileList.prototype.initialize;
    Window_SavefileList.prototype.initialize = function(x, y, width, height) {
        _Window_SavefileList_initialize.apply(this, arguments);
        this._coinNormalMax = Esrt.stageScoreFanc.GetAllStageHighCoin(Esrt.gameModes.normal)[1];
        this._coinExtraMax  = Esrt.stageScoreFanc.GetAllStageHighCoin(Esrt.gameModes.extra)[1];
    };

    var _Window_SavefileList_maxVisibleItems = Window_SavefileList.prototype.maxVisibleItems;
    Window_SavefileList.prototype.maxVisibleItems = function() {
        return 4;
    };

    Window_SavefileList.prototype.drawContents = function(info, rect, valid) {
        var bottom = rect.y + rect.height;
        var lineHeight = this.lineHeight();
        if (rect.width >= 420) {
            this.drawText("魔法石", rect.x + 442, rect.y, rect.width - 192);
            var coinNormalText = (info.coinNormal) ? ('   ' + info.coinNormal).slice(-3) : " --";
            this.drawText("　N " + coinNormalText + "／" + this._coinNormalMax + "　", rect.x + 192, rect.y, rect.width - 192, "right");

            var coinExtraText = (info.coinExtra) ? ('   ' + info.coinExtra).slice(-3) : " --";
            this.drawText("EX " + coinExtraText + "／" + this._coinExtraMax + "　", rect.x + 192, rect.y + lineHeight, rect.width - 192, "right");

            var iconId = 0;
            iconId = (info.coinNormal >= this._coinNormalMax) ? 88 : 90;
            this.drawIcon(iconId, rect.x + 542, rect.y + 5);

            iconId = (info.coinExtra >= this._coinExtraMax) ? 87 : 90;
            this.drawIcon(iconId, rect.x + 542, rect.y + 5 + lineHeight);

            var lastStageText = (!info.lastStage || (info.lastStage && info.lastStage.match(/^\d+-\d+$/))) ? "ステージ" : "";
            lastStageText += (info.lastStage) ? info.lastStage : "?-?";
            lastStageText += (info.gameMode == 1) ? "EX" : "";
            this.drawText(lastStageText, rect.x + 182, rect.y, rect.width - 192);

            this.changeTextColor(this.systemColor());
            this.drawText("受精カウント", rect.x + 182, rect.y + lineHeight, 134);
            this.resetTextColor();
            if (info.ferCntMax) {
                this.drawIcon(Esrt.GetFertilizationIconNumber(info.ferCnt, info.ferCntMax), rect.x + 332, rect.y + 5 + lineHeight);
                var max   = info.ferCntMax > 5 ? "∞" : info.ferCntMax;
                this.drawText(info.ferCnt + "／" + max, rect.x + 382, rect.y + lineHeight);
            } else {
                this.drawIcon(Esrt.GetFertilizationIconNumber(0, 5), rect.x + 332, rect.y + 5 + lineHeight);
                this.drawText("--／--", rect.x + 382, rect.y + lineHeight);

            }

            this.changeTextColor(this.systemColor());
            this.drawText("処女", rect.x + 182, rect.y + lineHeight * 2, 134);
            this.resetTextColor();
            var lastStageText = (typeof info.virgin === "undefined") ? "不明" : info.virgin == 0 ? "完全処女" : info.virgin == 1 ? "変身前は処女" : "経験済";
            this.drawText(lastStageText, rect.x + 332, rect.y + lineHeight * 2, rect.width - 192);

            if (valid) {
                this.drawPartyCharacters(info, rect.x + 60, bottom - 4);
            }
        }
        var y2 = bottom - lineHeight;
        this.drawPlaytime(info, rect.x, y2, rect.width);
    };

    //=============================================================================
    // ・テキスト文字数判定
    //=============================================================================
    Esrt.getCommonEventLength = function(id) {
        if (!(Utils.isNwjs() && Utils.isOptionValid('test'))) {return;}

        var totalLength = 0;
        var index = 0;

        $dataCommonEvents.forEach(function(commonEvenet) {
            if (commonEvenet && commonEvenet.id === id){
                commonEvenet.list.forEach(function(data) {
                    if(data.code != 401) return;

                    totalLength += data.parameters[0].length;
                    index += 1;
                    console.info(index, totalLength, data.parameters[0].length, data.parameters[0]);
                });
                return commonEvenet.id;
            }
        });
        console.info(totalLength);

    };

    //=============================================================================
    // ・全コモンイベントテキスト文字数判定
    //=============================================================================
    Esrt.getAllCommonEventLength = function() {
        if (!(Utils.isNwjs() && Utils.isOptionValid('test'))) {return;}

        var totalLength = 0;
        var index = 0;
        var resultArray = [];

        $dataCommonEvents.forEach(function(commonEvenet) {
            totalLength = 0;
            index = 0;

            if(!commonEvenet) return;

            commonEvenet.list.forEach(function(data) {
                if(data.code != 401) return;

                totalLength += data.parameters[0].length;
                index += 1;
            });
            resultArray.push(commonEvenet.name+"\t"+index+"\t"+totalLength)

            return;
        });
        return resultArray.join("\n");
    };

    //=============================================================================
    // ・受精カウントMAXかの判定
    //=============================================================================
    Esrt.isFertilized = function() {
        return $gameVariables.value(202) < 6 && $gameVariables.value(201) >= $gameVariables.value(202);
    }

    //=============================================================================
    // ・妊娠解除イベント経験済みかの判定(妊娠経験=2or苗床経験=1)
    //=============================================================================
    Esrt.isExperiencedRemoval = function() {
        return $gameVariables.value(227) == 2 || $gameVariables.value(228) == 1;
    }

    //=============================================================================
    // ・妊娠解除イベント経験済みかの判定(妊娠経験=2or苗床経験=1)
    //=============================================================================
    var _Game_Event_isCollidedWithEvents      = Game_Event.prototype.isCollidedWithEvents;
    Game_Event.prototype.isCollidedWithEvents = function(x, y) {
        if (this.event().meta.eventThrough) {return;}

        return _Game_Event_isCollidedWithEvents.apply(this, arguments);
    };
    
    //=============================================================================
    // ゲームパッドの接続確認
    //=============================================================================
    Input.isGamepadsConnected = function() {
    	if (navigator.getGamepads) {
    		var gamepads = navigator.getGamepads();
    		if (gamepads) {
    			for (var i = 0; i < gamepads.length; i++) {
    				var gamepad = gamepads[i];
    				if (gamepad && gamepad.connected) return true;
    			}
    		}
    	}
    	return false;
    };
})();

// 画像が表示されない系問題全般を改善するかもしれないプラグイン @koma_neko様
(function(){
    'use strict';

    (function(o, p) {
        var f = o[p]; o[p] = function(source, sx, sy, sw, sh, dx, dy, dw, dh) {
            var bitmap = (!this.width) ? this : (!source.width) ? source : null;
            if (bitmap) {
                bitmap.addLoadListener(function(){
                    this.blt(source, sx, sy, sw, sh, dx, dy, dw, dh);
                }.bind(this));
                return;
            }
            f.apply(this, arguments);
        };
    }(Bitmap.prototype, 'blt'));
}());

// デベロッパーツールを開きながらeval関数を実行すると動作が重くなるため、スクリプトの処理をプラグインコマンドにする
(function() {
    'use strict';

    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'ScriptAlter'){
            switch(args[0]){
                case 'CoinCollide':
                    if(CoinCollide.call(this)){
                        CoinGet.call(this);
                    }
                    break;
                case 'CoinCollideMoveDown':
                    if(CoinCollideMoveDown.call(this)){
                        CoinGet.call(this);
                    }
                    break;
                case 'CoinCollideMoveLeftDown':
                    if(CoinCollideMoveLeftDown.call(this)){
                        CoinGet.call(this);
                    }
                    break;
                case 'CoinCollideMoveRightDown':
                    if(CoinCollideMoveRightDown.call(this)){
                        CoinGet.call(this);
                    }
                    break;
                case 'CoinCollideDummy':
                    if(CoinCollide.call(this)){
                        CoinGetDummy.call(this);
                    }
                    break;
                default:
                    break;
            }
        }
    };

    // 魔法石の当たり判定
    function CoinCollide() {
        return this.character(-1).x == this.character(0).x && this.character(-1).y == this.character(0).y;
    }

    // 移動中の魔法石の当たり判定
    function CoinCollideMoveDown() {
        return Math.abs(this.character(-1).x - this.character(0).x) <= 1 && (this.character(-1).y == this.character(0).y || this.character(-1).y == this.character(0).y - 1);
    }

    // ←↓移動中の魔法石の当たり判定
    function CoinCollideMoveLeftDown() {
        return (this.character(-1).x == this.character(0).x || this.character(-1).x == this.character(0).x + 1) && (this.character(-1).y == this.character(0).y || this.character(-1).y == this.character(0).y - 1);
    }

    // →↓移動中の魔法石の当たり判定
    function CoinCollideMoveRightDown() {
        return (this.character(-1).x == this.character(0).x || this.character(-1).x == this.character(0).x - 1) && (this.character(-1).y == this.character(0).y || this.character(-1).y == this.character(0).y - 1);
    }

    // 魔法石の取得
    function CoinGet() {
        $gameVariables.setValue(128, $gameVariables.value(128) + 1);
        AudioManager.playSe({"name":"Coin","volume":40,"pitch":100,"pan":0});
        if (this._eventId > 0) {
            var key = [this._mapId, this._eventId, "B"];
            $gameSelfSwitches.setValue(key, true);
        }
    }

    // 魔法石総数を増やさない疑似取得
    function CoinGetDummy() {
        AudioManager.playSe({"name":"Coin","volume":90,"pitch":100,"pan":0});
        if (this._eventId > 0) {
            var key = [this._mapId, this._eventId, "A"];
            $gameSelfSwitches.setValue(key, true);
        }
    }
}());
