//=============================================================================
// ESRT_Racing.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc エリア5ボス・レーシングステージ用のプラグインコマンドを追加するプラグイン
 * @author 白
 *
 *
 * @help 
 *
 */


//=============================================================================
//    汎用関数の定義
//=============================================================================
(function() {
    'use strict';
	var pluginName    = 'ESRT_Racing';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'Racing'){
            switch(args[0]){
                case 'getAheadRacer':
                    this.getAheadRacer(args);
                    break;
                case 'getFireTargetRacer':
                    this.getFireTargetRacer(args);
                    break;
                case 'getLeadRacer':
                    this.getLeadRacer(args);
                    break;
                case 'getNearestRacer':
                    this.getNearestRacer(args);
                    break;
                default:
                    break;
            }
        }
    };
    
    // 前方にいるレーサーの数を取得し変数に代入
    Game_Interpreter.prototype.getAheadRacer = function(args) {
        var playerY = this.character(-1).y;
        var rank = 1;
        $gameMap.events().forEach(function(event) {
            if (event.event().meta.racer) {
                if(event.y < playerY){
                    rank += 1;
                }
            }
        });
        $gameVariables.setValue(args[1], rank);
        return;
    }
    
    // 赤甲羅のターゲットを取得
    // ・キャラクターと同軸前方に敵がいる場合はその敵を
    // ・それ以外の場合、レース全体で一番前にいる敵をターゲットにする
    Game_Interpreter.prototype.getFireTargetRacer = function(args) {
        if(!this.getFrontRacer(args)) {
            this.getLeadRacer(args);
        }
        
        return;
    }
    
    // 先頭にいるレーサーを取得
    Game_Interpreter.prototype.getFrontRacer = function(args) {
        var playerX = this.character(-1).x;
        var playerY = this.character(-1).y;
        var minY = 99;
        var targetEvent = null;
        $gameMap.events().forEach(function(event) {
            if (event.event().meta.racer) {
                if (playerX == event.x && playerY > event.y && minY > event.y) {
                    minY =  event.y;
                    targetEvent = event;
                }
            }
        });
        
        if (targetEvent) {
            this.character(0)._chaseTargetEvent = targetEvent;
            return true;
        }
        return false;
    }
    
    // 先頭にいるレーサーを取得
    Game_Interpreter.prototype.getLeadRacer = function(args) {
        var playerY = this.character(-1).y;;
        var minY = 99;
        var targetEvent = null;
        $gameMap.events().forEach(function(event) {
            if (event.event().meta.racer) {
                if (minY > event.y) {
                    minY =  event.y;
                    targetEvent = event;
                }
            }
        });
        
        if (targetEvent) {
            this.character(0)._chaseTargetEvent = targetEvent;
            return true;
        }
        
        return false;
    }
    // 最も近くにいるレーサーを取得
    Game_Interpreter.prototype.getNearestRacer = function(args) {
        var playerY = this.character(-1).y;
        var frontMaxY = -1;
        var backMinY = 99;
        var frontNearestEvent = null;
        var backNearestEvent = null;
        $gameMap.events().forEach(function(event) {
            if (event.event().meta.racer) {
                if(event.y < playerY){
                    if (frontMaxY < event.y) {
                        frontMaxY = event.y;
                        frontNearestEvent = event;
                    }
                } else{
                    if (backMinY > event.y) {
                        backMinY =  event.y;
                        backNearestEvent = event;
                    }
                }
            }
        });
        
        if (frontNearestEvent) {
            this.character(0)._chaseTargetEvent = frontNearestEvent;
        } else if (backNearestEvent) {
            this.character(0)._chaseTargetEvent = backNearestEvent;
        }
    }
    
    //=============================================================================
    // Game_Event
    //  プラグインコマンドを追加定義します。
    //=============================================================================
	// 現在のレーススピードに合わせて下に移動
    Game_Event.prototype.moveDownRaceSpeed = function() {
        this.setRaceSpeed();
        this.moveStraight(2);
	};
    
	// 現在のレーススピードに合わせて左下に移動
    Game_Event.prototype.moveLeftDownRaceSpeed = function() {
        this.setRaceSpeed();
        this.moveDiagonally(4, 2);
	};
    
	// 現在のレーススピードに合わせて右下に移動
    Game_Event.prototype.moveRightDownRaceSpeed = function() {
        this.setRaceSpeed();
        this.moveDiagonally(6, 2);
	};
    
	// 現在のレーススピードに合わせて下に移動（マップ外に突き抜け）
    Game_Event.prototype.moveY1RaceSpeed = function() {
        this.setRaceSpeed();
        this._y += 1
	};
    
	// 現在のレーススピードに合わせて移動速度を設定
    Game_Event.prototype.setRaceSpeed = function() {
        if(typeof Esrt.EventRespawnList == "undefined" || typeof Esrt.EventRespawnList.moveSpeed == "undefined") {return;}
        
        this.setMoveSpeed(Esrt.EventRespawnList.moveSpeed);
	};
}());
