//=============================================================================
// ESRT_SceneGallery.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:ja
 * @plugindesc 独自プラグイン
 * @author 白
 *
 * @param タイトルに追加
 * @desc タイトル画面にギャラリーを追加します。(ON/OFF)
 * @default ON
 *
 * @param サムネイル幅
 * @desc リスト用縮小画像の幅 (px)
 * @default 140
 *
 * @param サムネイル高さ
 * @desc リスト用縮小画像の高さ (px)
 * @default 110
 *
 * @param 未開放サムネイル
 * @desc 空白部分の画像のファイル名
 * @default gallery_hidden
 *
 * @help
 *
 *
 */

var Esrt = Esrt || {};
Esrt.gallery = Esrt.gallery || {};
Esrt.gallery.topRow = 0;
Esrt.gallery.index = 0;
Esrt.gallery.isFromGallery = false;
Esrt.gallery.isCGMode = 0;
Esrt.galleryListImages = Esrt.galleryListImages || {};
Esrt.gallery.typeSelected = 0;  // 回想から復帰時に元の回想タイプに自動遷移するフラグ
Esrt.gallery.fromTitle = false; // タイトルから回想か、ステージセレクトから回想か
Esrt.gallery.needShuntSave = false; // タイトルから回想か、ステージセレクトから回想か

// ステージセレクトから遷移時の、プレイデータの一時保存先
Esrt.gallery.tmpGameSwitches = null;
Esrt.gallery.tmpGameVariables = null; 
Esrt.gallery.tmpStageScore = null;
Esrt.gallery.tmpEventHistory = null;

Esrt.storyGallery = Esrt.storyGallery || {};
Esrt.storyGallery.topRow = 0;
Esrt.storyGallery.index = 0;

Esrt.WindowMove.gallery = Esrt.WindowMove.gallery || {};
Esrt.WindowMove.gallery.isMoveIn = true;
Esrt.WindowMove.gallery.windowMoveDuration = 20;
Esrt.WindowMove.gallery.windowMoveCount = 0;
Esrt.WindowMove.gallery.windowMoveType = 1;
Esrt.WindowMove.gallery.windowMoveRate = 0;

Esrt.WindowMove.eventGallery = Esrt.WindowMove.eventGallery || {};
Esrt.WindowMove.eventGallery.isMoveIn = true;
Esrt.WindowMove.eventGallery.windowMoveDuration = 20;
Esrt.WindowMove.eventGallery.windowMoveCount = 0;
Esrt.WindowMove.eventGallery.windowMoveType = 1;
Esrt.WindowMove.eventGallery.windowMoveRate = 0;

Esrt.WindowMove.storyGallery = Esrt.WindowMove.storyGallery || {};
Esrt.WindowMove.storyGallery.isMoveIn = true;
Esrt.WindowMove.storyGallery.windowMoveDuration = 20;
Esrt.WindowMove.storyGallery.windowMoveCount = 0;
Esrt.WindowMove.storyGallery.windowMoveType = 1;
Esrt.WindowMove.storyGallery.windowMoveRate = 0;

var $dataGallery  = null;

function Scene_Gallery() {
    this.initialize.apply(this, arguments);
};

function Scene_EventGallery() {
    this.initialize.apply(this, arguments);
};

function Scene_StoryGallery() {
    this.initialize.apply(this, arguments);
};

(function() {
    'use strict';
	var pluginName    = 'ESRT_SceneGallery';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var thumbnailWidth = parseInt(parameters['サムネイル幅']);
    var thumbnailHeigth = parseInt(parameters['サムネイル高さ']);
    var hiddenThumbnail = parameters['未開放サムネイル'];
    var modeVariable       = 81;
    var menuShowSwitch     = 211;
    var menuAllowSwitch    = menuShowSwitch + 6;

    Esrt.galleryListImages.normal = ImageManager.loadPictureTumbnail('gallerySelect');
    Esrt.galleryListImages.focus = ImageManager.loadPictureTumbnail('gallerySelect_focus');
    Esrt.galleryListImages.close = ImageManager.loadPictureTumbnail(hiddenThumbnail);
    Esrt.galleryListImages.closeFocus = ImageManager.loadPictureTumbnail(hiddenThumbnail + 'Focus');

    //=============================================================================
    // DataManager
    //  ギャラリー設定ファイル読み込み処理を定義します
    //=============================================================================
    DataManager.loadGalleryList = function() {
        this.loadDataFile('$dataGallery', 'GalleryList.json');
        return;
    };
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'StoryCommon'){
            this.CollCommon();
        }
    };
    
    Game_Interpreter.prototype.CollCommon = function() {
        var commonEvent = $dataCommonEvents[$gameVariables.value(83)];
        if (commonEvent) {
            var eventId = this.isOnCurrentMap() ? this._eventId : 0;
            this.setupChild(commonEvent.list, eventId);
        }
        return true;
    };

    //=============================================================================
    // Scene_Boot
    //  ステージ設定ファイル読み込み処理を追加定義します
    //=============================================================================
    var _Scene_Boot_create      = Scene_Boot.prototype.create;
    Scene_Boot.prototype.create = function() {
        _Scene_Boot_create.call(this);
        DataManager.loadGalleryList();
    };
    
    //=============================================================================
    // Scene_Map
    //    ウィンドウ移動情報を初期化します
    //=============================================================================
    var _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this, arguments);
        
        Esrt.WindowMove.gallery.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.gallery.isMoveIn;
        Esrt.WindowMove.gallery.windowMoveCount = (Esrt.WindowMove.gallery.isMoveIn) ? 0 : Esrt.WindowMove.gallery.windowMoveDuration;
        
        Esrt.WindowMove.eventGallery.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.eventGallery.isMoveIn;
        Esrt.WindowMove.eventGallery.windowMoveCount = (Esrt.WindowMove.eventGallery.isMoveIn) ? 0 : Esrt.WindowMove.eventGallery.windowMoveDuration;
        
        Esrt.WindowMove.storyGallery.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.storyGallery.isMoveIn;
        Esrt.WindowMove.storyGallery.windowMoveCount = (Esrt.WindowMove.storyGallery.isMoveIn) ? 0 : Esrt.WindowMove.storyGallery.windowMoveDuration;
    };

    //=============================================================================
    // Scene_Gallery
    //  回想種別選択画面を扱うクラスです。
    //=============================================================================
    Scene_Gallery.prototype             = Object.create(Scene_MenuBase.prototype);
    Scene_Gallery.prototype.constructor = Scene_Gallery;

    Scene_Gallery.prototype.initialize = function() {
        $gameSelfSwitches.clear();
        
        Scene_MenuBase.prototype.initialize.call(this);
        
        this._touchInterval = 0;
        
        // ウィンドウ移動演出
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.gallery;
        this._windowMoveObject.windowMoveType = 1;
        this._windowMoveObject.windowMoveRate = 0;
        if (this._windowMoveObject.windowMoveCount > 0 && this._windowMoveObject.windowMoveCount < this._windowMoveObject.windowMoveDuration) {
            this._windowMoveObject.windowMoveCount = this._windowMoveObject.windowMoveDuration;
            this._windowMoveObject.windowMoveType = 0;
            this._windowMoveObject.isMoveIn = false;
        }
        
        if (Esrt.gallery.needShuntSave) {
            Esrt.gallery.needShuntSave = false;
            
            // ステージセレクトから遷移時、プレイデータを一時保存（回想オプションで性経験などを操作できてしまうため）
            Esrt.gallery.tmpGameSwitches  = $gameSwitches._data.concat();
            Esrt.gallery.tmpGameVariables = $gameVariables._data.concat();
            Esrt.gallery.tmpStageScore    = JSON.parse(JSON.stringify(Esrt.stageScore));
            Esrt.gallery.tmpEventHistory = Esrt.eventHistory.concat();
        }
    };
    
    Scene_Gallery.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createCommandWindow();
        this.createButtonWindows();
    };

    Scene_Gallery.prototype.createBackground = function() {
        var width = Graphics.boxWidth * 2;
        var height = Graphics.boxHeight * 2;
    
        this._backgroundSprite = new Sprite();
        this.addChild(this._backgroundSprite);
        
        this._backgroundTile = new TilingSprite(ImageManager.loadTitle1("Background_3"));
        this._backgroundTile.move(0,0, width, height);
        this._backgroundTile.anchor.x = 0.5;
        this._backgroundTile.anchor.y = 0.5;
        this._backgroundSprite.addChild(this._backgroundTile);
    };
    
    Scene_Gallery.prototype.createButtonWindows = function() {
        this._backButtonWindow = new Window_Button(0, 550, 190, 70, 'button_back_left');
        this._backButtonWindow.setHandler('touchInside', this.commandBack.bind(this));
        this.addWindow(this._backButtonWindow);
    }
    
    Scene_Gallery.prototype.createCommandWindow = function() {
        this._commandWindow = new Window_GalleryTypeSelect();
        this._commandWindow.setHandler('event',   this.commandEvent.bind(this));
        this._commandWindow.setHandler('story',   this.commandStory.bind(this));
        this._commandWindow.setHandler('history', this.commandHistory.bind(this));
        this._commandWindow.setHandler('cancel',  this.commandBack.bind(this));
        this.addWindow(this._commandWindow);
    };
    
    Scene_Gallery.prototype.commandEvent = function() {
        SceneManager.push(Scene_EventGallery);
    };
    
    Scene_Gallery.prototype.commandStory = function() {
        SceneManager.push(Scene_StoryGallery);
    };
    
    Scene_Gallery.prototype.commandHistory = function() {
        SceneManager.push(Scene_EventHistory);
    };

    Scene_Gallery.prototype.commandBack = function() {
        if(this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        
        SoundManager.playCancel();
        
        if (Esrt.gallery.tmpGameSwitches) {
            // ステージセレクトから遷移時、プレイデータを元に戻す
            $gameSwitches._data = Esrt.gallery.tmpGameSwitches.concat();
            $gameVariables._data = Esrt.gallery.tmpGameVariables.concat();
            Esrt.eventHistory = Esrt.gallery.tmpEventHistory.concat();
            Esrt.stageScore = JSON.parse(JSON.stringify(Esrt.gallery.tmpStageScore));
            ConfigManager.importCustomParams();
            
            Esrt.gallery.tmpGameSwitches = null;
            Esrt.gallery.tmpGameVariables = null;
            Esrt.gallery.tmpStageScore = null;
            Esrt.gallery.tmpEventHistory = null;
        }
        
        if (Esrt.gallery.fromTitle) {
            Esrt.gallery.fromTitle = false;
            SceneManager.goto(Scene_Title);
            
        } else {
            $gameSelfSwitches.clear();
            $gamePlayer.reserveTransfer(2, 1, 1);
            this.popScene();
        }
    };
    
    Scene_Gallery.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
        
        if (this._touchInterval > 0) { this._touchInterval -= 1; }
        
         // 回想から復帰時に元の回想タイプに自動遷移する
        if (Esrt.gallery.typeSelected == 1) { SceneManager.push(Scene_EventGallery); }
        if (Esrt.gallery.typeSelected == 2) { SceneManager.push(Scene_StoryGallery); }
        if (Esrt.gallery.typeSelected == 3) { SceneManager.push(Scene_EventHistory); }
        
    	this.updateBackgroundEffects();
    }
    
    Scene_Gallery.prototype.updateBackgroundEffects = function() {
    	this._backgroundTile.origin.x += 0.5;
    	this._backgroundTile.origin.y += 0.5;	
    };
    
    // オプションから戻ったときにはムーブイン演出を行わないようにフラグを立てる
    Scene_Gallery.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.gallery.isMoveIn = false;
        Esrt.WindowMove.gallery.windowMoveCount = Esrt.WindowMove.gallery.windowMoveDuration;
    }
    
    //=============================================================================
    // Window_GalleryTypeSelect
    //  ギャラリーリストウィンドウです。
    //=============================================================================
    function Window_GalleryTypeSelect() {
        this.initialize.apply(this, arguments);
    }

    Window_GalleryTypeSelect.prototype             = Object.create(Window_Command.prototype);
    Window_GalleryTypeSelect.prototype.constructor = Window_GalleryTypeSelect;

    Window_GalleryTypeSelect.prototype.initialize = function() {
        // ウィンドウ移動演出
        this._windowPositionDirection = 2; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.gallery;
        
        Window_Command.prototype.initialize.call(this, 250, 200);
    };

    Window_GalleryTypeSelect.prototype.windowWidth = function() {
        return 300;
    };

    Window_GalleryTypeSelect.prototype.makeCommandList = function() {
        
        this.addCommand("敗北イベント回想", 'event');
        this.addCommand("ストーリー回想",   'story');
        
        var historyText = "セックス履歴";
        historyText += (Esrt.gallery.fromTitle) ? "（ゲーム中のみ）" : "";
        var historyEnabled = (!Esrt.gallery.fromTitle) ? true : false;
        
        this.addCommand(historyText,       'history', historyEnabled);
    };

    Window_GalleryTypeSelect.prototype.lineHeight = function() {
        return 72;
    };
    
    //=============================================================================
    // Scene_EventGallery
    //  敗北シーン回想画面を扱うクラスです。
    //=============================================================================
    Scene_EventGallery.prototype             = Object.create(Scene_MenuBase.prototype);
    Scene_EventGallery.prototype.constructor = Scene_EventGallery;

    Scene_EventGallery.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);

        this._startFlag = false;
        $gameVariables.setValue(modeVariable, Esrt.gallery.isCGMode);
        if (!$gameSwitches.value(137)) {
            // 全開放中は開放状態をリセットしない
            //   ※リセットしないと回想を見ることで次の段階が未見でも回想が開いてしまう
            utakata.CommonSaveManager.load();
        }

        this._touchInterval = 0;
        this._backgroundLoading = false;
        
        // ウィンドウ移動演出
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.eventGallery;
        this._windowMoveObject.windowMoveType = 1;
        this._windowMoveObject.windowMoveRate = 0;
        if (this._windowMoveObject.windowMoveCount > 0 && this._windowMoveObject.windowMoveCount < this._windowMoveObject.windowMoveDuration) {
            this._windowMoveObject.windowMoveCount = this._windowMoveObject.windowMoveDuration;
            this._windowMoveObject.windowMoveType = 0;
            this._windowMoveObject.isMoveIn = false;
        }
        Esrt.gallery.typeSelected = 0;
    };

    Scene_EventGallery.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createHelpWindow();
        this.createGalleryListWindow();
        this.createButtonWindows();
    };

    Scene_EventGallery.prototype.createBackground = function() {
        var width = Graphics.boxWidth * 2;
        var height = Graphics.boxHeight * 2;
    
        this._backgroundSprite = new Sprite();
        this.addChild(this._backgroundSprite);
        
        this._backgroundTile = new TilingSprite(ImageManager.loadTitle1("Background_3"));
        this._backgroundTile.move(0,0, width, height);
        this._backgroundTile.anchor.x = 0.5;
        this._backgroundTile.anchor.y = 0.5;
        this._backgroundSprite.addChild(this._backgroundTile);
    };

    Scene_EventGallery.prototype.createHelpWindow = function() {
        this._helpWindow = new Window_Help(3);
        this.addWindow(this._helpWindow);
        
        this._helpWindow.setWindowPositionDirection(8);
        this._helpWindow.setWindowMoveInOut(true);
        this._helpWindow.setWindowMoveObject(Esrt.WindowMove.eventGallery);
    };
    
    Scene_EventGallery.prototype.createGalleryListWindow = function() {
        var wy              = this._helpWindow.height;
        var wh              = Graphics.boxHeight - wy - 100;
        var ww              = Graphics.boxWidth;

        this._galleryListWindow = new Window_EventGalleryList(0, wy, ww, wh);
        this._galleryListWindow.setHelpWindow(this._helpWindow);
        this._galleryListWindow.setHandler('ok', this.commandPlayGallery.bind(this));
        this._galleryListWindow.setHandler('cancel', this.commandBack.bind(this));
        this._galleryListWindow.setHandler('shift', this.commandModeChange.bind(this));
        this._galleryListWindow.setHandler('control', this.commandOption.bind(this));
        this._galleryListWindow.setHandler('touchInside', this.activateGalleryList.bind(this));
        this._galleryListWindow.activate();
        this.addWindow(this._galleryListWindow);

    };
    
    Scene_EventGallery.prototype.createButtonWindows = function() {
        var mode = $gameVariables.value(modeVariable);
        var modeButton = (Number(mode) === 0) ? 'button_galleryMode1' : 'button_galleryMode2';
        this._changeButtonWindow = new Window_Button(630, 550, 190, 70, modeButton);
        this._changeButtonWindow.setHandler('touchInside', this.commandModeChange.bind(this));
        this.addWindow(this._changeButtonWindow);

        this._backButtonWindow = new Window_Button(0, 550, 190, 70, 'button_back_left');
        this._backButtonWindow.setHandler('touchInside', this.commandBack.bind(this));
        this.addWindow(this._backButtonWindow);

        this._optionButtonWindow = new Window_Button(300, 550, 190, 70, 'button_option_center');
        this._optionButtonWindow.setHandler('touchInside', this.commandOption.bind(this));
        this.addWindow(this._optionButtonWindow);
    }

    Scene_EventGallery.prototype.activateGalleryList = function() {
        AudioManager.playSe({"name":"Coin","volume":100,"pitch":100,"pan":0});
        this._galleryListWindow.activate();
    };

    Scene_EventGallery.prototype.commandModeChange = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        SoundManager.playOk();
        $gameVariables.setValue(modeVariable, ($gameVariables.value(modeVariable) + 1 ) % 2);
        var mode = $gameVariables.value(modeVariable);
        if(Number(mode) === 0){
            this._changeButtonWindow.setSprite('button_galleryMode1');
        }else{
            this._changeButtonWindow.setSprite('button_galleryMode2');
        }
    };

    Scene_EventGallery.prototype.commandBack = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        SoundManager.playCancel();
        this.popScene();
    };

    Scene_EventGallery.prototype.commandOption = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        Esrt.gallery._isClickLeftKeyForOption = true;
        SoundManager.playOk();
        SceneManager.push(Scene_Options);
    };

    Scene_EventGallery.prototype.commandPlayGallery = function() {
        if(this._startFlag === true ){ // 複数呼び出されることに対する対処
            return;
        }
        this._startFlag = true;
        Esrt.gallery.topRow = this._galleryListWindow.topRow();
        Esrt.gallery.index = this._galleryListWindow.index();
        Esrt.gallery.isFromGallery = true;
        Esrt.gallery.typeSelected  = 1; // 回想閲覧後敗北回想画面に戻るためのフラグ
        
        this._windowMoveObject.windowMoveType = -1;

        Esrt.gallery.isCGMode = $gameVariables.value(modeVariable);

        for(var i = 0; i <= 100; i++){$gameScreen.erasePicture(i)}

        if(Esrt.gallery.isCGMode){
            $gameSwitches.setValue(menuShowSwitch, false);
            $gameSwitches.setValue(menuAllowSwitch, false);
        }else{
            $gameSwitches.setValue(menuShowSwitch, true);
            $gameSwitches.setValue(menuAllowSwitch, true);
        }

        $gamePlayer.setTransparent(255);
        this.fadeOutAll();

        // 変数の指定
        $gameVariables.setValue(201,0); // 受精カウントを0に初期化
        
        var item = this._galleryListWindow.item();
        var isStreet = (item.isStreet) ? true : false;
        var isNakedness = (item.isNakedness) ? true : false;
        var fertilizationId = (item.fertilizationId) ? item.fertilizationId : 0;
        var fertilizationFlag = (item.fertilizationId > 0) ? true : false;
        var endId = (item.endId) ? item.endId : 0;
        var ifId = (item.ifId) ? item.ifId : 0;
        $gameSwitches.setValue(102,item.forceTranse);    // 強制変身フラグ
        $gameSwitches.setValue(105,isStreet);            // 大通り変身フラグ
        $gameSwitches.setValue(115,item.discharge);      // 変身解除フラグ
        $gameSwitches.setValue(116,item.trap);           // 束縛状態フラグ
        $gameSwitches.setValue(120,isNakedness);         // 半裸フラグ
        $gameSwitches.setValue(132,true);                // 敗北メッセージウィンドウフラグ
        $gameSwitches.setValue(136,fertilizationFlag);   // 受精回想フラグ
        $gameVariables.setValue(111,item.bossBattleId);  // ボス戦闘フラグ
        $gameVariables.setValue(161,item.stageId);       // ステージ番号
        $gameVariables.setValue(206,fertilizationId);    // 受精イベント振り分け変数
        $gameVariables.setValue(207,endId);              // エンディング振分フラグ
        $gameVariables.setValue(208,ifId);                 // if振分フラグ

        if(item.setVariables){
            Object.keys(item.setVariables).forEach(function(key) {
                $gameVariables.setValue(key, item.setVariables[key]);
            });
        }

        $gamePlayer.reserveTransfer(7, 0, 0, 0);
        this.popScene();
        this.popScene();

        this._galleryListWindow.activate();
    };

    Scene_EventGallery.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
        if(this._touchInterval > 0){
            this._touchInterval -= 1;
        }
        this.updateBackgroundEffects();
    };
    
    Scene_EventGallery.prototype.updateBackgroundEffects = function() {
    	this._backgroundTile.origin.x += 0.5;
    	this._backgroundTile.origin.y += 0.5;	
    };
    
    // オプションから戻ったときにはムーブイン演出を行わないようにフラグを立てる
    Scene_EventGallery.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.eventGallery.isMoveIn = false;
        Esrt.WindowMove.eventGallery.windowMoveCount = Esrt.WindowMove.eventGallery.windowMoveDuration;
    }

    //=============================================================================
    // Window_EventGalleryList
    //  ギャラリーリストウィンドウです。
    //=============================================================================
    function Window_EventGalleryList() {
        this.initialize.apply(this, arguments);
    }

    Window_EventGalleryList.prototype             = Object.create(Window_Selectable.prototype);
    Window_EventGalleryList.prototype.constructor = Window_EventGalleryList;

    Window_EventGalleryList.prototype.initialize = function(x, y, width, height) {
        // ウィンドウ移動演出
        this._windowPositionDirection = 2; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.eventGallery;
        
        Window_Selectable.prototype.initialize.call(this, x, y, width, height);
        this._data = [];
        this.thumIndex = 0;
        this.refresh();
        this.setTopRow(Esrt.gallery.topRow);
        this.select(Esrt.gallery.index);
    };

    var Window_EventGalleryList_select = Window_EventGalleryList.prototype.select;
    Window_EventGalleryList.prototype.select = function(index) {
        Window_EventGalleryList_select.apply(this, arguments);
        this.refresh();
    };

    // ● 行数の取得
    Window_EventGalleryList.prototype.maxCols = function() {
        return Math.floor(this.contentsWidth() / thumbnailWidth);
    };

    // ● 項目の高さの取得
    Window_EventGalleryList.prototype.itemHeight = function() {
        var rows = Math.floor(this.contentsHeight() / thumbnailHeigth);
        return Math.floor(this.contentsHeight() / rows);
    };

    Window_EventGalleryList.prototype.maxItems = function() {
        return this._data ? this._data.length : 1;
    };

    Window_EventGalleryList.prototype.makeItemList = function() {
        if(!$dataGallery){ return; }
        
        this._data = [];
        
        $dataGallery.galleryList.forEach(function(key) {
            if ($dataGallery.eventData[key]) {this._data.push($dataGallery.eventData[key])};
        }, this);
    };

    Window_EventGalleryList.prototype.item = function() {
        var index = this.index();
        return this._data && index >= 0 ? this._data[index] : null;
    };

    Window_EventGalleryList.prototype.isEnabled = function(item) {
        if(!item) return false;

        var value = $gameVariables.value(item.requireVariable)
        var result = (value >= item.requireValue) ? true : false;
        return result;
    };

    Window_EventGalleryList.prototype.isCurrentItemEnabled = function() {
        return this.isEnabled(this._data[this.index()]);
    };

    Window_EventGalleryList.prototype.drawItem = function(index) {
        var item = this._data[index];
        if (item) {
            var rect = this.itemRect(index);
            rect.width -= this.textPadding();
            var focusFlag = false;
            if(index === this.index()){
                focusFlag = true;
            }
            this.drawPicture(index, item, focusFlag);
        }
    };

    // ● ピクチャーの描画
    Window_EventGalleryList.prototype.drawPicture = function(index, item, flag) {
        var rect = this.itemRect(index);
        var width = rect.width || thumbnailWidth;
        var height = rect.height || thumbnailHeigth;

        this.thumIndex = typeof item.thumIndex != "undefined" ? item.thumIndex : this.thumIndex;

        var pw = thumbnailWidth;
        var ph = thumbnailHeigth;
        var sw = Math.min(width, pw);
        var sh = Math.min(height, ph);
        var dx = Math.floor(rect.x + Math.max(width - pw, 0) / 2);
        var dy = Math.floor(rect.y + Math.max(height - ph, 0) / 2);
        var sx = (pw - sw) / 2 + (this.thumIndex % 5) * pw;
        var sy = (ph - sh) / 2 + Math.floor(this.thumIndex / 5) * ph;
        var bitmap = Esrt.galleryListImages.normal;
        if(this.isEnabled(item)){
            if(flag){
                bitmap = Esrt.galleryListImages.focus;
            }else{
                bitmap = Esrt.galleryListImages.normal;
            }
        }else{
            if(flag){
                bitmap = Esrt.galleryListImages.closeFocus;
            }else{
                bitmap = Esrt.galleryListImages.close;
            }
        }
        this.contents.blt(bitmap, sx, sy, sw, sh, dx, dy);
        this.thumIndex += 1;
    };

    Window_EventGalleryList.prototype.drawItemName = function(item, x, y, width) {
        this.resetTextColor();
        this.drawText(this.isEnabled(item) ? item.displayName : '？？？', x, y, width);
    };

    Window_EventGalleryList.prototype.updateHelp = function() {
        var item = this.item();
        this._helpWindow.setText(item ? this.getHelp(item) : '');
    };

    Window_EventGalleryList.prototype.getHelp = function(item) {
        return this.isEnabled(item) ?
             '【' + item.displayName + '】' : item.hint;
    };

    Window_EventGalleryList.prototype.refresh = function() {
        this.makeItemList();
        this.createContents();
        this.drawAllItems();
    };

    Window_EventGalleryList.prototype.processCancel = function() {
        Window_Selectable.prototype.processCancel.call(this);
        Esrt.gallery.topRow = this.topRow();
        Esrt.gallery.index = this.index();
    };
    
    //=============================================================================
    //  ・タップ時にアクティブ化処理
    //  ・シフト押下時にシフトハンドラー追加処理
    //    ・トリアコンタン様のサウンドテストプラグインを参考にしました。
    //=============================================================================
    var _Window_Selectable_processTouch      = Window_Selectable.prototype.processTouch;
    Window_EventGalleryList.prototype.processTouch = function() {
        Window_Selectable.prototype.processTouch.apply(this, arguments);
        if (this._canProcessTouchNonActive) {
            this.processTouchNonActive();
        }
    };

    Window_EventGalleryList.prototype.processTouchNonActive = function() {
        if (this.isOpen() && !this.active && this.isTouchedInsideFrame() &&
            this.isHandled('touchInside') && TouchInput.isTriggered()) {
            this.callHandler('touchInside');
        }
    };

    Window_EventGalleryList.prototype.processShift = function() {
        this.callHandler('shift');
    };
    
    Window_EventGalleryList.prototype.processControl = function() {
        this.callHandler('control');
    };

    Window_EventGalleryList.prototype.processHandling = function() {
        Window_Selectable.prototype.processHandling.apply(this, arguments);
        if (this.isOpenAndActive()) {
            if (this.isShiftEnabled() && this.isShiftTriggered()) {
                this.processShift();
            }
            if (this.isControlEnabled() && this.isControlTriggered()) {
                this.processControl();
            }
        }
    };

    Window_EventGalleryList.prototype.isShiftEnabled = function() {
        return this.isHandled('shift');
    };

    Window_EventGalleryList.prototype.isShiftTriggered = function() {
        return Input.isRepeated('shift');
    };

    Window_EventGalleryList.prototype.isControlEnabled = function() {
        return this.isHandled('control');
    };

    Window_EventGalleryList.prototype.isControlTriggered = function() {
        return Input.isRepeated('control');
    };
    
    //=============================================================================
    // Scene_StoryGallery
    //  ストーリーシーン回想画面を扱うクラスです。
    //=============================================================================
    Scene_StoryGallery.prototype             = Object.create(Scene_MenuBase.prototype);
    Scene_StoryGallery.prototype.constructor = Scene_StoryGallery;

    Scene_StoryGallery.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);

        this._startFlag = false;
        utakata.CommonSaveManager.load();

        this._touchInterval = 0;
        this._backgroundLoading = false;
        
        // ウィンドウ移動演出
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.storyGallery;
        this._windowMoveObject.windowMoveType = 1;
        this._windowMoveObject.windowMoveRate = 0;
        if (this._windowMoveObject.windowMoveCount > 0 && this._windowMoveObject.windowMoveCount < this._windowMoveObject.windowMoveDuration) {
            this._windowMoveObject.windowMoveCount = this._windowMoveObject.windowMoveDuration;
            this._windowMoveObject.windowMoveType = 0;
            this._windowMoveObject.isMoveIn = false;
        }
        Esrt.gallery.typeSelected = 0;
    };

    Scene_StoryGallery.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createGalleryListWindow();
        this.createButtonWindows();
    };

    Scene_StoryGallery.prototype.createBackground = function() {
        var width = Graphics.boxWidth * 2;
        var height = Graphics.boxHeight * 2;
    
        this._backgroundSprite = new Sprite();
        this.addChild(this._backgroundSprite);
        
        this._backgroundTile = new TilingSprite(ImageManager.loadTitle1("Background_3"));
        this._backgroundTile.move(0,0, width, height);
        this._backgroundTile.anchor.x = 0.5;
        this._backgroundTile.anchor.y = 0.5;
        this._backgroundSprite.addChild(this._backgroundTile);
    };
    
    Scene_StoryGallery.prototype.createGalleryListWindow = function() {
        var wy              = 0;
        var wh              = Graphics.boxHeight - wy - 100;
        var ww              = Graphics.boxWidth - 200;

        this._galleryListWindow = new Window_StoryGalleryList(100, wy, ww, wh);
        this._galleryListWindow.setHandler('ok', this.commandPlayGallery.bind(this));
        this._galleryListWindow.setHandler('cancel', this.commandBack.bind(this));
        this._galleryListWindow.setHandler('touchInside', this.activateGalleryList.bind(this));
        this._galleryListWindow.activate();
        this.addWindow(this._galleryListWindow);

    };
    
    Scene_StoryGallery.prototype.createButtonWindows = function() {
        this._backButtonWindow = new Window_Button(0, 550, 190, 70, 'button_back_left');
        this._backButtonWindow.setHandler('touchInside', this.commandBack.bind(this));
        this.addWindow(this._backButtonWindow);
    }

    Scene_StoryGallery.prototype.activateGalleryList = function() {
        AudioManager.playSe({"name":"Coin","volume":100,"pitch":100,"pan":0});
        this._galleryListWindow.activate();
    };

    Scene_StoryGallery.prototype.commandModeChange = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        SoundManager.playOk();
        $gameVariables.setValue(modeVariable, ($gameVariables.value(modeVariable) + 1 ) % 2);
        var mode = $gameVariables.value(modeVariable);
        if(Number(mode) === 0){
            this._changeButtonWindow.setSprite('button_galleryMode1');
        }else{
            this._changeButtonWindow.setSprite('button_galleryMode2');
        }
    };

    Scene_StoryGallery.prototype.commandBack = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        SoundManager.playCancel();
        this.popScene();
    };

    Scene_StoryGallery.prototype.commandOption = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        Esrt.gallery._isClickLeftKeyForOption = true;
        SoundManager.playOk();
        SceneManager.push(Scene_Options);
    };

    Scene_StoryGallery.prototype.commandPlayGallery = function() {
        if(this._startFlag === true ){ // 複数呼び出されることに対する対処
            return;
        }
        this._startFlag = true;
        Esrt.storyGallery.topRow = this._galleryListWindow.topRow();
        Esrt.storyGallery.index = this._galleryListWindow.index();
        Esrt.gallery.isFromGallery = true;
        Esrt.gallery.typeSelected  = 2; // 回想閲覧後ストーリー回想画面に戻るためのフラグ
        
        this._windowMoveObject.windowMoveType = -1;

        for(var i = 0; i <= 100; i++){$gameScreen.erasePicture(i)}

        $gameSwitches.setValue(menuShowSwitch, true);
        $gameSwitches.setValue(menuAllowSwitch, true);

        $gamePlayer.setTransparent(255);
        this.fadeOutAll();

        // 変数の指定
        
        var item = this._galleryListWindow.item();
        $gameVariables.setValue(83, Number(item.commonId)); // コモンイベント番号
        $gameVariables.setValue(162, 0);  // ギャラリー帰還用
        $gameVariables.setValue(163, 0);  // ギャラリー帰還用
        $gameVariables.setValue(164, 35); // ギャラリー帰還用

        if(item.setVariables){
            Object.keys(item.setVariables).forEach(function(key) {
                $gameVariables.setValue(key, item.setVariables[key]);
            });
        }

        $gamePlayer.reserveTransfer(220, 0, 0, 0);
        this.popScene();
        this.popScene();

        this._galleryListWindow.activate();
    };

    Scene_StoryGallery.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
        if(this._touchInterval > 0){
            this._touchInterval -= 1;
        }
        this.updateBackgroundEffects();
    };
    
    Scene_StoryGallery.prototype.updateBackgroundEffects = function() {
    	this._backgroundTile.origin.x += 0.5;
    	this._backgroundTile.origin.y += 0.5;	
    };
    
    // オプションから戻ったときにはムーブイン演出を行わないようにフラグを立てる
    Scene_StoryGallery.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.storyGallery.isMoveIn = false;
        Esrt.WindowMove.storyGallery.windowMoveCount = Esrt.WindowMove.storyGallery.windowMoveDuration;
    }

    //=============================================================================
    // Window_StoryGalleryList
    //  ストーリーシーンリストウィンドウです。
    //=============================================================================
    function Window_StoryGalleryList() {
        this.initialize.apply(this, arguments);
    }

    Window_StoryGalleryList.prototype             = Object.create(Window_Selectable.prototype);
    Window_StoryGalleryList.prototype.constructor = Window_StoryGalleryList;

    Window_StoryGalleryList.prototype.initialize = function(x, y, width, height) {
        // ウィンドウ移動演出
        this._windowPositionDirection = 2; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.storyGallery;
        
        Window_Selectable.prototype.initialize.call(this, x, y, width, height);
        this._data = [];
        this.thumIndex = 0;
        this.refresh();
        this.setTopRow(Esrt.storyGallery.topRow);
        this.select(Esrt.storyGallery.index);
    };

    var Window_StoryGalleryList_select = Window_StoryGalleryList.prototype.select;
    Window_StoryGalleryList.prototype.select = function(index) {
        Window_StoryGalleryList_select.apply(this, arguments);
        this.refresh();
    };

    Window_StoryGalleryList.prototype.maxItems = function() {
        return this._data ? this._data.length : 1;
    };

    Window_StoryGalleryList.prototype.makeItemList = function() {
        if(!$dataGallery){ return; }
        this._data = $dataGallery.story.filter(function(item) {
            return item != null;
        }, this);
    };

    Window_StoryGalleryList.prototype.item = function() {
        var index = this.index();
        return this._data && index >= 0 ? this._data[index] : null;
    };

    Window_StoryGalleryList.prototype.isEnabled = function(item) {
        if(!item) return false;

        var value = $gameVariables.value(item.requireVariable)
        var result = (value >= item.requireValue) ? true : false;
        return result;
    };

    Window_StoryGalleryList.prototype.isCurrentItemEnabled = function() {
        return this.isEnabled(this._data[this.index()]);
    };

    Window_StoryGalleryList.prototype.drawItem = function(index) {
        var item = this._data[index];
        
        var rect = this.itemRectForText(index);
        
        if (item) {
            this.drawItemName(item, rect.x, rect.y, this.contentsWidth());
        }
    };

    Window_StoryGalleryList.prototype.drawItemName = function(item, x, y, width) {
        this.resetTextColor();
        if (!this.isEnabled(item)) {
            this.changeTextColor(this.textColor(8));
        }
        this.drawText(this.isEnabled(item) ? "【" + item.stageText + "】" + item.sceneText :  "【" + item.stageText + "】", x, y, width);
    };

    Window_StoryGalleryList.prototype.refresh = function() {
        this.makeItemList();
        this.createContents();
        this.drawAllItems();
    };

    Window_StoryGalleryList.prototype.processCancel = function() {
        Window_Selectable.prototype.processCancel.call(this);
        Esrt.storyGallery.topRow = this.topRow();
        Esrt.storyGallery.index = this.index();
    };
    
    //=============================================================================
    //  ・タップ時にアクティブ化処理
    //  ・シフト押下時にシフトハンドラー追加処理
    //    ・トリアコンタン様のサウンドテストプラグインを参考にしました。
    //=============================================================================
    var _Window_Selectable_processTouch      = Window_Selectable.prototype.processTouch;
    Window_StoryGalleryList.prototype.processTouch = function() {
        Window_Selectable.prototype.processTouch.apply(this, arguments);
        if (this._canProcessTouchNonActive) {
            this.processTouchNonActive();
        }
    };

    Window_StoryGalleryList.prototype.processTouchNonActive = function() {
        if (this.isOpen() && !this.active && this.isTouchedInsideFrame() &&
            this.isHandled('touchInside') && TouchInput.isTriggered()) {
            this.callHandler('touchInside');
        }
    };

    Window_StoryGalleryList.prototype.processShift = function() {
        this.callHandler('shift');
    };
    
    Window_StoryGalleryList.prototype.processControl = function() {
        this.callHandler('control');
    };

    Window_StoryGalleryList.prototype.processHandling = function() {
        Window_Selectable.prototype.processHandling.apply(this, arguments);
        if (this.isOpenAndActive()) {
            if (this.isShiftEnabled() && this.isShiftTriggered()) {
                this.processShift();
            }
            if (this.isControlEnabled() && this.isControlTriggered()) {
                this.processControl();
            }
        }
    };

    Window_StoryGalleryList.prototype.isShiftEnabled = function() {
        return this.isHandled('shift');
    };

    Window_StoryGalleryList.prototype.isShiftTriggered = function() {
        return Input.isRepeated('shift');
    };

    Window_StoryGalleryList.prototype.isControlEnabled = function() {
        return this.isHandled('control');
    };

    Window_StoryGalleryList.prototype.isControlTriggered = function() {
        return Input.isRepeated('control');
    };
})();
