//=============================================================================
// ESRT_SceneMenu_override.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc メニュー画面上書き
 * @author 白
 *
 * @param リトライ表示フラグ
 * @desc (ミニステータス表示フラグを使う)表示のオンオフに使用するスイッチを指定します。
 * @default 114
 *
 * @help 
 *
 *
 */
 
var Esrt = Esrt || {};
Esrt.hoverButton   = Esrt.hoverButton || {};
Esrt.imageReservationIds = Esrt.imageReservationIds || {};

(function() {
    'use strict';
	var pluginName    = 'ESRT_SceneMenu_override';
    
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var retryShowSwitch = parseInt(parameters['リトライ表示フラグ']);
    
    //=============================================================================
    // Scene_Menu
	//  メニューシーンでステータス・所持金ウィンドウを表示しない
	//  メニューコマンドにリトライ・ステージ選択へ戻るを追加する
    //=============================================================================
	// OVERRIDE
    Scene_Menu.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createCommandWindow();
        this.createStatusWindow();
    };
    
    Scene_Menu.prototype.start = function() {
        Scene_MenuBase.prototype.start.call(this);
    };
    
    var _Scene_Menu_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
    Scene_Menu.prototype.createCommandWindow = function() {
        _Scene_Menu_createCommandWindow.call(this);
        
        this._commandWindow.setHandler('stageSelect',this.commandStageSelect.bind(this));
        this._commandWindow.setHandler('retray',     this.commandRetray.bind(this));
        this._commandWindow.setHandler('gallery',    this.commandGallery.bind(this));
        this._commandWindow.setHandler('back',     this.commandBack.bind(this));
    };
    
    var _Scene_Menu_createStatusWindow = Scene_Menu.prototype.createStatusWindow;
    Scene_Menu.prototype.createStatusWindow = function() {
        this._statusWindow = new Window_MenuStatusBack(this._commandWindow.width, 0);
        this.addWindow(this._statusWindow);
        this._statusWindow = new Window_MenuStatus(this._commandWindow.width, 0);
        this.addWindow(this._statusWindow);
    };
    
    Scene_Menu.prototype.commandStageSelect = function() {
        $gameSelfSwitches.clear();
        $gamePlayer.reserveTransfer(2, 1, 1);
        this.popScene();
    };
    
    Scene_Menu.prototype.commandRetray = function() {
        $gameSelfSwitches.clear();
        $gamePlayer.setTransparent(true);
        $gamePlayer.reserveTransfer(36, 2, 2);
        this.popScene();
    };
    
    Scene_Menu.prototype.commandGallery = function() {
        
        if (Esrt.gallery)     { Esrt.gallery.needShuntSave = true; }
        if (Esrt.stageSelect) { Esrt.stageSelect.onceSkipAutoSave = true; }
        
        $gameSelfSwitches.clear();
        $gamePlayer.setTransparent(true);
        $gamePlayer.reserveTransfer(35, 2, 2);
        this.popScene();
        this.popScene();
    };
    
    Scene_Menu.prototype.commandBack = function() {
        $gameVariables.setValue(104,5);
        this.popScene();
    };
    
    var _Window_MenuCommand_makeCommandList = Window_MenuCommand.prototype.makeCommandList;
    Window_MenuCommand.prototype.makeCommandList = function() {
        var enabled = $gameSwitches.value(retryShowSwitch) ? true : false;
        
        this.addCommand("ゲームに戻る", 'back', true);
        this.addCommand("ステージ選択", 'stageSelect', enabled);
        this.addCommand("リトライ", 'retray', enabled);
        this.addCommand("回想", 'gallery', !enabled);
        
        _Window_MenuCommand_makeCommandList.call(this);
        
    };
    
    var _Window_MenuStatus_initialize = Window_MenuStatus.prototype.initialize;
    Window_MenuStatus.prototype.initialize = function() {
        _Window_MenuStatus_initialize.apply(this, arguments);
    };
    
    Window_MenuStatus.prototype.drawAllItems = function() {
        this.addDrawItem();
    };
    
    Window_MenuStatus.prototype.addDrawItem = function() {
        var x = 170;
        var y = 10 + this.lineHeight();
        // this.changeTextColor(this.systemColor());
        // this.drawText("Lv", x + 50, y, 134);
        // this.resetTextColor();
        // this.drawText(12, x + 200, y);
        // y += this.lineHeight();
        // 
        // this.changeTextColor(this.systemColor());
        // this.drawText("ジョブLv", x + 50, y, 134);
        // this.resetTextColor();
        // this.drawText(" 6", x + 200, y);
        // y += this.lineHeight() * 2;
        
        {
            var score = Esrt.stageScoreFanc.GetAllStageHighCoin(Esrt.gameModes.normal);
            this.changeTextColor(this.systemColor());
            var iconId = (score[0] >= score[1]) ? 88 : 90;
            this.drawIcon(iconId, x + 160, y);
            this.drawText("獲得魔法石", x + 10, y, 134);
            this.resetTextColor();
            this.drawText(score[0] + "／" + score[1], x + 220, y);
        }
        y += this.lineHeight();
        
        if($gameSwitches.value(182)){
            var score = Esrt.stageScoreFanc.GetAllStageHighCoin(Esrt.gameModes.extra);
            this.changeTextColor(this.systemColor());
            var iconId = (score[0] >= score[1]) ? 87 : 90;
            this.drawIcon(iconId, x + 160, y);
            this.drawText("獲得魔法石(Ex)", x + 10, y, 134);
            this.resetTextColor();
            this.drawText(score[0] + "／" + score[1], x + 220, y);
        }
        y += this.lineHeight() * 2;
        
        {
            var count = $gameVariables.value(201);
            var max   = $gameVariables.value(202) > 5 ? "∞" : $gameVariables.value(202);
            this.changeTextColor(this.systemColor());
            this.drawIcon(Esrt.GetFertilizationIconNumber(), x + 160, y);
            this.drawText("受精カウント", x + 10, y, 134);
            this.resetTextColor();
            this.drawText(count + "／" + max, x + 220, y);
        }
        y += this.lineHeight() * 3;
        
        x = 10;
        this.changeTextColor(this.systemColor());
        this.drawText("処女", x + 50, y, 134);
        this.resetTextColor();
        var penisText = $gameVariables.value(241) == 0 ? "完全処女" : $gameVariables.value(241) == 1 ? "変身前は処女" : "経験済";
        this.drawText(penisText, x + 220, y);
        
        y += this.lineHeight();
        this.changeTextColor(this.systemColor());
        this.drawText("おちんちん", x + 50, y, 134);
        this.resetTextColor();
        var penisText = $gameVariables.value(221) == 0 ? "パパのは見た" : $gameVariables.value(221) == 1 ? "知ってる" : "見慣れた";
        this.drawText(penisText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("保健", x + 50, y, 134);
        this.resetTextColor();
        var healthText = $gameVariables.value(222) == 0 ? "習っただけ" : "実習済み";
        this.drawText(healthText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("射精", x + 50, y, 134);
        this.resetTextColor();
        var ejaculationText = $gameVariables.value(223) == 0 ? "見たことない" : $gameVariables.value(223) == 1 ? "見たことある" : "何度も見た";
        this.drawText(ejaculationText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("手コキ", x + 50, y, 134);
        this.resetTextColor();
        var pregnancyText = $gameVariables.value(224) == 0 ? "知らない" : $gameVariables.value(224) == 1 ? "したことある" : "何度もした";
        this.drawText(pregnancyText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("フェラ", x + 50, y, 134);
        this.resetTextColor();
        var pregnancyText = $gameVariables.value(225) == 0 ? "知らない" : $gameVariables.value(225) == 1 ? "したことある" : "何度もした";
        this.drawText(pregnancyText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("セックス", x + 50, y, 134);
        this.resetTextColor();
        var sexText = $gameVariables.value(226) == 0 ? "知らない" : $gameVariables.value(226) == 1 ? "したことある" : "よくする";
        this.drawText(sexText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("妊娠", x + 50, y, 134);
        this.resetTextColor();
        var pregnancyText = $gameVariables.value(227) == 0 ? "したことない" :　$gameVariables.value(227) == 1 ? "怖い" : "なんとかなった";
        this.drawText(pregnancyText, x + 220, y);
        y += this.lineHeight();
        
        this.changeTextColor(this.systemColor());
        this.drawText("苗床", x + 50, y, 134);
        this.resetTextColor();
        var pregnancyText = $gameVariables.value(228) == 0 ? "なにそれ" :　"なんとかなった";
        this.drawText(pregnancyText, x + 220, y);
        y += this.lineHeight();
    };
    
    Window_MenuStatus.prototype.systemColor = function() {
        return this.textColor(16);
    };
    
    Window_MenuStatus.prototype.drawActorSimpleStatus = function(actor, x, y, width) {
        var lineHeight = this.lineHeight();
        var x2 = x + 180;
        var width2 = Math.min(200, width - 180 - this.textPadding());
        this.drawActorLevel(actor, 10, 10);
    };
    //=============================================================================
    // Window_MenuStatusBack
    //    ミニステータスウィンドウを定義します。
    //=============================================================================
    function Window_MenuStatusBack() {
        this.initialize.apply(this, arguments);
    }
    Window_MenuStatusBack.prototype = Object.create(Window_MenuStatus.prototype);
    Window_MenuStatusBack.prototype.constructor = Window_MenuStatusBack;
    
    var _Window_MenuStatusBack_initialize = Window_MenuStatusBack.prototype.initialize;
    Window_MenuStatusBack.prototype.initialize = function() {
        _Window_MenuStatusBack_initialize.apply(this, arguments);
        
        this.opacity = 0;
        this.sprite = new Sprite();
        this.addChild(this.sprite);
        this.sprite.bitmap = ImageManager.loadPicture("kana_still_1_0001");
        this.sprite.x = -60;
        this.sprite.y = 10;
        this.sprite.visible = true;
    };
    
    Window_MenuStatusBack.prototype.drawAllItems = function() {
    };
})();
