//=============================================================================
// ESRT_SceneStageSelect.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:ja
 * @plugindesc ステージ選択・プラグイン
 * @author 白
 *
 * @param サムネイル幅
 * @desc リスト用縮小画像の幅 (px)
 * @default 100
 *
 * @param サムネイル高さ
 * @desc リスト用縮小画像の高さ (px)
 * @default 110
 *
 * @param 未開放サムネイル
 * @desc 空白部分の画像のファイル名
 * @default stageSelect_close
 *
 * @param ボーナスモード開放フラグ
 * @desc ボーナスモード開放状態を記憶するフラグ
 * @default 181
 *
 * @param エクストラモード開放フラグ
 * @desc エクストラモード開放状態を記憶するフラグ
 * @default 182
 *
 * @help
 *
 *
 */

var Esrt = Esrt || {};
Esrt.stageListImages = Esrt.stageListImages || {};
Esrt.stageSelect = Esrt.stageSelect || {};
Esrt.stageSelect.stageIndex = 0;
Esrt.stageSelect.areaIndex = 0;
Esrt.stageSelect.focus = 1;
Esrt.stageSelect.onceSkipAutoSave = false;
Esrt.stageSelect.areaNum = 0;

Esrt.WindowMove.stageSelect = Esrt.WindowMove.stageSelect || {};
Esrt.WindowMove.stageSelect.isMoveIn = true;
Esrt.WindowMove.stageSelect.windowMoveDuration = 20;
Esrt.WindowMove.stageSelect.windowMoveCount = 0;
Esrt.WindowMove.stageSelect.windowMoveType = 1;
Esrt.WindowMove.stageSelect.windowMoveRate = 0;
var $dataStage  = null;

function Scene_StageSelect() {
    this.initialize.apply(this, arguments);
};

(function() {
    'use strict';
	var pluginName    = 'ESRT_SceneStageSelect';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var thumbnailWidth = parseInt(parameters['サムネイル幅']);
    var thumbnailHeigth = parseInt(parameters['サムネイル高さ']);
    var hiddenThumbnail = parameters['未開放サムネイル'];
    var isOpenBonusModeSwitch = parseInt(parameters['ボーナスモード開放フラグ']);
    var isOpenExtraModeSwitch = parseInt(parameters['エクストラモード開放フラグ']);
    var systemMenuShowSwitch = 201;
    var stageNumVariable     = 161;
    var gameModeVariable     = 171;
    var scoreWindowWidth     = 300;
    var scoreWindowHeight    = 320;
    var stageListUpdateFrame = 5; // 付加対策用。毎フレーム描画するとiOSで落ちる。

    Esrt.stageListImages.normal     = ImageManager.loadPictureTumbnail('stageSelect');
    Esrt.stageListImages.focus      = ImageManager.loadPictureTumbnail('stageSelect_focus');
    Esrt.stageListImages.extra      = ImageManager.loadPictureTumbnail('stageSelect_ex');
    Esrt.stageListImages.focus_ex   = ImageManager.loadPictureTumbnail('stageSelect_focus_ex');
    Esrt.stageListImages.close      = ImageManager.loadPictureTumbnail(hiddenThumbnail);
    Esrt.stageListImages.closeFocus = ImageManager.loadPictureTumbnail(hiddenThumbnail + 'Focus');
    Esrt.stageListImages.state      = ImageManager.loadPictureTumbnail('stageSelect_state');

    //=============================================================================
    // DataManager
    //  ステージ設定ファイル読み込み処理を定義します
    //=============================================================================
    DataManager.loadStageList = function() {
        this.loadDataFile('$dataStage', 'StageList.json');
        
        return;
    };

    //=============================================================================
    // Scene_Boot
    //  ステージ設定ファイル読み込み処理を追加定義します
    //=============================================================================
    var _Scene_Boot_create      = Scene_Boot.prototype.create;
    Scene_Boot.prototype.create = function() {
        _Scene_Boot_create.call(this);
        
        DataManager.loadStageList();
    };
    
    //=============================================================================
    // Scene_Menu
    //    メニューからの遷移フラグを追加します
    //=============================================================================
    var _Scene_Menu_initialize = Scene_Menu.prototype.initialize;
    Scene_Menu.prototype.initialize = function() {
        _Scene_Menu_initialize.apply(this, arguments);
        
        Esrt.WindowMove.stageSelect.isMoveIn = false;
        Esrt.WindowMove.stageSelect.windowMoveCount = Esrt.WindowMove.stageSelect.windowMoveDuration;
        Esrt.WindowMove.stageSelect.windowMoveType = 0;
    };
    
    //=============================================================================
    // Scene_Map
    //    ウィンドウ移動情報を初期化します
    //=============================================================================
    var _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this, arguments);
        
        Esrt.WindowMove.stageSelect.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.stageSelect.isMoveIn;
        Esrt.WindowMove.stageSelect.windowMoveCount = (Esrt.WindowMove.stageSelect.isMoveIn) ? 0 : Esrt.WindowMove.stageSelect.windowMoveDuration;
    };
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        
        if (command === 'SceneStageSelect') {
            SceneManager.push(Scene_StageSelect);
        }
    };

    //=============================================================================
    // Scene_StageSelect
    //  ステージ選択画面を扱うクラスです。
    //=============================================================================
    Scene_StageSelect.prototype             = Object.create(Scene_MenuBase.prototype);
    Scene_StageSelect.prototype.constructor = Scene_StageSelect;

    Scene_StageSelect.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
        
        this._backgroundLoading = false;
        this._startFlag         = false;
        
        // ウィンドウ移動演出
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
        this._windowMoveObject.windowMoveType = 1;
        this._windowMoveObject.windowMoveRate = 0;

        this.resetBulletData();
    };

    // セーブデータが重くなるため、TMShooting.jsの弾情報を削除する。
    Scene_StageSelect.prototype.resetBulletData = function() {
        $gameMap._playerBullets      = [];
        $gameMap._alivePlayerBullets = [];
        $gameMap._blankPlayerBullets = [];
        $gameMap._enemyBullets       = [];
        $gameMap._aliveEnemyBullets  = [];
        $gameMap._blankEnemyBullets  = [];
    };
    
    Scene_StageSelect.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);

        this.createWindow();
    };

    Scene_StageSelect.prototype.createScoreWindow = function() {
    };

    Scene_StageSelect.prototype.createBackground = function() {
        var width = Graphics.boxWidth * 2;
        var height = Graphics.boxHeight * 2;
    
        this._backgroundSprite = new Sprite();
        this.addChild(this._backgroundSprite);
        
        this._backgroundTile = new TilingSprite(ImageManager.loadTitle1("Background_4"));
        this._backgroundTile.move(0,0, width, height);
        this._backgroundTile.anchor.x = 0.5;
        this._backgroundTile.anchor.y = 0.5;
        this._backgroundSprite.addChild(this._backgroundTile);
    };

    Scene_StageSelect.prototype.update = function() {
        this._changeFocusWindow.updateTouch(); // suppressEventsを効かせるために先に判定する

        Scene_MenuBase.prototype.update.call(this);
        
        this.updateBackgroundEffects();
        this.updateInterval();
    };
    
    Scene_StageSelect.prototype.updateBackgroundEffects = function() {
    	this._backgroundTile.origin.x += 0.4;
    	this._backgroundTile.origin.y += 0.4;
    };
    
    // ウィンドウの移動割合計算
    Scene_StageSelect.prototype.getWindowMoveRate = function(object) {
        if (object.windowMoveCount >= 0) {
            var easingFunction = (object.windowMoveType == 1) ? jQueryEasing.easeOutQuint : jQueryEasing.easeInQuad;
            return easingFunction(object.windowMoveCount, 0, 1, object.windowMoveDuration);
        } else {
            return 1;
        };
    }
    
    Scene_StageSelect.prototype.updateInterval = function() {
        this._stageListWindow._touchInterVal += 1;
        if (this._touchInterval > 0) {
            this._touchInterval -= 1;
        }

        if (this._activateInterval > 0) {  // ウィンドウタッチのアクティベート時にステージを選択してしまわないようクールタイム
            this._activateInterval -= 1;
        }

        if ($gameSystem._saveInterval > 0) {  // Torigoya_SaveCommand.jsと連動 ロード時に即自動セーブしないようクールタイム
            $gameSystem._saveInterval -= 1;
        }
    }

    Scene_StageSelect.prototype.createWindow = function() {
        this.createStageScoreWindow();
        this.createAreaListWindow();
        this.createStageListWindow();
        this.createFertilizationCountWindow();
        this.createChangeFocusWindow();
        this.createButtonWindow();
        this.createSaveMessageWindow();
        this.setSceneAddress();
        this.activateWindow();
        
        this.checkAreaListShow();

    };
    Scene_StageSelect.prototype.checkAreaListShow = function() {
        if (!this.isArea1Clear()) { // ノーマルモードエリア1未クリアの場合、必要がないためエリアウィンドウを隠す
            this._areaListWindow.hide();
            this._areaListWindow.close();
            this._changeFocusWindow.hide();
            this._changeFocusWindow.close();
        }
    }

    // ノーマルモードエリア1がクリア済かチェックする
    Scene_StageSelect.prototype.isArea1Clear = function() {
        var gameMode = 0;
        if (Esrt.stageScore[11] &&
           Esrt.stageScore[11][gameMode] &&
           Esrt.stageScore[11][gameMode][Esrt.stageDatas.isOpen]) {
            return true;
        }

        return false;
    }

    Scene_StageSelect.prototype.setSceneAddress = function() {
        this._areaListWindow.setSceneAddress(this);
        this._stageListWindow.setSceneAddress(this);
        this._changeFocusWindow.setSceneAddress(this);
    }

    Scene_StageSelect.prototype.activateWindow = function() {
        if (Esrt.stageSelect.focus) {
            this._stageListWindow.activate();
        } else {
            this._areaListWindow.activate();
        }
    }

    Scene_StageSelect.prototype.createStageScoreWindow = function() {
        this._scoreWindow = new Window_Score(0, 130);
        this.addWindow(this._scoreWindow);
    };

    Scene_StageSelect.prototype.createAreaListWindow = function() {
        var wy              = 50;
        var ww              = Graphics.boxWidth;
        var stageListPadding = 15;

        this._areaListWindow = new Window_AreaList(0, wy, ww - scoreWindowWidth, 130);
        this._areaListWindow.setHandler('ok',          this.commandAreaSelect.bind(this));
        this._areaListWindow.setHandler('cancel',      this.commandMenu.bind(this));
        this._areaListWindow.setHandler('touchInside', this.activateAreaList.bind(this));
        if ($gameSwitches.value(isOpenExtraModeSwitch)) {
            this._areaListWindow.setHandler('shift', this.commandModeChange.bind(this));
        }
        this.addWindow(this._areaListWindow);

    }
    Scene_StageSelect.prototype.createStageListWindow = function() {
        var wy = 180;
        var wh = Graphics.boxHeight - wy;
        var ww = Graphics.boxWidth;
        var stageListPadding = 15;

        this._stageListWindowBack = new Window_StageListBack(0, wy, ww - scoreWindowWidth, wh);
        this.addWindow(this._stageListWindowBack);
        
        this._stageListWindow = new Window_StageList(0 + stageListPadding, wy + stageListPadding + 50, ww - scoreWindowWidth, wh);
        this._stageListWindow.setHandler('ok',          this.commandStageSelect.bind(this));
        this._stageListWindow.setHandler('cancel',      this.commandMenu.bind(this));
        this._stageListWindow.setHandler('menu',        this.commandMenu.bind(this));
        this._stageListWindow.setHandler('touchInside', this.activateStageList.bind(this));
        if ($gameSwitches.value(isOpenExtraModeSwitch)) {
            this._stageListWindow.setHandler('shift', this.commandModeChange.bind(this));
        }
        this.addWindow(this._stageListWindow);
    };

    Scene_StageSelect.prototype.createFertilizationCountWindow = function() {
        this._FertilizationCountWindow = new Window_FertilizationCount(0, 50);
        this.addWindow(this._FertilizationCountWindow);
    };

    Scene_StageSelect.prototype.createChangeFocusWindow = function() {
        this._changeFocusWindow = new Window_ChangeFocusSpriteButton(150, 110);
        this.addWindow(this._changeFocusWindow);
    };

    Scene_StageSelect.prototype.createButtonWindow = function() {

        this._startButtonWindow = new Window_Button(630, 550, 190, 70, 'button_start');
        this._startButtonWindow.setHandler('touchInside', this.commandStageSelect.bind(this));
        this.addWindow(this._startButtonWindow);

        if ($gameSwitches.value(isOpenExtraModeSwitch)) {
            this._changeButtonWindow = new Window_Button(630, 450, 190, 70, 'button_mode1');
            this._changeButtonWindow.setHandler('touchInside', this.commandModeChange.bind(this));
            this.addWindow(this._changeButtonWindow);
        }

        this._menuButtonWindow = new Window_Button(660, 0, 190, 70, 'button_sysmenu');
        this._menuButtonWindow.setHandler('touchInside', this.commandMenu.bind(this));
        this.addWindow(this._menuButtonWindow);
    };

    Scene_StageSelect.prototype.createSaveMessageWindow = function() {
        if (Esrt.autoSave.isShow) {
            this._saveMessageWindow = new Window_SaveMessage(50, -10, 400,100);
            this.addWindow(this._saveMessageWindow);

            Esrt.autoSave.isShow = false;
        }
    };

    Scene_StageSelect.prototype.activateAreaList = function() {
        SoundManager.playOk();
        this._areaListWindow.activate();
        this._stageListWindow.deactivate();
        this._areaListWindow.processTouch();
    };

    Scene_StageSelect.prototype.commandAreaSelect = function() {
        if (this._areaListWindow.isNowEnabled()) {
            SoundManager.playOk();
            this._areaListWindow.deactivate();
            this._stageListWindow.activate();
        }else{
            SoundManager.playBuzzer();
            this._areaListWindow.activate();
        }
    };

    Scene_StageSelect.prototype.activateStageList = function() {
        if (this._areaListWindow.isNowEnabled()) {
            this._activateInterval = 15;
            SoundManager.playOk();
            this._areaListWindow.deactivate();
            this._stageListWindow.activate();
            this._stageListWindow.processTouch();
        }else{
            SoundManager.playBuzzer();
        }
    };

    Scene_StageSelect.prototype.commandModeChange = function() {
        if (this._startFlag === true || this._touchInterval > 0 ) {return;} // 1クリックで複数呼び出されることに対処＋スタート時のフェードアウト中に呼び出されることに対処

        this._touchInterval = 15;
        AudioManager.playSe({"name":"Coin","volume":100,"pitch":100,"pan":0});

        var gameMode = $gameVariables.value(gameModeVariable);
            gameMode = $gameSwitches.value(isOpenExtraModeSwitch) ? (gameMode + 1) % 2 : 0;

        $gameVariables.setValue(gameModeVariable, gameMode);
        
        if($gameSwitches.value(isOpenExtraModeSwitch)){ // エクストラ開放時、スライドインして現在ステージが未開放なら最新ステージに移動する
            Esrt.WindowMove.stageSelect.isMoveIn = true;
            Esrt.WindowMove.stageSelect.windowMoveCount = 0;
            var item = this._areaListWindow.item();
            if (!this._areaListWindow.isEnabled(item)) {
                Esrt.stageSelect.areaIndex = this._areaListWindow.getMaxEnabledIndex();
                this._areaListWindow.select(Esrt.stageSelect.areaIndex);
            }
            this._areaListWindow.refreshWindowSkin();
            this._stageListWindowBack.refreshWindowSkin();
            this._scoreWindow.refreshWindowSkin();
            this._FertilizationCountWindow.refreshWindowSkin();
        }
        
        var buttonName = (Number(gameMode) === 0) ? 'button_mode1' : 'button_mode2';
        this._changeButtonWindow.setSprite(buttonName);
        this._areaListWindow.refresh();
        this._stageListWindow.refresh();
        this._stageListWindow.callUpdateScore();
    };

    Scene_StageSelect.prototype.commandMenu = function() {
        if (this._startFlag === true || this._touchInterval > 0 ) {return;} // 1クリックで複数呼び出されることに対処＋スタート時のフェードアウト中に呼び出されることに対処

        this._touchInterval = 15;
        
        SoundManager.playOk();
        SceneManager.push(Scene_Menu);
    };

    Scene_StageSelect.prototype.commandStageSelect = function() {
        this._stageListWindow.activate();
        
        if (this._startFlag === true || this._activateInterval > 0) {return;} // 複数呼び出されることに対処・アクティベート後即座にステージを選択してしまわない処理

        Esrt.stageSelect.stageIndex = this._stageListWindow.index();
        
        var item = this._stageListWindow.item();
        if (!this._stageListWindow.isEnabled(item)) { // 未開放時の処理
            SoundManager.playBuzzer();
            return;
        }
        
        this._startFlag = true;
        SoundManager.playSave();
        
        Esrt.WindowMove.stageSelect.windowMoveCount = 0;
        Esrt.WindowMove.stageSelect.windowMoveType = -1;

        for(var i = 0; i <= 100; i++) {$gameScreen.erasePicture(i)} // ピクチャの全削除

        // 位置情報のセット
        var gameMode = $gameVariables.value(gameModeVariable);
        var mapData = item[Esrt.gameModeName[gameMode]];
        
        $gameVariables.setValue(stageNumVariable, item.stageNum);
        $gameVariables.setValue(stageNumVariable + 1, mapData.x);
        $gameVariables.setValue(stageNumVariable + 2, mapData.y);
        $gameVariables.setValue(stageNumVariable + 3, mapData.mapId);
        $gameSwitches.setValue(systemMenuShowSwitch, true);

        // マップ移動命令
        $gamePlayer.setTransparent(255);
        this.fadeOutAll();
        $gameScreen.startFadeOut(1);
        $gamePlayer.reserveTransfer(mapData.mapId, mapData.x, mapData.y, 2);
        this.popScene();

    };
    //=============================================================================
    // Window_AreaList
    //  エリアリストウィンドウです。
    //=============================================================================
    function Window_AreaList() {
        this.initialize.apply(this, arguments);
    }

    Window_AreaList.prototype             = Object.create(Window_Selectable.prototype);
    Window_AreaList.prototype.constructor = Window_AreaList;

    Window_AreaList.prototype.initialize = function(x, y, width, height) {
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
        
        Window_Selectable.prototype.initialize.apply(this, arguments);

        this._canProcessTouchNonActive = true;
        this._data = [];
        this.refreshWindowSkin();
        this.select(Esrt.stageSelect.areaIndex);
    };
    
    var Window_AreaList_activate = Window_AreaList.prototype.activate;
    Window_AreaList.prototype.activate = function(index) {
        Window_AreaList_activate.call(this);
        
        Esrt.stageSelect.focus = 0;
        this.opacity           = 255;
        this.alpha             = 1;
        this._scene._changeFocusWindow.changeButtonValid();
    }

    var Window_AreaList_deactivate = Window_AreaList.prototype.deactivate;
    Window_AreaList.prototype.deactivate = function() {
        Window_AreaList_deactivate.call(this);
        
        this.opacity = 127;
        this.alpha   = 0.75;
    };

    var Window_AreaList_select = Window_AreaList.prototype.select;
    Window_AreaList.prototype.select = function(index) {
        Window_AreaList_select.apply(this, arguments);

        this.refresh();
        this.callUpdateScore();

        if (index > -1) {
            Esrt.stageSelect.areaIndex = index;
        }
        
        if (this._scene && this._scene._stageListWindow) {
            this._scene._stageListWindow.setAreaNum(this._data[index].areaNum);
        }
    };

    Window_AreaList.prototype.makeItemList = function() {
        if (!$dataStage) return;
        if (!$dataStage["stageList"]) return;
        
        this._data = [];
        
        Object.keys($dataStage["stageList"]).forEach(function(key) {
            var areaData = $dataStage["stageList"][key];
            if (areaData != null && this.isShowItem(areaData)) {
                this._data.push(areaData);
            }
        }, this);
    };

    Window_AreaList.prototype.drawItem = function(index) {
        var rect = this.itemRectForText(index);
        var align = this.itemTextAlign();
        
        this.resetTextColor();
        this.changePaintOpacity(this.isEnabled(this._data[index]));
        
        this.drawText(this.commandName(index), rect.x, rect.y, rect.width, align);
    };

    Window_AreaList.prototype.commandName = function(index) {
        if (index < 0 || !this._data) return;
        return this._data[index].areaText;
    };

    Window_AreaList.prototype.maxCols = function() {
        return 8;
    };

    Window_AreaList.prototype.maxItems = function() {
        return this._data ? this._data.length : 1;
    };

    Window_AreaList.prototype.item = function() {
        var index = this.index();
        
        return this._data && index >= 0 ? this._data[index] : null;
    };

    Window_AreaList.prototype.itemTextAlign = function() {
        return 'left';
    };

    Window_AreaList.prototype.isEnabled = function(item) {
        var stageNum = item ? item.repStageNum : 0;
        var gameMode = $gameVariables.value(gameModeVariable);

        return stageNum && Esrt.stageScore[stageNum] ? Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.isOpen] : false;

    };
    
    Window_AreaList.prototype.getMaxEnabledIndex = function() {
        for (var i = 1; i < this._data.length; i++) {
            var item = this._data[i];
            if (!this.isEnabled(item)){
                return i - 1;
            }
        }
        return this._data.length - 1;
    };

    Window_AreaList.prototype.isShowItem = function(item) {
        var stageNum = item ? item.shadowRepStageNum : 0;
        var gameMode = $gameVariables.value(gameModeVariable);
        
        if (stageNum == -1) return true;
        if (!stageNum) return false;
        if (item.notEX && gameMode === 1) return false;
        
        return stageNum && Esrt.stageScore[stageNum] ? Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.clearCount] : false;

    };

    Window_AreaList.prototype.setSceneAddress = function(scene) {
        this._scene = scene;
        this.refresh();
        
        if (this._scene._stageListWindow && this._data[this.index()]) {
            this._scene._stageListWindow.setAreaNum(this._data[this.index()].areaNum);
        }
        
        this.callUpdateScore();
    }

    Window_AreaList.prototype.callUpdateScore = function() {
        if (this.active && this._scene._scoreWindow) {
            this.updateScore();
        }
    };

    Window_AreaList.prototype.updateScore = function() {
        var item = this._data[this.index()];
        var gameMode = $gameVariables.value(gameModeVariable);
        var areaText   = item ? item.areaNum   : "?";
            areaText   = item && item.scoreWindowText ? item.scoreWindowText : areaText;
            areaText   += gameMode === 1 ? "EX" : "";
        
        this._scene._scoreWindow.setAreaData(item ? {"areaNum":item.areaNum,"areaText":areaText} : {"areaNum":0,"areaText":areaText});
        this._scene._scoreWindow.drawData("area");
    };

    Window_AreaList.prototype.cursorDown = function(wrap) {
        if (this.isNowEnabled()) {
            SoundManager.playOk();
            
            this._scene._stageListWindow.activate();
            this.deactivate();
            
            Input.clear();
            
        }else{
            SoundManager.playBuzzer();
        }
    };

    Window_AreaList.prototype.isNowEnabled = function() {
        return this.isEnabled(this._data[this.index()]);
    };
    
    Window_AreaList.prototype.refreshWindowSkin = function() {
        var gameMode = $gameVariables.value(gameModeVariable);
        var windowSkinFile = 'Window_StageList';
        windowSkinFile = (gameMode === 1) ? 'Window_StageList_EX' : windowSkinFile;
        this.windowskin = ImageManager.loadSystem(windowSkinFile);
        
        this._refreshAllParts();
    };

    Window_AreaList.prototype.refresh = function() {
        this.makeItemList();
        this.createContents();
        this.drawAllItems();
    };
    
    Window_AreaList.prototype.playOkSound = function() {
    };
    
    //=============================================================================
    // Window_StageListBack
    //  ステージリストウィンドウ背景です。
    //=============================================================================
    function Window_StageListBack() {
        this.initialize.apply(this, arguments);
    }

    Window_StageListBack.prototype             = Object.create(Window_Base.prototype);
    Window_StageListBack.prototype.constructor = Window_StageList;

    Window_StageListBack.prototype.initialize = function(x, y, width, height) {
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
    
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this.refreshWindowSkin();
    };
    
    Window_StageListBack.prototype.refreshWindowSkin = function() {
        var gameMode = $gameVariables.value(gameModeVariable);
        var windowSkinFile = 'Window_StageList';
        windowSkinFile = (gameMode === 1) ? 'Window_StageList_EX' : windowSkinFile;
        this.windowskin = ImageManager.loadSystem(windowSkinFile);
        
        this._refreshAllParts();
    };
    //=============================================================================
    // Window_StageList
    //  ステージリストウィンドウです。
    //=============================================================================
    function Window_StageList() {
        this.initialize.apply(this, arguments);
        this._stageListWindow = true;
    }

    Window_StageList.prototype             = Object.create(Window_Selectable.prototype);
    Window_StageList.prototype.constructor = Window_StageList;

    Window_StageList.prototype.initialize = function(x, y, width, height) {
        this._areaNum = 1;
        
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
        
        Window_Selectable.prototype.initialize.call(this, x, y, width, height);

        this._canProcessTouchNonActive = true;
        this._data = [];
        
        this.refresh();
        this.select(Esrt.stageSelect.stageIndex);
        this.opacity = 0;
    };

    var Window_StageList_activate = Window_StageList.prototype.activate;
    Window_StageList.prototype.activate = function(index) {
        Window_StageList_activate.call(this);
        
        Esrt.stageSelect.focus = 1;
        this._scene._changeFocusWindow.changeButtonValid();
        this.alpha = 1;
        
        if (this._scene &&this._scene._stageListWindowBack) this._scene._stageListWindowBack.opacity = 255;
    }

    var Window_StageList_deactivate = Window_StageList.prototype.deactivate;
    Window_StageList.prototype.deactivate = function() {
        Window_StageList_deactivate.call(this);
        
        this.alpha = 0.5;
        
        if (this._scene && this._scene._stageListWindowBack) this._scene._stageListWindowBack.opacity = 127;
    };

    var Window_StageList_select = Window_StageList.prototype.select;
    Window_StageList.prototype.select = function(index) {
        Window_StageList_select.apply(this, arguments);

        this.refresh();
        this.callUpdateScore();

        if (index > -1) {
            Esrt.stageSelect.stageIndex = index;
        }
    };

    Window_StageList.prototype.itemRect = function(index) {
        var rect = new Rectangle();
        var maxCols = this.maxCols();
        var maxRows = 5;
        
        rect.width  = this.itemWidth();
        rect.height = this.itemHeight()- 7;
        rect.x      = index % maxCols             * rect.width  - this._scrollX;
        rect.y      = Math.floor(index / maxRows) * rect.height - this._scrollY;
        return rect;
    };

    Window_StageList.prototype.maxCols = function() {
        return Math.floor(this.contentsWidth() / thumbnailWidth);
    };

    Window_Selectable.prototype.maxRows = function() {
        return Math.max(Math.ceil(this.maxItems() / this.maxCols()), 1);
    };

    Window_StageList.prototype.itemWidth = function() {
        return thumbnailHeigth;
    };

    Window_StageList.prototype.itemHeight = function() {
        var rows = Math.floor(this.contentsHeight() / thumbnailHeigth);
        return thumbnailHeigth + 7;
    };

    Window_StageList.prototype.maxItems = function() {
        return this._data ? this._data.length : 1;
    };

    Window_StageList.prototype.makeItemList = function() {
        if (!$dataStage) return;
        if (!$dataStage["stageList"][this._areaNum]) return;
        var gameMode = $gameVariables.value(gameModeVariable);
        
        this._data = $dataStage["stageList"][this._areaNum]["stages"].filter(function(item) {
            return item != null && !(item.notEX && gameMode === 1);
        }, this);
    };

    Window_StageList.prototype.item = function() {
        var index = this.index();
        
        return this._data && index >= 0 ? this._data[index] : null;
    };

    Window_StageList.prototype.isEnabled = function(item) {
        var stageNum = item ? item.stageNum : 0;
        var gameMode = $gameVariables.value(gameModeVariable);

        return stageNum ? Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.isOpen] : false;
    };

    Window_StageList.prototype.isCurrentItemEnabled = function() {
        return this.isEnabled(this._data[this.index()]);
    };

    Window_StageList.prototype.drawItem = function(index) {
        var item = this._data[index];
        
        if (item) {
            var rect      = this.itemRect(index);
            var focusFlag = false;
            
            rect.width -= this.textPadding();
            
            if (index === this.index()) {
                focusFlag = true;
            }
            this.drawPicture(index, item, focusFlag);
        }
    };

    // ● ピクチャーの描画
    Window_StageList.prototype.drawPicture = function(index, item, flag) {
        if (typeof Esrt.stageScore[item.stageNum] === "undefined") {
            Esrt.stageScoreFanc.GetValue(["",item.stageNum,"normal","highScore",121]); // ゲーム初起動時のオブジェクト生成用空アクセス
        }

        this.drawStagePicture(index, item, flag);
        this.drawStatePicture(index, item, flag);
    };

    // ステージピクチャの表示
    Window_StageList.prototype.drawStagePicture = function(index, item, flag) {
        var gameMode    = $gameVariables.value(171);
        var stageScores = Esrt.stageScore[item.stageNum][gameMode];

        var rect   = this.itemRect(index);
        var width  = rect.width  || thumbnailWidth;
        var height = rect.height || thumbnailHeigth;

        var thumIndex = ($dataStage["stageList"][this._areaNum]) ? $dataStage["stageList"][this._areaNum]["thumIndex"] : 0;
        thumIndex += index;
        var pw = thumbnailWidth;
        var ph = thumbnailHeigth;
        var sw = Math.min(width, pw);
        var sh = Math.min(height, ph);
        var dx = Math.floor(rect.x + Math.max(width - pw, 0) / 2);
        var dy = Math.floor(rect.y + Math.max(height - ph, 0) / 2);
        var sx = (pw - sw) / 2 + (thumIndex % 5) * pw;
        var sy = (ph - sh) / 2 + Math.floor(thumIndex / 5) * ph;
        var bitmap = (gameMode === 0) ? Esrt.stageListImages.normal : Esrt.stageListImages.extra;
        
        // 選択中ステージの画像
        if (stageScores[Esrt.stageDatas.isOpen]) {
            if (flag) {
                bitmap = (gameMode === 0) ? Esrt.stageListImages.focus : Esrt.stageListImages.focus_ex;
            }
        }else{
            bitmap = (flag) ? Esrt.stageListImages.closeFocus : Esrt.stageListImages.close;
        }
        
        this.contents.blt(bitmap, sx, sy, sw, sh, dx, dy);
    };

    // 攻略状態ピクチャの表示
    Window_StageList.prototype.drawStatePicture = function(index, item, flag) {
        var gameMode    = $gameVariables.value(171);
        var stageScores = Esrt.stageScore[item.stageNum][gameMode];

        var rect   = this.itemRect(index);
        var width  = rect.width  || thumbnailWidth;
        var height = rect.height || thumbnailHeigth;

        var pw = thumbnailWidth;
        var ph = thumbnailHeigth;
        var sw = Math.min(width, pw);
        var sh = Math.min(height, ph);
        var dx = Math.floor(rect.x + Math.max(width - pw, 0) / 2);
        var dy = Math.floor(rect.y + Math.max(height - ph, 0) / 2);
        var sx = (pw - sw) / 2 + (1 % 5) * pw;
        var sy = (ph - sh) / 2 + Math.floor(1 / 5) * ph;
        var bitmap = Esrt.stageListImages.state;

        if (!stageScores[Esrt.stageDatas.isOpen]) return;

        var highCoin = stageScores[Esrt.stageDatas.highCoin];
        var maxCoin  = ($dataStage.maxCoin[item.stageNum]) ? $dataStage.maxCoin[item.stageNum][Esrt.gameModeName[gameMode]] : 0;

        // 魔法石の取得状態表示
        if (maxCoin !== 0) {
            var stateNum = (highCoin >= maxCoin) ? 1 : 0;
            
            sx = (pw - sw) / 2 + (stateNum % 5) * pw;
            sy = (ph - sh) / 2 + Math.floor(stateNum / 5) * ph;
            
            this.contents.blt(bitmap, sx, sy, sw, sh, dx, dy);
        }
        
        var clearCount = stageScores[Esrt.stageDatas.clearCount];
        
        // 新規ステージの表示
        if (clearCount == 0) {
            var stateNum = 2;
            
            sx = (pw - sw) / 2 + (stateNum % 5) * pw;
            sy = (ph - sh) / 2 + Math.floor(stateNum / 5) * ph;
            
            this.contents.blt(bitmap, sx, sy, sw, sh, dx, dy);
        }
    };

    Window_StageList.prototype.drawItemName = function(item, x, y, width) {
        this.resetTextColor();
        this.drawText(this.isEnabled(item) ? item.displayName : '？？？', x, y, width);
    };

    Window_StageList.prototype.setAreaNum = function(areaNum) {
        this._areaNum = areaNum;
        Esrt.stageSelect.areaNum = areaNum;
        
        this.refresh();
        
        if (this._data.length - 1 < this.index()) {
            this.select(0);
        }
    };

    Window_StageList.prototype.setSceneAddress = function(scene) {
        this._scene = scene;
    }

    Window_StageList.prototype.callUpdateScore = function() {
        if (this.active && this._scene._scoreWindow) {
            this.updateScore();
        }
    };

    Window_StageList.prototype.updateScore = function() {
        var item = this.item();
        var gameMode = $gameVariables.value(gameModeVariable);
        var stageText   = item ? item.stageText   : "?-?";
            stageText   += gameMode === 1 ? "EX" : "";
        var premiseText = item ? item.premiseText : "?-?";
            premiseText += gameMode === 1 ? "EX" : "";
        
        this._scene._scoreWindow.setStageData(item ?
             {"stageNum":item.stageNum, "stageText":stageText, "premiseText":premiseText, "clearTerms":item.clearTerms} : 
             {"stageNum": 0,            "stageText":stageText, "premiseText":premiseText});
        this._scene._scoreWindow.drawData("stage");
    };

    Window_StageList.prototype.getHelp = function(item) {
        return this.isEnabled(item) ? '【' + item.displayName + '】' : item.hint;
    };

    Window_StageList.prototype.cursorUp = function(wrap) {
        var index = this.index();
        var maxItems = this.maxItems();
        var maxCols = this.maxCols();

        if (index < maxCols && this._scene._areaListWindow) {
            if (this._scene.isArea1Clear()) {
                SoundManager.playOk();
                this._scene._areaListWindow.activate();
                this.deactivate();
            }
        }
        if (index >= maxCols || (wrap && maxCols === 1)) {
            this.select((index - maxCols + maxItems) % maxItems);
        }
    };

    Window_StageList.prototype.refresh = function() {
        this.makeItemList();
        this.createContents();
        this.drawAllItems();
    };
    
    Window_StageList.prototype.playOkSound = function() {
    };

    //=============================================================================
    // Window_ChangeFocusSpriteButton
    //    タップ用エリア選択・ステージ選択切り替えボタン
    //=============================================================================
    function Window_ChangeFocusSpriteButton() {
        this.initialize.apply(this, arguments);
    }
    Window_ChangeFocusSpriteButton.prototype = Object.create(Window_ButtonSprites.prototype);
    Window_ChangeFocusSpriteButton.prototype.constructor = Window_ChangeFocusSpriteButton;

    Window_ChangeFocusSpriteButton.prototype.initialize = function(x, y) {
        var width = 200;
        var height = 160;
        
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
        
        Window_ButtonSprites.prototype.initialize.call(this, x, y, width, height);
        this.opacity = 0;

        var spriteX = 0;
        var spriteY = 0;
        
        this.createSpriteButton("_areaSelectButton", "button_area_select", spriteX, spriteY);
        
        spriteY += 85;
        this.createSpriteButton("_stageSelectButton", "button_stage_select", spriteX, spriteY);

        this._hoverCheckSpritelist.push("_areaSelectButton");
        this._hoverCheckSpritelist.push("_stageSelectButton");
        this._touchCheckSpritelist.push({button:"_areaSelectButton",   window:"_areaListWindow"});
        this._touchCheckSpritelist.push({button:"_stageSelectButton",  window:"_stageListWindow"});
    };

    Window_ChangeFocusSpriteButton.prototype.setSceneAddress = function(scene) {
        this._scene = scene;
    }

    // スプライトのタッチ状態を取得
    // タッチされていたら対応するスイッチをtrueにして、以降のゲーム内タッチ処理をキャンセルする
    Window_ChangeFocusSpriteButton.prototype.updateTouch = function() {
        if (!this.visible) return;

        if (this._areaSelectButton && this.isTriggeredButton(this._areaSelectButton)) {
            this._scene._areaListWindow.activate();
            this._scene._stageListWindow.deactivate();
            
            SoundManager.playOk();
            
            TouchInput.suppressEvents();
            
            return;
            
        }else if (this._stageSelectButton && this.isTriggeredButton(this._stageSelectButton)) {
            if (this._scene._areaListWindow.isNowEnabled()) {
                this._scene._areaListWindow.deactivate();
                this._scene._stageListWindow.activate();
                
                SoundManager.playOk();
                
            }else{
                SoundManager.playBuzzer();
                
            }
            TouchInput.suppressEvents();
            
            return;
        }
    };

    Window_ChangeFocusSpriteButton.prototype.changeButtonValid = function() {
        if (Esrt.stageSelect.focus) {
            this._areaSelectButton.setValid(true);
            this._stageSelectButton.setValid(false);
            
        }else{
            this._areaSelectButton.setValid(false);
            this._stageSelectButton.setValid(true);
        }
    }

    //=============================================================================
    // Window_Score
    //    スコアウィンドウを定義します。
    //=============================================================================
    function Window_Score() {
        this.initialize.apply(this, arguments);
    }
    Window_Score.prototype = Object.create(Window_Base.prototype);
    Window_Score.prototype.constructor = Window_Score;
    Window_Score.prototype.initialize = function(x, y) {
        var width  = scoreWindowWidth;
        var height = scoreWindowHeight;
        var x      = Graphics.boxWidth - width;
        var y      = y;
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
        
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        this.refreshWindowSkin();
    };

    Window_Score.prototype.setAreaData = function(areaData) {
        this._areaData = areaData;
    }

    Window_Score.prototype.setStageData = function(stageData) {
        this._stageData = stageData;
    }

    var _Window_Base_loadWindowskin = Window_Base.prototype.loadWindowskin;
    Window_Base.prototype.loadWindowskin = function() {
        ImageManager.loadSystem('Window_StageList');
        ImageManager.loadSystem('Window_StageList_EX');
        
        _Window_Base_loadWindowskin.call(this);
    };

    Window_Score.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_StageList');
        ImageManager.loadSystem('Window_StageList_EX');
    };

    Window_Score.prototype.drawData = function(type) {
        if (!$dataSystem) { return false;}
        
        this.contents.clear();
        
        if (type === "area") { this.drawAreaData();}
        else               { this.drawStageData();}
    }
    
    // スコアウィンドウにエリア情報を描画
    Window_Score.prototype.drawAreaData = function() {
        this.drawY = 0;
		this.drawText("エリア" + this._areaData.areaText, 0, this.drawY);
        
        this.drawY += 40;

        var gameMode = $gameVariables.value(gameModeVariable);

        var areaData    = $dataStage["stageList"][this._areaData.areaNum];
        var repStageNum = areaData.repStageNum;
        if (!(repStageNum && Esrt.stageScore[repStageNum] && Esrt.stageScore[repStageNum][gameMode][Esrt.stageDatas.isOpen])) {
    		this.drawText("　未開放", 0, this.drawY);
            
            this.drawY += 80;
            
            this.drawAreaOpenTerms(areaData);
            
            return;
        }
    }
    
    // スコアウィンドウにステージ情報を描画
    Window_Score.prototype.drawStageData = function() {
        this.drawY = 0;
        
        var gameMode    = $gameVariables.value(171);
        var stageScores = Esrt.stageScore[this._stageData.stageNum][gameMode];

        this.drawStageName();
        this.drawStageOpenTerms(stageScores);
        
        if (!stageScores[Esrt.stageDatas.isOpen]) {return;}

        this.drawY += 40;
        this.drawStageClearTerms();
        
        var maxCoin  = ($dataStage.maxCoin[this._stageData.stageNum]) ? $dataStage.maxCoin[this._stageData.stageNum][Esrt.gameModeName[gameMode]] : 0;
        var highCoin = stageScores[Esrt.stageDatas.highCoin];
        
        
        this.drawY += 40;
        this.drawStageCoin(highCoin, maxCoin);
        
        this.drawY += 40;
        this.drawFastestStageTime(highCoin, maxCoin, stageScores); // 全取得時にはタイムも表示
        
    	this.resetTextColor();
    }
    
    // エリア解放条件の表示
    Window_Score.prototype.drawAreaOpenTerms = function(areaData) {
        if (areaData.openRequirement) {
            var text = "解放条件\n";
            
            if (areaData.openRequirement == "if3lose") {
                text += "エリア3：\n" +
                       "大通りで変身して\n" +
                       "ステージ失敗する";
                       
            } else if (areaData.openRequirement == "area7") {
                var gameMode    = $gameVariables.value(171);
                if (gameMode == 0) {
                    var max = ($dataStage.openArea7) ? $dataStage.openArea7.normal : "---";
                    text += "妊娠or苗床経験有＆\n" +
                           "魔法石合計" + max + "個以上\n" +
                           "で6-11クリア";
                } else {
                    text += "6-11クリア";
                }
                
            } else {
                text += areaData.openRequirement + "をクリア";
            }
            
            this.drawTextEx(text, 0, this.drawY);
        }
    }
    
    // ステージ解放条件の表示
    Window_Score.prototype.drawStageName = function() {
        var text = "";
        
        text += (this._stageData.stageText.match(/^\d+-\d+$/)) ? "ステージ" : "";
        text += this._stageData.stageText;
        
        this.drawText(text, 0, this.drawY);
    }
    
    // ステージ解放条件の表示
    Window_Score.prototype.drawStageOpenTerms = function(stageScores) {
        if (!stageScores[Esrt.stageDatas.isOpen]) {
            this.drawY += 40;
            
    		this.drawText("　未開放", 0, this.drawY);
            
            if (this._stageData.premiseText) {
                this.drawY += 40;
                
                var text  = "解放条件\n";
                    text += "　" + this._stageData.premiseText + "をクリア";
                    
                this.drawTextEx(text, 0, this.drawY);
            }
        }
    }
    
    // クリア条件の表示
    Window_Score.prototype.drawStageClearTerms = function() {
        var text = "クリア条件:\n"
        text += "　　";
        
        if      (this._stageData.clearTerms === "defeat")     { text += "怪人を倒す！";  }
        else if (this._stageData.clearTerms === "ridding")    { text += "怪人の巣を駆除";}
        else if (this._stageData.clearTerms === "runAway")    { text += "ゴールまで走る";}
        else if (this._stageData.clearTerms === "bigBoss")    { text += "破壊する！";}
        else if (this._stageData.clearTerms === "bossRush")   { text += "変身して倒す！";}
        else if (this._stageData.clearTerms === "lastBattle") { text += "- - - - -";}
        else if (this._stageData.clearTerms === "charm")      { text += "誘惑する";}
        else                                                  { text += "変身する";      }
        
        this.drawTextEx(text, 0, this.drawY);
        
        this.drawY += 40;
        this.resetTextColor();
    }
    
    // 取得した魔法石の個数を表示
    Window_Score.prototype.drawStageCoin = function(highCoin, maxCoin) {
        if (highCoin === 0 && maxCoin === 0) {
            this.drawY += 40;
            return;
        }
        
		this.changeTextColor(this.textColor(5));
        
        this.drawIcon(303, 1, this.drawY);
        var expText = (maxCoin !== 0) ? "拾った魔法石" : "";
		this.drawText(expText,40, this.drawY);
        
		this.resetTextColor();
        
        this.drawY += 40;
        
        // 全取得時には表彰アイコンを表示
        if (highCoin >= maxCoin) {
            this.drawIcon(87, 80, this.drawY);
        }
        
        var coinText = highCoin;
        coinText += (maxCoin) ? "／" + maxCoin : "";
        
		this.drawText(coinText, 0, this.drawY, scoreWindowWidth - 50 , 'right');
    }
    
    // 当該ステージの全魔法石回収最速タイムを表示
    Window_Score.prototype.drawFastestStageTime = function(highCoin, maxCoin, stageScores) {
        if (highCoin === 0 && maxCoin === 0) {
            if (stageScores[Esrt.stageDatas.clearCount] > 0) {
                this.drawIcon(220, 1, this.drawY);
                
        		this.changeTextColor(this.textColor(5));
        		this.drawText("最短タイム",40, this.drawY);
        		this.resetTextColor();
                
                this.drawY += 40;
                var time = stageScores[Esrt.stageDatas.allCoinFastestTime];
                var times = GetTime(time, 5184000);
        		this.drawText(times.hour + ":" + times.min + ":" + times.sec + "." + times.msec, 0, this.drawY, scoreWindowWidth - 50 , 'right');
            }
            
        } else {
            if (highCoin >= maxCoin) {
                this.drawIcon(220, 1, this.drawY);
                
        		this.changeTextColor(this.textColor(5));
        		this.drawText("全回収最短タイム",40, this.drawY);
        		this.resetTextColor();
                
                this.drawY += 40;
                var time = stageScores[Esrt.stageDatas.allCoinFastestTime];
                var times = GetTime(time, 5184000);
        		this.drawText(times.hour + ":" + times.min + ":" + times.sec + "." + times.msec, 0, this.drawY, scoreWindowWidth - 50 , 'right');
            }
        }
        
    }
    
    Window_Score.prototype.refreshWindowSkin = function() {
        var gameMode = $gameVariables.value(gameModeVariable);
        var windowSkinFile = 'Window_StageList';
        windowSkinFile = (gameMode === 1) ? 'Window_StageList_EX' : windowSkinFile;
        this.windowskin = ImageManager.loadSystem(windowSkinFile);
        
        this._refreshAllParts();
    };

    function GetTime(time, maxTime) {
        var times = {"day":"--","hour":"--","min":"--","sec":"--","msec":"--"};
        if (maxTime !=0 && time < maxTime) {
            times.msec = time % 60;
            
            time = parseInt(time / 60);
            times.sec = time % 60;
            
            time = parseInt(time / 60);
            times.min = time % 60;
            
            time = parseInt(time / 60);
            times.hour = time;

        }
        return times;
    }

    //=============================================================================
    // Window_FertilizationCount
    //    受精カウントウィンドウを定義します。
    //=============================================================================
    function Window_FertilizationCount() {
        this.initialize.apply(this, arguments);
    }
    
    Window_FertilizationCount.prototype = Object.create(Window_Base.prototype);
    Window_FertilizationCount.prototype.constructor = Window_FertilizationCount;
    Window_FertilizationCount.prototype.initialize = function(x, y) {
        var width  = scoreWindowWidth;
        var height = 80;
        var x      = Graphics.boxWidth - width;
        var y      = y;
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.stageSelect;
        
        Window_Base.prototype.initialize.call(this, x, y, width, height);

        this.drawData();
        this.refreshWindowSkin();
    };

    Window_FertilizationCount.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_StageList');
    };

    Window_FertilizationCount.prototype.drawData = function(type) {
        this.contents.clear();

        var y = 0;
        var count = $gameVariables.value(201);
        var max   = $gameVariables.value(202) > 5 ? "∞" : $gameVariables.value(202);

		this.changeTextColor(this.textColor(2));
        this.drawIcon(Esrt.GetFertilizationIconNumber(), 1, y);
		this.drawText("受精カウント", 40, y, 120);
    	this.resetTextColor();
		this.drawText(count + "／" + max, 160, y, 100, "right");

    }
    
    Window_FertilizationCount.prototype.refreshWindowSkin = function() {
        var gameMode = $gameVariables.value(gameModeVariable);
        var windowSkinFile = 'Window_StageList';
        windowSkinFile = (gameMode === 1) ? 'Window_StageList_EX' : windowSkinFile;
        this.windowskin = ImageManager.loadSystem(windowSkinFile);
        
        this._refreshAllParts();
    };

    //=============================================================================
    // Window_SaveMessage
    //    メニュー表示タップ用ウィンドウを定義します。
    //=============================================================================
    function Window_SaveMessage() {
        this.initialize.apply(this, arguments);
    }

    Window_SaveMessage.prototype = Object.create(Window_Base.prototype);
    Window_SaveMessage.prototype.constructor = Window_SaveMessage;

    Window_SaveMessage.prototype.initialize = function(x, y, width, height) {
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        var text = Esrt.autoSave.isSaveSuccess ? "オートセーブしました" : "オートセーブに失敗しました";
        
    	this.drawText(text,0, 0);

        this.opacity    = 0;
        this._showCount = 120;
    };

    var _Window_SaveMessage_update              = Window_SaveMessage.prototype.update;
    Window_SaveMessage.prototype.update = function() {
        _Window_SaveMessage_update.apply(this, arguments);
        
        if (this._showCount) {
            this._showCount--;
        }
        
        if (this._showCount <= 0 && this.alpha > 0) {
            this.alpha -= 0.01;
        }
    };
    //=============================================================================
    // DataManager
    //  ステージ選択データをセーブに含めます
    //=============================================================================
    var _DataManager_makeSaveContents = DataManager.makeSaveContents;
    DataManager.makeSaveContents = function() {
        var contents = _DataManager_makeSaveContents.call(this);
        
        contents.stageSelect = Esrt.stageSelect;
        
        return contents;
    };
    
    var _DataManager_extractSaveContents = DataManager.extractSaveContents;
    DataManager.extractSaveContents = function(contents) {
        _DataManager_extractSaveContents.call(this, contents);
        
        Esrt.stageSelect = contents.stageSelect;
    };
})();
