//=============================================================================
// ESRT_SceneTitle.js
// ----------------------------------------------------------------------------
// Copyright (c) 2017 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:ja
 * @plugindesc タイトルコマンド上書きプラグイン
 * @author 白
 *
 * @param タイトルに回想を追加
 * @desc タイトル画面にギャラリーを追加します。(ON/OFF)
 * @default ON
 * 
 * @param 回想名称
 * @desc タイトル画面に表示されるコマンド名です。
 * @default 回想
 * 
 * @help 
 *
 *
 */

(function() {
    'use strict';
	var pluginName    = 'ESRT_SceneTitle';
    
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var titleGalleryVisible = ( parameters['タイトルに回想を追加'].toUpperCase() === 'ON' );
    
    
    //=============================================================================
    // Scene_Title
    //=============================================================================
    var _Scene_Title_createCommandWindow      = Scene_Title.prototype.createCommandWindow;
    Scene_Title.prototype.createCommandWindow = function() {
        _Scene_Title_createCommandWindow.apply(this, arguments);
        this._commandWindow.setHandler('gallery', this.commandGallery.bind(this));
        if (StorageManager.isLocalMode()) { this._commandWindow.setHandler('windowClose', this.commandClose.bind(this)); }
    };

    Scene_Title.prototype.commandGallery = function() {
        var preMapId = $dataSystem.startMapId;
        var preStartX = $dataSystem.startX;
        var preStartY = $dataSystem.startY;
        
        if (Esrt.gallery) { Esrt.gallery.fromTitle = true; }
        
        this._commandWindow.close();
        $dataSystem.startMapId = 35;
        $dataSystem.startX     = 1;
        $dataSystem.startY     = 1;
        
        this.commandNewGame();
        
        $dataSystem.startMapId = preMapId;
        $dataSystem.startX     = preStartX;
        $dataSystem.startY     = preStartY;
    };
    
    Scene_Title.prototype.commandClose = function() {
        window.close();
    };

    //=============================================================================
    // Window_TitleCommand
    //=============================================================================
    var _Window_TitleCommand_makeCommandList      = Window_TitleCommand.prototype.makeCommandList;
    Window_TitleCommand.prototype.makeCommandList = function() {
        
        this.addCommand(TextManager.newGame,   'newGame');
        this.addCommand(TextManager.continue_, 'continue', this.isContinueEnabled());
        if (titleGalleryVisible) { this.addCommand(parameters['回想名称'], 'gallery'); }
        if (typeof TMPlugin != "undefined" && TMPlugin.LinkCredit && TMPlugin.LinkCredit.CreditsCommand) {
            this.addCommand(TMPlugin.LinkCredit.CreditsCommand, 'credits');
        }
        this.addCommand(TextManager.options,   'options');
        //if(StorageManager.isLocalMode()) {
        //    this.addCommand("ゲーム終了", 'windowClose');
        //}
    };
})();
