//=============================================================================
// ESRT_StageClearWindow.js
// ----------------------------------------------------------------------------
// Copyright (c) 2020 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================
 
/*:ja
 * @plugindesc マップ上にステージクリアウィンドウ・スコアを表示する
 * @author 白
 * 
 * @help 
 *
 * このプラグインには、プラグインコマンドはありません。
 * 
 * This plugin is released under the MIT License.
 *
 */
 
 var Esrt = Esrt || {};
 Esrt.StageClear = Esrt.StageClear || {};
 Esrt.StageClear.showFlag = 0;
 Esrt.StageClear.recordShowFlag = 0;
 Esrt.WindowMove = Esrt.WindowMove || {};
 Esrt.WindowMove.StageClear = Esrt.WindowMove.StageClear || {};
 Esrt.WindowMove.StageClear.isMoveIn = true;
 Esrt.WindowMove.StageClear.windowMoveDuration = 60;
 Esrt.WindowMove.StageClear.windowMoveCount = 0;
 Esrt.WindowMove.StageClear.windowMoveType = 0;
 Esrt.WindowMove.StageClear.windowMoveRate = 0;
 Esrt.WindowMove.StageClearRecord = Esrt.WindowMove.StageClearRecord || {};
 Esrt.WindowMove.StageClearRecord.isMoveIn = true;
 Esrt.WindowMove.StageClearRecord.windowMoveDuration = 60;
 Esrt.WindowMove.StageClearRecord.windowMoveCount = 0;
 Esrt.WindowMove.StageClearRecord.windowMoveType = 1;
 Esrt.WindowMove.StageClearRecord.windowMoveRate = 0;
 
(function() {
    'use strict';
    var pluginName    = 'ESRT_MapStatusWindow';
    var TRANS_COUNT_CYCLE = 1296;

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var stageClearWindowWidth  = 220;
    var stageClearWindowHeight = 500;
    var stageClearWindowY      = 250;
    var stageClearRecordWindowWidth  = 400;
    var stageClearRecordWindowHeight = 200;
    var stageClearRecordWindowY = 300;
    var stillTypeVariable = 91;
    var forceTransSwitch = 102;
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'StageClear'){
            switch(args[0]){
                case 'on':
                    this.setStageClearFlag(1);
                    break;
                case 'off':
                    this.setStageClearFlag(-1);
                    break;
                case 'recordOn':
                    this.setStageClearRecordFlag(1);
                    break;
                case 'recordOff':
                    this.setStageClearRecordFlag(-1);
                    break;
                case 'setStageData':
                    this.setStageData(true);
                    break;
                default:
                    break;
            }
        }
    };
    
    Game_Interpreter.prototype.setStageClearFlag = function(flag) {
        Esrt.StageClear.showFlag = flag;
    };
    
    Game_Interpreter.prototype.setStageClearRecordFlag = function(flag) {
        Esrt.StageClear.recordShowFlag = flag;
    };
    
    Game_Interpreter.prototype.setStageData = function(flag) {
        Esrt.StageClear.setStageDataFlag = flag;
    };
        
    //=============================================================================
    // Scene_Map
    //    ステージクリアウィンドウをマップ上に表示します。
    //=============================================================================
    var _Scene_Menu_addSceneMapUpdateObject = Scene_Map.prototype.addSceneMapUpdateObject;
    Scene_Map.prototype.addSceneMapUpdateObject = function() {
        _Scene_Menu_addSceneMapUpdateObject.apply(this, arguments);
        
        Esrt.WindowMove.SceneMapUpdateObjects.push(Esrt.WindowMove.StageClear);
        Esrt.WindowMove.SceneMapUpdateObjects.push(Esrt.WindowMove.StageClearRecord);
    };
    
    var _Scene_Map_createAllWindows = Scene_Map.prototype.createAllWindows;
    Scene_Map.prototype.createAllWindows = function() {
        this.createStageClearWindow();
        this.createRecordWindow();
        
        _Scene_Map_createAllWindows.call(this);
    };
    
    Scene_Map.prototype.createStageClearWindow = function() {
        this._stageClearWindow = new Window_StageClear(100, 0);
        this.addWindow(this._stageClearWindow);
    };
    
    Scene_Map.prototype.createRecordWindow = function() {
        this._recordWindow = new Window_Record(100, 0);
        this.addWindow(this._recordWindow);
    };
    
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update.call(this);
        
        if (this._time % 12 != 0) {return;}
        
        this.updateStageClearWindow();
        this.updateRecordWindow();
        this.updateStageData();
    };
    
    Scene_Map.prototype.updateStageClearWindow = function() {
        if (!Esrt.StageClear.showFlag) {return;}
        
        if (Esrt.StageClear.showFlag == 1) {
            Esrt.StageClear.showFlag = 0;
            this._stageClearWindow.showWindow();
            
        } else if (Esrt.StageClear.showFlag == -1) {
            Esrt.StageClear.showFlag = 0;
            this._stageClearWindow.hideWindow();
            
        }
    };
    
    Scene_Map.prototype.updateRecordWindow = function() {
        if (!Esrt.StageClear.recordShowFlag) {return;}
        
        if (Esrt.StageClear.recordShowFlag == 1) {
            Esrt.StageClear.recordShowFlag = 0;
            this._recordWindow.showWindow();
            
        } else if (Esrt.StageClear.recordShowFlag == -1) {
            Esrt.StageClear.recordShowFlag = 0;
            this._recordWindow.hideWindow();
        }
    };
    
    Scene_Map.prototype.updateStageData = function() {
        if (!Esrt.StageClear.setStageDataFlag) {return;}
        
        Esrt.StageClear.setStageDataFlag  = false;
        
        this._recordWindow.setStageData();
    };
    
    //=============================================================================
    // Sprite_StageClear
    //    立ち絵スプライトを定義します。
    //=============================================================================
    function Sprite_StageClear()
    {
        this.initialize.apply(this, arguments);
    }

    Sprite_StageClear.prototype = Object.create(Sprite_Base.prototype);
    Sprite_StageClear.prototype.constructor = Sprite_StageClear;

    Sprite_StageClear.prototype.initialize = function()
    {
        Sprite_Base.prototype.initialize.call(this);
        
     	this.scale.x = 0;
     	this.scale.y = this.scale.x;
        this.anchor.x = 0.5;
        this.anchor.y = 0.5;
        this._baseOpacity = 255;
        
        this.createStageClearEffect();
        this.createStageClearSprite();
        
    };
    
    Sprite_StageClear.prototype.createStageClearEffect = function()
    {
        this.effectSprite = new Sprite_StageClearEffect();
        this.effectSprite.anchor.x = 0.5;
        this.effectSprite.anchor.y = 0.5;
        this.addChild(this.effectSprite);
    };
    
    Sprite_StageClear.prototype.createStageClearSprite = function()
    {
        this.stageClearSprite = new Sprite(ImageManager.loadPicture("StageClear_0001"));
        this.addChild(this.stageClearSprite);
        this.stageClearSprite.y = -45;
        this.stageClearSprite.anchor.x = 0.5;
        this.stageClearSprite.anchor.y = 0.5;
    };
    
    Sprite_StageClear.prototype.reloadStageClearSprite = function()
    {
        var picture_number = ("0000" + ($gameVariables.value(105) % 2 + 1)).substr(-4);
        picture_number = ($gameVariables.value(161) == 71 || $gameVariables.value(161) == 72) ? "0003" : picture_number;
        picture_number = ($gameVariables.value(161) == 73) ? "0003" : picture_number;
        
        this.stageClearSprite.bitmap = ImageManager.loadPicture("StageClear_" + picture_number);
    };
    
    Sprite_StageClear.prototype.showSprite = function()
    {
        this.reloadStageClearSprite();
        this.effectSprite.showSprite();
    };
    
    //=============================================================================
    // Sprite_StageClearEffect
    //    立ち絵スプライト背景の演出を定義します。
    //=============================================================================
    function Sprite_StageClearEffect()
    {
        this.initialize.apply(this, arguments);
    }

    Sprite_StageClearEffect.prototype = Object.create(Sprite_Base.prototype);
    Sprite_StageClearEffect.prototype.constructor = Sprite_StageClearEffect;

    Sprite_StageClearEffect.prototype.initialize = function()
    {
        Sprite_Base.prototype.initialize.call(this);
        
        this.scale.x = 2;
        this.scale.y = 0.8;
        
        this.effectSprite1 = new Sprite(ImageManager.loadPicture("StageClear_radiation"));
        this.effectSprite1.anchor.x = 0.5;
        this.effectSprite1.anchor.y = 0.5;
        this.addChild(this.effectSprite1);
        this.effectSprite2 = new Sprite(ImageManager.loadPicture("StageClear_radiation"));
        this.effectSprite2.anchor.x = 0.5;
        this.effectSprite2.anchor.y = 0.5;
        this.effectSprite2.scale.x = 0.8;
        this.effectSprite2.scale.y = 0.8;
        this.addChild(this.effectSprite2);
        
        this._baseOpacity = 255;
        this._isShow = false;
        this._count = 0;
    };
    
    Sprite_StageClearEffect.prototype.showSprite = function()
    {
        this._isShow = true;
    };
    
    var _Sprite_StageClearEffect_update = Sprite_StageClearEffect.prototype.update;
    Sprite_StageClearEffect.prototype.update = function()
    {
        _Sprite_StageClearEffect_update.apply(this, arguments);
        
        if (!this._isShow) {return;}
        
        this._count += 1;
        
        if (this._count % 3 != 0) {return;}
        
        this.effectSprite1.rotation += 0.05;
        this.effectSprite2.rotation -= 0.02;
        
    };
    
    //=============================================================================
    // Window_StageClear
    //    ステージクリアウィンドウを定義します。
    //=============================================================================
    function Window_StageClear() {
        this.initialize.apply(this, arguments);
    }
    Window_StageClear.prototype = Object.create(Window_Base.prototype);
    Window_StageClear.prototype.constructor = Window_StageClear;
    Window_StageClear.prototype.initialize = function() {
        var width = stageClearWindowWidth;
        var height = stageClearWindowHeight;
        var x = (Graphics.boxWidth - width) / 2;
        var y = stageClearWindowY;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        this.hide();
        this._still = new Sprite_StageClear();
        this.addChild(this._still);
        
        this._count           = 0;
        this._showCount       = -1;
        this._hideCount       = -1;
        this._scaleUpCount    = -1;
        
        this._windowMoveInOut = false;
        this._windowMoveObject = Esrt.WindowMove.StageClear;
        this.opacity = 0;
    };
    
    Window_StageClear.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_Other');
    };
    
    var _Window_StageClear_update = Window_StageClear.prototype.update;
    Window_StageClear.prototype.update = function() {
        _Window_StageClear_update.call(this);
        
        this.updateShow();
        this.updateScaleUp();
        
        this._count++;
        if (!this.drawCheck()) {return;}
        
        this.updateHide();
        //this.drawStageClear(0,0,144,144);
        //this.drawEffect(0,0,144,144);
    }
    
    Window_StageClear.prototype.updateShow = function() {
        if (this._showCount > 0) {
            this._showCount -= 1;
        }
        
        if (this._showCount == 0) {
            this._showCount = -1;
            this.show();
            this._still.showSprite();
        } 
    }
    
    Window_StageClear.prototype.updateScaleUp = function() {
        if (this._scaleUpCount > 0) {
            if(typeof jQueryEasing === "undefined") return;
            var duration = 60;
            var baseScale    = (this._scaleUpCount > 0) ? jQueryEasing.easeInBounce(this._scaleUpCount, 1, -1, duration) : 1;

            this._still.scale.x = (Math.sin(this._scaleUpCount / 200) / 10 + baseScale) * 1.2;
            this._still.scale.y = this._still.scale.x;
            
            this._scaleUpCount -= 1;
        }
    }
    
    Window_StageClear.prototype.updateHide = function() {
        if (this._hideCount > 0) {
            this._hideCount -= 1;
        }
        
        if (this._hideCount == 0) {
            this._hideCount = -1;
            this.hide();
        } 
    }
    
    Window_StageClear.prototype.drawCheck = function() {
        if (!$dataSystem) {return false;}
        if (this._count % 3 != 0) {return false;}
        
        return this.visible;
        
    };
    
    Window_StageClear.prototype.showWindow = function() {
        Esrt.WindowMove.StageClear.windowMoveType = 0;
        this._showCount = 5;
        this._scaleUpCount = 60;
    };
    
    Window_StageClear.prototype.hideWindow = function() {
        Esrt.WindowMove.StageClear.windowMoveType = 0;
        this.hide();
    };
    
    //=============================================================================
    // Window_Record
    //    レーススコアウィンドウを定義します。
    //=============================================================================
    function Window_Record() {
        this.initialize.apply(this, arguments);
    }
    Window_Record.prototype = Object.create(Window_Base.prototype);
    Window_Record.prototype.constructor = Window_Record;
    Window_Record.prototype.initialize = function() {
        var width = stageClearRecordWindowWidth;
        var height = stageClearRecordWindowHeight;
        var x = 200;
        var y = stageClearRecordWindowY;
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        
        this.hide();
        
        this._count           = 0;
        this._hideCount       = -1;
        
        this._record          = 0
        this._goalTime        = 0;
        
        this._maxCoin = 0;
        this._allCoinFastestTime = 99999999;
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.StageClearRecord;
        
        this.opacity = 0;
    };
    
    Window_Record.prototype.loadWindowskin = function() {
        this.windowskin = ImageManager.loadSystem('Window_Other');
    };
    
    var _Window_StageClear_update = Window_Record.prototype.update;
    Window_Record.prototype.update = function() {
        _Window_StageClear_update.call(this);
        
        this.updateShow();
        
        this._count++;
        if (!this.drawCheck()) {return;}
        
        this.updateHide();
    }
    
    Window_Record.prototype.updateShow = function() {
        if (this._showCount > 0) {
            this._showCount -= 1;
        }
        
        if (this._showCount == 0) {
            this._showCount = -1;
            this.show();
        } 
    }
    
    Window_Record.prototype.updateHide = function() {
        if (this._hideCount > 0) {
            this._hideCount -= 1;
        }
        
        if (this._hideCount == 0) {
            this._hideCount = -1;
            this.hide();
        } 
    }
    
    Window_Record.prototype.drawCheck = function() {
        
        if (!$dataSystem) {return false;}
        if (this._count % 3 != 0) {return false;}
        
        return this.visible;
        
    };
    
    Window_Record.prototype.showWindow = function() {
        Esrt.WindowMove.StageClearRecord.windowMoveCount = 0;
        Esrt.WindowMove.StageClearRecord.windowMoveType = 1;
        this._showCount = 5;
        this.drawRecord();
    };
    
    Window_Record.prototype.hideWindow = function() {
        Esrt.WindowMove.StageClearRecord.windowMoveType = 0;
        this.hide();
    };
    
    Window_Record.prototype.setStageData = function() {
        var stageNum = $gameVariables.value(161);
        var gameMode = $gameVariables.value(171);
        var stageScores = Esrt.stageScore[stageNum][gameMode];

        this._maxCoin  = ($dataStage.maxCoin[stageNum]) ? $dataStage.maxCoin[stageNum][Esrt.gameModeName[gameMode]] : 0;
        this._allCoinFastestTime = stageScores[Esrt.stageDatas.allCoinFastestTime];
    };
    
    Window_Record.prototype.drawRecord = function(record) {
        this._record = $gameVariables.value(122);
        var coin = $gameVariables.value(128);
        
        if (this._maxCoin > 0) {
            if (coin >= this._maxCoin) {
                this.drawIcon(87, 100, 0);
            }
            this.drawText("Stone：", 5, 0);
    		this.drawText(coin + "／" + this._maxCoin, 140, 0);
        }
        
        if (coin >= this._maxCoin && this._record <= this._allCoinFastestTime) {
            this.drawIcon(87, 100, 30);
        }
        var times = GetTime(this._record, 5184000);
        this.drawText("Time：", 5, 30);
        this.drawText(times.hour + ":" + times.min + ":" + times.sec + "." + times.msec, 140, 30);
        
        if (coin >= this._maxCoin) {
            if (this._record > this._allCoinFastestTime) {
                times = GetTime(this._allCoinFastestTime, 5184000);
            }
            this.drawText("Best：", 5, 60);
            this.drawText(times.hour + ":" + times.min + ":" + times.sec + "." + times.msec, 140, 60);
        }
    };
    
    // 当該ステージの全魔法石回収最速タイムを表示
    Window_Record.prototype.drawFastestStageTime = function(highCoin, maxCoin, stageScores) {
        if (highCoin >= maxCoin) {
            this.drawIcon(220, 1, this.drawY);
            
    		this.changeTextColor(this.textColor(5));
    		this.drawText("Record",40, this.drawY);
    		this.resetTextColor();
            
            this.drawY += 40;
            var time = stageScores[Esrt.stageDatas.allCoinFastestTime];
            var times = GetTime(time, 5184000);
    		this.drawText(times.hour + ":" + times.min + ":" + times.sec + "." + times.msec, 0, this.drawY, scoreWindowWidth - 50 , 'right');
        }
    }

    function GetTime(time, maxTime) {
        var times = {"day":"--","hour":"--","min":"--","sec":"--","msec":"--"};
        if (maxTime !=0 && time < maxTime) {
            times.msec = time % 60;
            
            time = parseInt(time / 60);
            times.sec = time % 60;
            
            time = parseInt(time / 60);
            times.min = time % 60;
            
            time = parseInt(time / 60);
            times.hour = time;

        }
        return times;
    }
})();
