//=============================================================================
// ESRT_StageScore.js
// ----------------------------------------------------------------------------
// Copyright (c) 2016 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc Save and control stage score
 * @author white_esorat
 * 
 * @help 
 * プラグインコマンド詳細
 *  イベントコマンド「プラグインコマンド」から実行。
 *  （パラメータの間は半角スペースで区切る）
 *　代入値、代入先変数にv[1]と入力すると、変数1の値を使用する
 *
 * setValue     [ステージ番号]       [ゲームモード] [代入項目名][代入値]
 *
 * addValue     [ステージ番号]       [ゲームモード] [代入項目名][代入値]
 *
 * getValue     [ステージ番号]       [ゲームモード] [代入項目名][代入先変数]
 * getValueVS   [ステージ番号(変数)] [ゲームモード] [代入項目名][代入先変数]
 *
 */

/*:ja
 * @plugindesc スコアの記録・管理を行う
 * @author 白
 * 
 * @help 
 * プラグインコマンド詳細
 *  イベントコマンド「プラグインコマンド」から実行。
 *  （パラメータの間は半角スペースで区切る）
 *　代入値、代入先変数にv[1]と入力すると、変数1の値を使用する
 *
 * setValue     [ステージ番号]       [ゲームモード] [代入項目名][代入値]
 *
 * addValue     [ステージ番号]       [ゲームモード] [代入項目名][代入値]
 *
 * getValue     [ステージ番号]       [ゲームモード] [代入項目名][代入先変数]
 *
 */
 
var Esrt = Esrt || {};
Esrt.stageScore   = Esrt.stageScore || {};
Esrt.stageScoreFanc   = Esrt.stageScoreFanc || {};
Esrt.maxStageID   = 999;

Esrt.gameModes = {"normal":0,"extra":1};
Esrt.gameModeName = ["normal", "extra"];
Esrt.stageDatas = {"highScore":0,"longestTime":1,"fastestTime":2,"allCoinFastestTime":3,"totalTime":4,"highCoin":5,
"totalCoin":6,"highSearch":7,"totalSearch":8,"highDiscovery":9,"totalDiscovery":10,"isOpen":11,"clearCount":12,"highDefeat":13};

(function() {
    'use strict';
    var pluginName    = 'ESRT_StageScore';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================

    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'StageScore'){
            switch(args[0]){
                case 'setValue':
                    SetValue(args);
                    break;                    
                case 'setValueMax':
                    SetValueMax(args);
                    break;                    
                case 'setValueMin':
                    SetValueMin(args);
                    break;                    
                case 'addValue':
                    AddValue(args);
                    break;                    
                case 'getValue':
                    GetValue(args);
                    break;                    
                case 'getTotal':
                    GetTotal(args);
                    break;
                case 'initialize':
                    Initialize();
                    break;          
                default:
                    break;
            }
        }
    };
    
    // スコアの記録
    function SetValue(args) {
        var regVar = /v\[(\d+)\]/;
        var stageNum = args[1];
        if (regVar.test(stageNum)) {
            stageNum = Number(stageNum.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        if(typeof Esrt.stageScore[stageNum] === "undefined"){
            createStageData(stageNum);
        }
        
        var gameMode = GetGameMode(args[2]);
        
        var value = args[4];
        if (regVar.test(value)) {
            value = Number(value.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        if(args[3].match('Time')){
            value = (value > 99999999) ? 99999999 : value;
        }else{
            value = (value > 999999) ? 999999 : value;
        }
        Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas[ args[3] ]] = value;
    }
    
    // 最高スコアの記録
    function SetValueMax(args) {
        var regVar = /v\[(\d+)\]/;
        var stageNum = args[1];
        if (regVar.test(stageNum)) {
            stageNum = Number(stageNum.replace(regVar, $gameVariables.value(RegExp.$1)))
        }
        
        if(typeof Esrt.stageScore[stageNum] === "undefined"){
            createStageData(stageNum);
        }
        
        var gameMode = GetGameMode(args[2]);
        
        var value = args[4];
        if (regVar.test(value)) {
            value = Number(value.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        if(value > Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas[ args[3] ]]){
            SetValue(args);
        }
    }
    
    // 最短スコアの記録
    function SetValueMin(args) {
        var regVar = /v\[(\d+)\]/;
        var stageNum = args[1];
        if (regVar.test(stageNum)) {
            stageNum = Number(stageNum.replace(regVar, $gameVariables.value(RegExp.$1)))
        }
        
        if(typeof Esrt.stageScore[stageNum] === "undefined"){
            createStageData(stageNum);
        }
        
        var gameMode = GetGameMode(args[2]);
        
        var value = args[4];
        if (regVar.test(value)) {
            value = Number(value.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        if(value < Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas[ args[3] ]]){
            SetValue(args);
        }
    }
    
    // スコアの加算
    function AddValue(args) {
        var regVar = /v\[(\d+)\]/;
        var stageNum = args[1];
        if (regVar.test(stageNum)) {
            stageNum = Number(stageNum.replace(regVar, $gameVariables.value(RegExp.$1)))
        }
        
        if(typeof Esrt.stageScore[stageNum] === "undefined"){
            createStageData(stageNum);
        }
        
        var gameMode = GetGameMode(args[2]);
        
        var value = args[4];
        if (regVar.test(value)) {
            value = Number(value.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        var value = Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas[ args[3] ]] + value;
        
        if(args[3].match('Time')){
            value = (value > 99999999) ? 99999999 : value;
        }else{
            value = (value > 9999) ? 9999 : value;
        }
        Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas[ args[3] ]] = value;
    }
    
    // スコアの取得
    function GetValue(args) {
        var regVar = /v\[(\d+)\]/;
        var stageNum = args[1];
        if (regVar.test(stageNum)) {
            stageNum = Number(stageNum.replace(regVar, $gameVariables.value(RegExp.$1)))
        }
        
        if(typeof Esrt.stageScore[stageNum] === "undefined"){
            createStageData(stageNum);
        }
        
        var gameMode = GetGameMode(args[2]);
        
        $gameVariables.setValue(args[4], Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas[ args[3] ]]);
    }
    Esrt.stageScoreFanc.GetValue = GetValue;
        
    // ステージ記録オブジェクトの初期化
    function createStageData(stageNum) {
        Esrt.stageScore[stageNum] = [];
        Esrt.stageScore[stageNum][0] = new Array ( Object.keys(Esrt.stageDatas).length );
        Esrt.stageScore[stageNum][1] = new Array ( Object.keys(Esrt.stageDatas).length );
        Esrt.stageScore[stageNum][2] = new Array ( Object.keys(Esrt.stageDatas).length );
        
        for(var gameMode = 0;gameMode < Object.keys(Esrt.gameModes).length; gameMode++) {
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highScore] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.longestTime] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.fastestTime] = 99999999;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.allCoinFastestTime] = 99999999;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.totalTime] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highCoin] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.totalCoin] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highSearch] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.totalSearch] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highDiscovery] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.totalDiscovery] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.isOpen] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.clearCount] = 0;
            Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highDefeat] = 0;
        }
    }
    
    // 完全クリアの判定
    function CheckAllGet() {
        var gameMode    = $gameVariables.value(171);
        for(var stageNum=1; stageNum<=Esrt.maxStageID; stageNum++){
            if(typeof Esrt.stageScore[stageNum] === "undefined"){
                continue;
            }
            
            var value = Esrt.stageScore[stageNum][Esrt.gameModes.normal][Esrt.stageDatas.highCoin];
            var max = ($dataStage.maxCoin[stageNum]) ? $dataStage.maxCoin[stageNum][Esrt.gameModeName[gameMode]] : 0;
            
            if(value < max){
                return false;
            }
        }
        return true;
    }
    Esrt.stageScoreFanc.CheckAllGet = CheckAllGet;

    // ステージ7開放の判定
    function CheckOpenArea7() {
        var exp = $gameVariables.value(246) + $gameVariables.value(247) + $gameVariables.value(248);
        
        return (CheckOpenArea7Exp() && CheckOpenArea7Coin());
    }
    Esrt.stageScoreFanc.CheckOpenArea7 = CheckOpenArea7;
    
    // ステージ7開放の判定
    function CheckOpenArea7Exp() {
        var exp = $gameVariables.value(246) + $gameVariables.value(247) + $gameVariables.value(248);
        
        return (exp > 0);
    }
    Esrt.stageScoreFanc.CheckOpenArea7Exp = CheckOpenArea7Exp;
    
    // ステージ7開放の判定
    function CheckOpenArea7Coin() {
        var gameMode = $gameVariables.value(171);
        var value = 0;
        var max = ($dataStage.openArea7) ? $dataStage.openArea7[Esrt.gameModeName[gameMode]] : 0;
        
        for(var stageNum=1; stageNum<=Esrt.maxStageID; stageNum++){
            if(typeof Esrt.stageScore[stageNum] != "undefined"){
                value += Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highCoin];
            }
        }
        
        return (value >= max);
    }
    Esrt.stageScoreFanc.CheckOpenArea7Coin = CheckOpenArea7Coin;
    
    // 完全クリアの判定
    function GetAllStageHighCoin(gameMode) {
        var value = 0;
        var max = 0;
        for(var stageNum=1; stageNum<=Esrt.maxStageID; stageNum++){
            if(typeof Esrt.stageScore[stageNum] != "undefined"){
                value += Esrt.stageScore[stageNum][gameMode][Esrt.stageDatas.highCoin];
            }
            
            if(typeof $dataStage.maxCoin[stageNum] != "undefined" && typeof $dataStage.maxCoin[stageNum][Esrt.gameModeName[gameMode]] != "undefined"){
                max += $dataStage.maxCoin[stageNum][Esrt.gameModeName[gameMode]];
            }
        }
        return [value,max];
    }
    Esrt.stageScoreFanc.GetAllStageHighCoin = GetAllStageHighCoin;
    
    // ステージスコアの初期化
    function Initialize() {
        Esrt.stageScore   = {};
    }
    
    // ゲームモード番号を取得する。文字であれば、対応する番号に変換する
    function GetGameMode(text){
        var regVar = /v\[(\d+)\]/;
        
        var gameMode = text;
        
        if (regVar.test(gameMode)) {
            gameMode = Number(gameMode.replace(regVar, $gameVariables.value(RegExp.$1)));
        }
        
        if (isNaN(gameMode)) { // 引数が数字でないとき、数字に変換
            gameMode = Esrt.gameModes[gameMode];
        }
        return gameMode;
    }
    //=============================================================================
    // DataManager
    //  スコアデータをセーブに含めます
    //=============================================================================
    var _DataManager_makeSaveContents = DataManager.makeSaveContents;
        DataManager.makeSaveContents = function() {
        var contents = _DataManager_makeSaveContents.call(this);
        contents.stageScore = Esrt.stageScore;
        return contents;
    };
    var _DataManager_extractSaveContents = DataManager.extractSaveContents;
        DataManager.extractSaveContents = function(contents) {
        _DataManager_extractSaveContents.call(this, contents);
        Esrt.stageScore = contents.stageScore;
    };
    
})();
