//=============================================================================
// ESRT_StillPictureControll.js
// ----------------------------------------------------------------------------
// Copyright (c) 2017 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:ja
 * @plugindesc 立ち絵表示コマンド一括プラグイン
 * @author 白
 * 
 * @help 
 * 立ち絵の入り抜き・注目動作をプラグインコマンドで簡略化します。
 *
 *
 */
var Esrt = Esrt || {};
Esrt.still   = Esrt.still || {};

(function() {
    'use strict';
	var pluginName    = 'ESRT_StillPictureControll';
    
    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    
    
    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'Still'){
            switch(args[0]){
                case 'setPicture':
                    StillSetPicture(args);
                    break;
                case 'setIndex':
                    StillSet(args);
                    break;
                case 'attention':
                    StillAttention(args);
                    break;
                case 'changeOption':
                    ChangeStillOptions(args);
                    break;
                case 'clear':
                    StillClear(args);
                    break;
                case 'hidden':
                    StillHidden(args);
                    break;
                case 'allHidden':
                    StillAllHidden(args);
                    break;
                case 'allClear':
                    StillAllClear(args);
                    break;
                case 'preload':
                    StillPreload(args);
                    break;
                default:
                    break;
            }
        }
    };
    
        
    // 立ち絵ファイル名のセット
    function StillSetPicture(args) {
        var regVar = /v\[(\d+)\]/;
        var character  = args[1];
        var pictureId = Number(args[2]);
        var filename  = args[3];
        var options   = GetStillOptions(args, 4);
        
        var position  = (typeof options.position != 'undefined') ? options.position : 'left';
        var xScale    = (options.xReverse == true) ? -100 : 100;
        var x         = 0;
        var y         = 0;
        
        x  =  (typeof options.originX != 'undefined') ? options.originX : x;
        x  += (options.xReverse == true) ? 432 : 0;
        x  =  (typeof options.setX != 'undefined') ? options.setX : x;
        x  += (typeof options.addX != 'undefined') ? options.addX : 0;
        y  += (typeof options.addY != 'undefined') ? options.addY : 0;
        
        Esrt.still[character] = {'character' : character,
                                 'pictureId' : pictureId,
                                 'filename': filename, 
                                 'position': position,
                                 'index'   : 1,
                                 'originX' : x,
                                 'originY' : y,
                                 'x'       : x,
                                 'y'       : y,
                                 'xScale'  : xScale,
                                 'yScale'  : 100,
                                 'tone'    : [-68, -68, -68, 0],
                                 'opacity' : 0
                             };
        $gameScreen.showPicture(pictureId, filename + '_' + '0001', 0,
                                x, y, xScale, 100,
                                0, 0);
        
    }
    // 立ち絵ファイル名のセット
    function StillAttention(args) {
        var regVar = /v\[(\d+)\]/;
        var character = args[1];
        var options   = GetStillOptions(args, 2);
        
        Object.keys(Esrt.still).forEach(function(key) {
            var still = Esrt.still[key];
            if(still.character === character){
                if(options.index && still.index != options.index){
                    $gameScreen.showPicture(still.pictureId, still.filename + '_' + ('0000' + options.index).slice( -4 ), 0,
                                           still.x, still.y, still.xScale, 100,
                                           still.opacity, 0);
                    still.index = options.index;
                }
                
                ChangeStillOptions(args);
                
                $gameScreen.movePicture(still.pictureId, 0,
                                        still.x, still.y, still.xScale, 100,
                                        still.opacity, 0, still.duration);
            }else{
                // その他立ち絵を画面外に寄せて暗くする
                var offset = 0;                
                if(still.position === 'left')        offset = -50;
                else if(still.position === 'right')  offset = 50;
                else if(still.position === 'center') offset = 0;                
                still.x = still.originX + offset
                
                var otherOpacity = 128;
                still.opacity = (still.opacity > otherOpacity) ? otherOpacity : still.opacity;
                
                $gameScreen.movePicture(still.pictureId, 0, 
                                        still.x, still.y, still.xScale, 100,
                                        still.opacity, 0, 10);
            }
        });
        
    }
    // 立ち絵オプションの変更
    function ChangeStillOptions(args) {
        var regVar = /v\[(\d+)\]/;
        var character = args[1];
        var options   = GetStillOptions(args, 2);
        var still = Esrt.still[character];
        
        still.position  = (typeof options.position != 'undefined') ? options.position : still.position;
        if(options.xReverse == true){
            still.xScale    = still.xScale * -1;
            still.originX  += (still.xScale < 0) ? 432 : -432;
        }
        
        still.originX  =  (typeof options.originX != 'undefined') ? options.originX : still.originX;
        still.originX  += (typeof options.addX != 'undefined') ? options.addX : 0;
        still.originX  =  (typeof options.setX != 'undefined') ? options.setX : still.originX;
        
        var offset = 0;
        if(still.position === 'left')        offset = 50;
        else if(still.position === 'right')  offset = -50;
        else if(still.position === 'center') offset = 0;
        still.x = still.originX + offset;
        still.y += (typeof options.addY != 'undefined') ? options.addY : 0;
                
        still.opacity = (typeof options.opacity != 'undefined') ? options.opacity : 255;
        still.duration  = (typeof options.duration != 'undefined') ? options.duration : 10;
    }
    
    // 立ち絵オプション指定の取得
    function GetStillOptions(args, start) {
        var options = {};
        for(var i=start;i<20;i++){
            var argText    = args[i];
            if(typeof argText === 'undefined'){ break;}
            if(/opac/.test(argText)){
                argText = argText.replace(/[^\-\d]/g, '');
                options.opacity = Number.parseInt(argText);
                
            }else if(/dura/.test(argText)){
                argText = argText.replace(/[^\-\d]/g, '');
                options.duration = Number.parseInt(argText);
                
            }else if(/addX/.test(argText)){
                argText = argText.replace(/[^\-\d]/g, '');
                if(argText == ''){ continue;}
                options.addX = Number.parseInt(argText);  
                if(options.addX !== options.addX){ options.addX = undefined}
                
            }else if(/setX/.test(argText)){
                argText = argText.replace(/[^\-\d]/g, '');
                if(argText == ''){ continue;}
                options.setX = Number.parseInt(argText);  
                if(options.setX !== options.setX){ options.setX = undefined}
                
            }else if(/addY/.test(argText)){
                argText = argText.replace(/[^\-\d]/g, '');
                if(argText == ''){ continue;}
                options.addY = Number.parseInt(argText);  
                if(options.addY !== options.addY){ options.addY = undefined}
                
            }else if(/left/.test(argText)) {
                options.originX = 0;
                options.position = 'left';
                
            } else if(/right/.test(argText)) {
                options.originX = 400;
                options.position = 'right';
                
            } else if(/center/.test(argText)) {
                options.originX = 200;
                options.position = 'center';
                
            }else if(/xRev/.test(argText)){
                options.xReverse = true;
                
            }else{
                options.index = Number.parseInt(argText);
            }
        }
        return options;
    }
    
    // 立ち絵ファイル情報と当該ピクチャの削除
    function StillClear(args) {
        var character = args[1];
        $gameScreen.erasePicture(Esrt.still[character].pictureId);
        delete Esrt.still[character];
    }
    
    // 立ち絵ピクチャの一時透過
    function StillHidden(args) {
        var still = Esrt.still[args[1]];
        
        var offset = 0;
        if(still.position === 'left')        offset = -100;
        else if(still.position === 'right')  offset = 100;
        else if(still.position === 'center') offset = 0;
        
        still.x = still.originX + offset;
        still.opacity = 0;
        $gameScreen.movePicture(still.pictureId, 0, 
                                still.x, still.y, 100, 100,
                                still.opacity, 0, 10);
    }
    
    // 立ち絵ピクチャを全て一時透過
    function StillAllHidden() {
        Object.keys(Esrt.still).forEach(function(key) {
            StillHidden([null,key]);
        });
    }
    
    // 立ち絵ファイル情報と立ち絵ピクチャの全削除
    function StillAllClear() {
        Object.keys(Esrt.still).forEach(function(key) {
            StillClear([null,key]);
        });
        Esrt.still = {};
    }
    
    // 立ち絵ファイル名のセット
    function StillPreload(args) {
        var filename = args[1];
        var indexArray = eval(args[2]);
        indexArray.forEach(function(index) {
            ImageManager.requestPicture(filename + '_' + ( '0000' + index ).slice( -4 ));
        });
    }
    
    
})();
