//=============================================================================
// ESRT_WindowMoveInOut.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc シーン遷移時にウィンドウを画面外から表示・退避させる演出を追加するプラグイン
 * @author 白
 *
 * @param マップ遷移後初回のみ画面外から移動
 * @desc マップ遷移後初回のみウィンドウが画面外から移動するようにします。
 * @default false
 * @type boolean
 *
 * @param マップ遷移後初回のみ画面外から移動
 * @desc マップ遷移後初回のみウィンドウが画面外から移動するようにします。
 * @default false
 *
 * @param 演出時間
 * @desc ウィンドウが移動する演出時間（フレーム）
 * @default 10
 *
 * @help
 *
 *
 */

var Esrt = Esrt || {};
Esrt.WindowMove = Esrt.WindowMove || {};
Esrt.WindowMove.SceneMapUpdateObjects = [];

//=============================================================================
//    汎用関数の定義
//=============================================================================
(function() {
    'use strict';
	var pluginName    = 'ESRT_WindowMoveInOut';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var isMoveOnce = parameters['マップ遷移後初回のみ画面外から移動'];
    var durations = {};
    durations.Common = parseInt(parameters['演出時間']);
    
    //=============================================================================
    // 汎用変数・シーン別変数の初期化
    //=============================================================================
    {
        var objects = ["Common","Menu","File"];
        
        objects.forEach(function(key) {
            var object = {};
            
            object.isMoveIn = true;
            object.windowMoveDuration = durations["Common"] ? durations["Common"] : 10;
            object.windowMoveCount = 0;
            object.windowMoveIntervalCount = 0;
            object.windowMoveType = 0;
            object.windowMoveRate = 0;
            
            Esrt.WindowMove[key] = object;
        }, this);
        
    }
    
    //=============================================================================
    // Scene_Base
    //    ウィンドウ移動情報を計算します
    //=============================================================================
    var _Scene_Base_initialize = Scene_Base.prototype.initialize;
    Scene_Base.prototype.initialize = function() {
        _Scene_Base_initialize.apply(this, arguments);
        
        this._windowMoveInOut = this._windowMoveInOut || false;
        this._windowMoveObject = this._windowMoveObject || Esrt.WindowMove.Common;
        
        this._windowMoveObject.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : this._windowMoveObject.isMoveIn;
        this._windowMoveObject.windowMoveCount = (this._windowMoveObject.isMoveIn) ? 0 : this._windowMoveObject.moveDuration;
        
        this._windowInOutCount = 0;
    };
    
    var _Scene_Base_update = Scene_Base.prototype.update;
    Scene_Base.prototype.update = function() {
        _Scene_Base_update.call(this);
        
        if (this._windowMoveInOut) {
            this._windowMoveObject.windowMoveIntervalCount += 1;
            this._windowInOutCount += 1;
            this.updateWindowPosition();
        }
    };
    
    // ウィンドウの位置移動
    Scene_Base.prototype.updateWindowPosition = function() {
        if (this._windowInOutCount % 2 != 0){
            return false;
        }
        
        if (this._windowMoveObject.windowMoveCount < this._windowMoveObject.windowMoveDuration) {
            this._windowMoveObject.windowMoveRate = this.getWindowMoveRate(this._windowMoveObject);
            this._windowMoveObject.windowMoveCount += 1;
        }
        if (this._windowMoveObject.windowMoveCount == this._windowMoveObject.windowMoveDuration) {
            this.windowMoveEnd();
        }
        
        return;
    };
    
    // ウィンドウの移動終了時の処理
    Scene_Base.prototype.windowMoveEnd = function() {
    }
    
    // ウィンドウの移動割合計算
    Scene_Base.prototype.getWindowMoveRate = function(object) {
        if (object.windowMoveCount >= 0) {
            var easingFunction = (object.windowMoveType == 1) ? jQueryEasing.easeOutQuint : jQueryEasing.easeInQuad;
            return easingFunction(object.windowMoveCount, 0, 1, object.windowMoveDuration);
        } else {
            return 1;
        };
    }
    
    //=============================================================================
    // Scene_Menu
    //    ウィンドウ群にメニューからの遷移フラグを追加します
    //=============================================================================
    var _Scene_Menu_initialize = Scene_Menu.prototype.initialize;
    Scene_Menu.prototype.initialize = function() {
        _Scene_Menu_initialize.apply(this, arguments);
        
        Esrt.WindowMove.SceneMapUpdateObjects.forEach(function(object) {
            if (isMoveOnce) {
                object.isMoveIn = false;
                object.windowMoveCount = (object.isMoveIn) ? object.windowMoveDuration : 0;
                object.windowMoveType = 0;
            }
        }, this);
    };
    
    //=============================================================================
    // Scene_Map
    //    マップ遷移に伴って計算するウィンドウ群の移動情報を計算します
    //=============================================================================
    var _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this, arguments);
        
        this._windowMoveInOut = true;
        
        Esrt.WindowMove.SceneMapUpdateObjects = [];
        
        this.addSceneMapUpdateObject();
        
        Esrt.WindowMove.SceneMapUpdateObjects.forEach(function(object) {
            object.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : object.isMoveIn;
            object.windowMoveCount = (object.isMoveIn) ? 0 : object.windowMoveDuration;
        }, this);
    };
    
    Scene_Map.prototype.addSceneMapUpdateObject = function() {
        Esrt.WindowMove.SceneMapUpdateObjects.push(Esrt.WindowMove.Common);
    };
    
    // ウィンドウの位置移動
    Scene_Map.prototype.updateWindowPosition = function() {
        if (this._windowInOutCount % 2 != 0){
            return false;
        }
        
        Esrt.WindowMove.SceneMapUpdateObjects.forEach(function(object) {
            if (object.windowMoveCount == 0 && !object.windowMoveType == 0) {
                object.windowMoveCount ==  object.windowMoveDuration;
            }
            
            if (object.windowMoveCount < object.windowMoveDuration) {
                object.windowMoveRate = this.getWindowMoveRate(object);
                object.windowMoveCount += 1;
            }
        }, this);
        
        return;
    };
    
    //=============================================================================
    // Window_Base
    //    ウィンドウ移動情報を元に描画位置をずらします
    //=============================================================================
    var _Window_Base_initialize = Window_Base.prototype.initialize;
    Window_Base.prototype.initialize = function(x, y, width, height) {
        this._baseX = x ? x : 0;
        this._baseY = y ? y : 0;
        this._inOutCount = 0;
        this._windowPositionDirection = this._windowPositionDirection || 6; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = this._windowMoveInOut || false;
        this._windowMoveObject = this._windowMoveObject || Esrt.WindowMove.Common;
        
        var setX = x;
        var setY = y;
        
        if (this._windowMoveInOut && this._windowMoveObject.isMoveIn) {
            if (this._windowPositionDirection == 2) {
                setY = y + height * 2;
                
            } else if (this._windowPositionDirection == 4) {
                setX = x - width;
                
            } else if (this._windowPositionDirection == 6) {
                setX = x + width * 2;
                
            } else if (this._windowPositionDirection == 8) {
                setY = y - height;
            }
        }
        
        _Window_Base_initialize.call(this, setX, setY, width, height);
    };
    
    var _Window_Base_update = Window_Base.prototype.update;
    Window_Base.prototype.update = function() {
        _Window_Base_update.call(this);
        
        if (this._windowMoveInOut) {
            this._inOutCount++;
            this.updateWindowPosition();
        }
    }
    
    // ウィンドウの位置移動
    Window_Base.prototype.updateWindowPosition = function() {
        if (this._inOutCount % 2 != 0){
            return false;
        }
        
        if (this._windowMoveObject.windowMoveCount < this._windowMoveObject.windowMoveDuration) {
            this.setWindowPosition();
        }
        
        return;
    };
    
    Window_Base.prototype.setWindowPosition = function() {
        if (this._windowPositionDirection == 2) { // 下端から表示
            this.y = (this._windowMoveObject.windowMoveType == 1) ? this._baseY + (this.height * (1 - this._windowMoveObject.windowMoveRate)) : this._baseY + (this.height * this._windowMoveObject.windowMoveRate);
        } else if (this._windowPositionDirection == 4) { // 左端から表示
            this.x = (this._windowMoveObject.windowMoveType == 1) ? this._baseX + (this.width * (this._windowMoveObject.windowMoveRate - 1)) : this._baseX - (this.width * this._windowMoveObject.windowMoveRate);
        } else if (this._windowPositionDirection == 6) { // 右端から表示
            this.x = (this._windowMoveObject.windowMoveType == 1) ? this._baseX + (this.width * (1 - this._windowMoveObject.windowMoveRate)) : this._baseX + (this.width * this._windowMoveObject.windowMoveRate * 1.5);
        } else if (this._windowPositionDirection == 8) { // 上端から表示
            this.y = (this._windowMoveObject.windowMoveType == 1) ? this._baseY + (this.height * (this._windowMoveObject.windowMoveRate - 1)) : this._baseY - (this.height * this._windowMoveObject.windowMoveRate);
        }
        
        return;
    };
    
    Window_Base.prototype.setWindowPositionDirection = function(direction) {
        this._windowPositionDirection = direction; // ウィンドウ位置　4:左, 6:右
    }
    
    Window_Base.prototype.setWindowMoveInOut = function(flag) {
        this._windowMoveInOut = flag;
    }
    
    Window_Base.prototype.setWindowMoveObject = function(object) {
        this._windowMoveObject = object;
    }

}());

//=============================================================================
//    個別シーン・ウィンドウへの設定適用
//=============================================================================
(function() {
    'use strict';
	var pluginName    = 'ESRT_WindowMoveInOut';
    
    //=============================================================================
    // Scene_Map
    //    ウィンドウ移動情報を初期化します
    //=============================================================================
    var _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this, arguments);
        
        Esrt.WindowMove.Menu.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.stageSelect.isMoveIn;
        Esrt.WindowMove.File.isMoveIn = ($gamePlayer && $gamePlayer.isTransferring()) ? true : Esrt.WindowMove.stageSelect.isMoveIn;
    };
    
    //=============================================================================
    // Scene
    //    各シーンにウィンドウ演出フラグを追加します
    //=============================================================================
    var _Scene_Menu_initialize = Scene_Menu.prototype.initialize;
    Scene_Menu.prototype.initialize = function() {
        _Scene_Menu_initialize.apply(this, arguments);
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Menu;
        Esrt.WindowMove.Menu.windowMoveType = 1;
        Esrt.WindowMove.Menu.windowMoveRate = 0;
        Esrt.WindowMove.Menu.windowMoveCount = (Esrt.WindowMove.Menu.isMoveIn) ? 0 : Esrt.WindowMove.Menu.windowMoveDuration;
    };
    
    Scene_Menu.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.Menu.isMoveIn = false;
        Esrt.WindowMove.Menu.windowMoveCount = (Esrt.WindowMove.Menu.isMoveIn) ? 0 : Esrt.WindowMove.Menu.windowMoveDuration;
        Esrt.WindowMove.Menu.windowMoveType = 0;
    }
    
    var _Scene_File_initialize = Scene_File.prototype.initialize;
    Scene_File.prototype.initialize = function() {
        _Scene_File_initialize.apply(this, arguments);
        
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.File;
        Esrt.WindowMove.File.windowMoveType = 1;
        Esrt.WindowMove.File.windowMoveRate = 0;
        Esrt.WindowMove.File.windowMoveCount = (Esrt.WindowMove.File.isMoveIn) ? 0 : Esrt.WindowMove.File.windowMoveDuration;
    };
    
    Scene_File.prototype.windowMoveEnd = function() {
        Esrt.WindowMove.File.isMoveIn = false;
        Esrt.WindowMove.File.windowMoveCount = (Esrt.WindowMove.File.isMoveIn) ? 0 : Esrt.WindowMove.File.windowMoveDuration;
        Esrt.WindowMove.File.windowMoveType = 0;
    }
    
    var _Scene_File_createHelpWindow = Scene_File.prototype.createHelpWindow;
    Scene_File.prototype.createHelpWindow = function() {
        _Scene_File_createHelpWindow.apply(this, arguments);
        
        this._helpWindow.setWindowPositionDirection(8);
        this._helpWindow.setWindowMoveInOut(true);
        this._helpWindow.setWindowMoveObject(Esrt.WindowMove.File);
    };
    
    //=============================================================================
    // Window
    //    各ウィンドウにウィンドウ演出フラグを追加します
    //=============================================================================
    var _Window_MenuCommand_initialize = Window_MenuCommand.prototype.initialize;
    Window_MenuCommand.prototype.initialize = function(x, y) {
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Menu;
        
        _Window_MenuCommand_initialize.apply(this, arguments);
    };
    
    var _Window_Gold_initialize = Window_Gold.prototype.initialize;
    Window_Gold.prototype.initialize = function(x, y) {
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Menu;
        
        _Window_Gold_initialize.apply(this, arguments);
    };
    
    var _Window_MenuStatus_initialize = Window_MenuStatus.prototype.initialize;
    Window_MenuStatus.prototype.initialize = function(x, y) {
        this._windowPositionDirection = 6; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Menu;
        
        _Window_MenuStatus_initialize.apply(this, arguments);
    };
    
    var _Window_SavefileList_initialize = Window_SavefileList.prototype.initialize;
    Window_SavefileList.prototype.initialize = function(x, y) {
        this._windowPositionDirection = 2; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.File;
        
        _Window_SavefileList_initialize.apply(this, arguments);
    };
}());
